@@ -1,3 +1,17 @@
+// Copyright 2005-2020 Google LLC
+//
+// Licensed under the Apache License, Version 2.0 (the 'License');
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+//
+//     http://www.apache.org/licenses/LICENSE-2.0
+//
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an 'AS IS' BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
+//
 // See www.openfst.org for extensive documentation on this weighted
 // finite-state transducer library.
 //
@@ -8,7 +22,7 @@
 
 #include <algorithm>
 #include <functional>
-#include <unordered_map>
+#include <memory>
 #include <vector>
 
 #include <fst/log.h>
@@ -19,9 +33,12 @@
 #include <fst/expanded-fst.h>
 #include <fst/replace.h>
 
+#include <unordered_map>
+
 namespace fst {
 
 // This class accumulates arc weights using the semiring Plus().
+// Sum(w, aiter, begin, end) has time complexity O(begin - end).
 template <class A>
 class DefaultAccumulator {
  public:
@@ -56,6 +73,7 @@ class DefaultAccumulator {
 
 // This class accumulates arc weights using the log semiring Plus() assuming an
 // arc weight has a WeightConvert specialization to and from log64 weights.
+// Sum(w, aiter, begin, end) has time complexity O(begin - end).
 template <class A>
 class LogAccumulator {
  public:
@@ -99,8 +117,8 @@ class LogAccumulator {
     }
   }
 
-  WeightConvert<Weight, Log64Weight> to_log_weight_;
-  WeightConvert<Log64Weight, Weight> to_weight_;
+  const WeightConvert<Weight, Log64Weight> to_log_weight_{};
+  const WeightConvert<Log64Weight, Weight> to_weight_{};
 
   LogAccumulator &operator=(const LogAccumulator &) = delete;
 };
@@ -194,6 +212,9 @@ class MutableFastLogAccumulatorData : public FastLogAccumulatorData {
 // arc weight has a WeightConvert specialization to and from log64 weights. The
 // member function Init(fst) has to be called to setup pre-computed weight
 // information.
+// Sum(w, aiter, begin, end) has time complexity O(arc_limit_) or O(arc_period_)
+// depending on whether the state has more than arc_limit_ arcs
+// Space complexity is O(CountStates(fst) + CountArcs(fst) / arc_period_).
 template <class A>
 class FastLogAccumulator {
  public:
@@ -220,7 +241,7 @@ class FastLogAccumulator {
         state_weights_(nullptr),
         error_(false) {}
 
-  FastLogAccumulator(const FastLogAccumulator<Arc> &acc, bool safe = false)
+  FastLogAccumulator(const FastLogAccumulator &acc, bool safe = false)
       : to_log_weight_(),
         to_weight_(),
         arc_limit_(acc.arc_limit_),
@@ -370,8 +391,8 @@ class FastLogAccumulator {
     }
   }
 
-  const WeightConvert<Weight, Log64Weight> to_log_weight_;
-  const WeightConvert<Log64Weight, Weight> to_weight_;
+  const WeightConvert<Weight, Log64Weight> to_log_weight_{};
+  const WeightConvert<Log64Weight, Weight> to_weight_{};
   const ssize_t arc_limit_;   // Minimum number of arcs to pre-compute state.
   const ssize_t arc_period_;  // Saves cumulative weights per arc_period_.
   std::shared_ptr<FastLogAccumulatorData> data_;
@@ -411,7 +432,7 @@ class CacheLogAccumulatorData {
 
   void AddWeights(StateId s, std::vector<double> *weights) {
     if (cache_gc_ && cache_size_ >= cache_limit_) GC(false);
-    cache_.insert(std::make_pair(s, CacheState(weights, true)));
+    cache_.emplace(s, CacheState(weights, true));
     if (cache_gc_) cache_size_ += weights->capacity() * sizeof(double);
   }
 
@@ -457,7 +478,7 @@ class CacheLogAccumulatorData {
 // WeightConvert specialization to and from log64 weights. It is similar to the
 // FastLogAccumator. However here, the accumulated weights are pre-computed and
 // stored only for the states that are visited. The member function Init(fst)
-// has to be called to setup this accumulator.
+// has to be called to setup this accumulator. Space complexity is O(gc_limit).
 template <class Arc>
 class CacheLogAccumulator {
  public:
@@ -471,7 +492,7 @@ class CacheLogAccumulator {
         s_(kNoStateId),
         error_(false) {}
 
-  CacheLogAccumulator(const CacheLogAccumulator<Arc> &acc, bool safe = false)
+  CacheLogAccumulator(const CacheLogAccumulator &acc, bool safe = false)
       : arc_limit_(acc.arc_limit_),
         fst_(acc.fst_ ? acc.fst_->Copy() : nullptr),
         data_(safe ? std::make_shared<CacheLogAccumulatorData<Arc>>(*acc.data_)
@@ -553,9 +574,9 @@ class CacheLogAccumulator {
     auto pos = aiter->Position();
     if (weights_) {
       Extend(fst_->NumArcs(s_), aiter);
-      return std::lower_bound(weights_->begin() + pos + 1, weights_->end(),
-                              f, std::greater<double>()) -
-          weights_->begin() - 1;
+      return std::lower_bound(weights_->begin() + pos + 1, weights_->end(), f,
+                              std::greater<double>()) -
+             weights_->begin() - 1;
     } else {
       size_t n = 0;
       auto x = FloatLimits<double>::PosInfinity();
@@ -624,9 +645,8 @@ class CacheLogAccumulator {
     }
   }
 
-
-  WeightConvert<Weight, Log64Weight> to_log_weight_;
-  WeightConvert<Log64Weight, Weight> to_weight_;
+  const WeightConvert<Weight, Log64Weight> to_log_weight_{};
+  const WeightConvert<Log64Weight, Weight> to_weight_{};
   ssize_t arc_limit_;                    // Minimum # of arcs to cache a state.
   std::vector<double> *weights_;         // Accumulated weights for cur. state.
   std::unique_ptr<const Fst<Arc>> fst_;  // Input FST.
@@ -648,13 +668,8 @@ class ReplaceAccumulatorData {
   ReplaceAccumulatorData() : state_table_(nullptr) {}
 
   explicit ReplaceAccumulatorData(
-      const std::vector<Accumulator *> &accumulators)
-      : state_table_(nullptr) {
-    accumulators_.reserve(accumulators.size());
-    for (const auto accumulator : accumulators) {
-      accumulators_.emplace_back(accumulator);
-    }
-  }
+      std::vector<std::unique_ptr<Accumulator>> &&accumulators)
+      : state_table_(nullptr), accumulators_(std::move(accumulators)) {}
 
   void Init(const std::vector<std::pair<Label, const Fst<Arc> *>> &fst_tuples,
             const StateTable *state_table) {
@@ -662,7 +677,7 @@ class ReplaceAccumulatorData {
     accumulators_.resize(fst_tuples.size());
     for (Label i = 0; i < accumulators_.size(); ++i) {
       if (!accumulators_[i]) {
-        accumulators_[i].reset(new Accumulator());
+        accumulators_[i] = std::make_unique<Accumulator>();
         accumulators_[i]->Init(*(fst_tuples[i].second));
       }
       fst_array_.emplace_back(fst_tuples[i].second->Copy());
@@ -681,7 +696,7 @@ class ReplaceAccumulatorData {
   std::vector<std::unique_ptr<const Fst<Arc>>> fst_array_;
 };
 
-// This class accumulates weights in a ReplaceFst.  The 'Init' method takes as
+// This class accumulates weights in a ReplaceFst. The 'Init' method takes as
 // input the argument used to build the ReplaceFst and the ReplaceFst state
 // table. It uses accumulators of type 'Accumulator' in the underlying FSTs.
 template <class Accumulator,
@@ -701,10 +716,11 @@ class ReplaceAccumulator {
               ReplaceAccumulatorData<Accumulator, StateTable>>()),
         error_(false) {}
 
-  explicit ReplaceAccumulator(const std::vector<Accumulator *> &accumulators)
+  explicit ReplaceAccumulator(
+      std::vector<std::unique_ptr<Accumulator>> &&accumulators)
       : init_(false),
         data_(std::make_shared<ReplaceAccumulatorData<Accumulator, StateTable>>(
-            accumulators)),
+            std::move(accumulators))),
         error_(false) {}
 
   ReplaceAccumulator(const ReplaceAccumulator<Accumulator, StateTable> &acc,
@@ -874,7 +890,7 @@ class SafeReplaceAccumulator {
     ArcIteratorPtr(const ArcIteratorPtr &copy) {}
 
     void Set(const Fst<Arc> &fst, StateId state_id) {
-      ptr_.reset(new ArcIterator<Fst<Arc>>(fst, state_id));
+      ptr_ = std::make_unique<ArcIterator<Fst<Arc>>>(fst, state_id);
     }
 
     ArcIterator<Fst<Arc>> *get() { return ptr_.get(); }
