@@ -1,3 +1,17 @@
+// Copyright 2005-2020 Google LLC
+//
+// Licensed under the Apache License, Version 2.0 (the 'License');
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+//
+//     http://www.apache.org/licenses/LICENSE-2.0
+//
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an 'AS IS' BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
+//
 // See www.openfst.org for extensive documentation on this weighted
 // finite-state transducer library.
 //
@@ -6,18 +20,18 @@
 #ifndef FST_CACHE_H_
 #define FST_CACHE_H_
 
+#include <algorithm>
 #include <functional>
-#include <unordered_map>
-using std::unordered_map;
-using std::unordered_multimap;
 #include <list>
 #include <vector>
 
 #include <fst/flags.h>
+#include <fst/types.h>
 #include <fst/log.h>
 
 #include <fst/vector-fst.h>
 
+#include <unordered_map>
 
 DECLARE_bool(fst_default_cache_gc);
 DECLARE_int64(fst_default_cache_gc_limit);
@@ -29,8 +43,9 @@ struct CacheOptions {
   bool gc;          // Enables GC.
   size_t gc_limit;  // Number of bytes allowed before GC.
 
-  explicit CacheOptions(bool gc = FLAGS_fst_default_cache_gc,
-                        size_t gc_limit = FLAGS_fst_default_cache_gc_limit)
+  explicit CacheOptions(
+      bool gc = FST_FLAGS_fst_default_cache_gc,
+      size_t gc_limit = FST_FLAGS_fst_default_cache_gc_limit)
       : gc(gc), gc_limit(gc_limit) {}
 };
 
@@ -43,9 +58,10 @@ struct CacheImplOptions {
   CacheStore *store;  // Cache store.
   bool own_store;     // Should CacheImpl takes ownership of the store?
 
-  explicit CacheImplOptions(bool gc = FLAGS_fst_default_cache_gc,
-                            size_t gc_limit = FLAGS_fst_default_cache_gc_limit,
-                            CacheStore *store = nullptr)
+  explicit CacheImplOptions(
+      bool gc = FST_FLAGS_fst_default_cache_gc,
+      size_t gc_limit = FST_FLAGS_fst_default_cache_gc_limit,
+      CacheStore *store = nullptr)
       : gc(gc), gc_limit(gc_limit), store(store), own_store(true) {}
 
   explicit CacheImplOptions(const CacheOptions &opts)
@@ -53,11 +69,11 @@ struct CacheImplOptions {
 };
 
 // Cache flags.
-constexpr uint32 kCacheFinal = 0x0001;   // Final weight has been cached.
-constexpr uint32 kCacheArcs = 0x0002;    // Arcs have been cached.
-constexpr uint32 kCacheInit = 0x0004;    // Initialized by GC.
-constexpr uint32 kCacheRecent = 0x0008;  // Visited since GC.
-constexpr uint32 kCacheFlags =
+constexpr uint8 kCacheFinal = 0x01;   // Final weight has been cached.
+constexpr uint8 kCacheArcs = 0x02;    // Arcs have been cached.
+constexpr uint8 kCacheInit = 0x04;    // Initialized by GC.
+constexpr uint8 kCacheRecent = 0x08;  // Visited since GC.
+constexpr uint8 kCacheFlags =
     kCacheFinal | kCacheArcs | kCacheInit | kCacheRecent;
 
 // Cache state, with arcs stored in a per-state std::vector.
@@ -75,7 +91,7 @@ class CacheState {
 
   // Provides STL allocator for arcs.
   explicit CacheState(const ArcAllocator &alloc)
-      : final_(Weight::Zero()),
+      : final_weight_(Weight::Zero()),
         niepsilons_(0),
         noepsilons_(0),
         arcs_(alloc),
@@ -83,7 +99,7 @@ class CacheState {
         ref_count_(0) {}
 
   CacheState(const CacheState<A> &state, const ArcAllocator &alloc)
-      : final_(state.Final()),
+      : final_weight_(state.Final()),
         niepsilons_(state.NumInputEpsilons()),
         noepsilons_(state.NumOutputEpsilons()),
         arcs_(state.arcs_.begin(), state.arcs_.end(), alloc),
@@ -91,7 +107,7 @@ class CacheState {
         ref_count_(0) {}
 
   void Reset() {
-    final_ = Weight::Zero();
+    final_weight_ = Weight::Zero();
     niepsilons_ = 0;
     noepsilons_ = 0;
     ref_count_ = 0;
@@ -99,7 +115,7 @@ class CacheState {
     arcs_.clear();
   }
 
-  Weight Final() const { return final_; }
+  Weight Final() const { return final_weight_; }
 
   size_t NumInputEpsilons() const { return niepsilons_; }
 
@@ -113,31 +129,44 @@ class CacheState {
   const Arc *Arcs() const { return !arcs_.empty() ? &arcs_[0] : nullptr; }
 
   // Accesses flags; used by the caller.
-  uint32 Flags() const { return flags_; }
+  uint8 Flags() const { return flags_; }
 
   // Accesses ref count; used by the caller.
   int RefCount() const { return ref_count_; }
 
-  void SetFinal(Weight weight) { final_ = std::move(weight); }
+  void SetFinal(Weight weight = Weight::One()) {
+    final_weight_ = std::move(weight);
+  }
 
   void ReserveArcs(size_t n) { arcs_.reserve(n); }
 
   // Adds one arc at a time with all needed book-keeping; use PushArc and
   // SetArcs for a more efficient alternative.
   void AddArc(const Arc &arc) {
+    IncrementNumEpsilons(arc);
     arcs_.push_back(arc);
-    if (arc.ilabel == 0) ++niepsilons_;
-    if (arc.olabel == 0) ++noepsilons_;
+  }
+
+  void AddArc(Arc &&arc) {
+    IncrementNumEpsilons(arc);
+    arcs_.push_back(std::move(arc));
   }
 
   // Adds one arc at a time with delayed book-keeping; finalize with SetArcs().
   void PushArc(const Arc &arc) { arcs_.push_back(arc); }
 
+  void PushArc(Arc &&arc) { arcs_.push_back(std::move(arc)); }
+
+  // Adds one arc at a time with delayed book-keeping; finalize with SetArcs().
+  template <class... T>
+  void EmplaceArc(T &&... ctor_args) {
+    arcs_.emplace_back(std::forward<T>(ctor_args)...);
+  }
+
   // Finalizes arcs book-keeping; call only once.
   void SetArcs() {
     for (const auto &arc : arcs_) {
-      if (arc.ilabel == 0) ++niepsilons_;
-      if (arc.olabel == 0) ++noepsilons_;
+      IncrementNumEpsilons(arc);
     }
   }
 
@@ -145,8 +174,7 @@ class CacheState {
   void SetArc(const Arc &arc, size_t n) {
     if (arcs_[n].ilabel == 0) --niepsilons_;
     if (arcs_[n].olabel == 0) --noepsilons_;
-    if (arc.ilabel == 0) ++niepsilons_;
-    if (arc.olabel == 0) ++noepsilons_;
+    IncrementNumEpsilons(arc);
     arcs_[n] = arc;
   }
 
@@ -166,7 +194,7 @@ class CacheState {
   }
 
   // Sets status flags; used by the caller.
-  void SetFlags(uint32 flags, uint32 mask) const {
+  void SetFlags(uint8 flags, uint8 mask) const {
     flags_ &= ~mask;
     flags_ |= flags;
   }
@@ -194,11 +222,17 @@ class CacheState {
   }
 
  private:
-  Weight final_;                         // Final weight.
+  // Update the number of epsilons as a result of having added an arc.
+  void IncrementNumEpsilons(const Arc &arc) {
+    if (arc.ilabel == 0) ++niepsilons_;
+    if (arc.olabel == 0) ++noepsilons_;
+  }
+
+  Weight final_weight_;                  // Final weight.
   size_t niepsilons_;                    // # of input epsilons.
   size_t noepsilons_;                    // # of output epsilons.
   std::vector<Arc, ArcAllocator> arcs_;  // Arcs representation.
-  mutable uint32 flags_;
+  mutable uint8 flags_;
   mutable int ref_count_;  // If 0, available for GC.
 };
 
@@ -250,6 +284,9 @@ class CacheState {
 //   // Deletes all cached states.
 //   void Clear();
 //
+//   // Number of cached states.
+//   StateId CountStates();
+//
 //   // Iterates over cached states (in an arbitrary order); only needed if
 //   // opts.gc is true.
 //   bool Done() const;      // End of iteration.
@@ -284,7 +321,7 @@ class VectorCacheStore {
 
   ~VectorCacheStore() { Clear(); }
 
-  VectorCacheStore<State> &operator=(const VectorCacheStore<State> &store) {
+  VectorCacheStore &operator=(const VectorCacheStore &store) {
     if (this != &store) {
       CopyStates(store);
       Reset();
@@ -292,18 +329,22 @@ class VectorCacheStore {
     return *this;
   }
 
+  bool InBounds(StateId s) const {
+    return s < static_cast<StateId>(state_vec_.size());
+  }
+
   // Returns nullptr if state is not stored.
   const State *GetState(StateId s) const {
-    return s < state_vec_.size() ? state_vec_[s] : nullptr;
+    return InBounds(s) ? state_vec_[s] : nullptr;
   }
 
   // Creates state if state is not stored.
   State *GetMutableState(StateId s) {
     State *state = nullptr;
-    if (s >= state_vec_.size()) {
-      state_vec_.resize(s + 1, nullptr);
-    } else {
+    if (InBounds(s)) {
       state = state_vec_[s];
+    } else {
+      state_vec_.resize(s + 1, nullptr);
     }
     if (!state) {
       state = new (&state_alloc_) State(arc_alloc_);
@@ -328,13 +369,18 @@ class VectorCacheStore {
 
   // Deletes all cached states.
   void Clear() {
-    for (StateId s = 0; s < state_vec_.size(); ++s) {
-      State::Destroy(state_vec_[s], &state_alloc_);
+    for (State *s : state_vec_) {
+      State::Destroy(s, &state_alloc_);
     }
     state_vec_.clear();
     state_list_.clear();
   }
 
+  StateId CountStates() const {
+    return std::count_if(state_vec_.begin(), state_vec_.end(),
+                         [](const State *s) { return s != nullptr; });
+  }
+
   // Iterates over cached states (in an arbitrary order); only works if GC is
   // enabled (o.w. avoiding state_list_ overhead).
   bool Done() const { return iter_ == state_list_.end(); }
@@ -356,7 +402,7 @@ class VectorCacheStore {
   void CopyStates(const VectorCacheStore<State> &store) {
     Clear();
     state_vec_.reserve(store.state_vec_.size());
-    for (StateId s = 0; s < store.state_vec_.size(); ++s) {
+    for (size_t s = 0; s < store.state_vec_.size(); ++s) {
       State *state = nullptr;
       const auto *store_state = store.state_vec_[s];
       if (store_state) {
@@ -385,8 +431,8 @@ class HashCacheStore {
 
   using StateMap =
       std::unordered_map<StateId, State *, std::hash<StateId>,
-                         std::equal_to<StateId>,
-                         PoolAllocator<std::pair<const StateId, State *>>>;
+                          std::equal_to<StateId>,
+                          PoolAllocator<std::pair<const StateId, State *>>>;
 
   // Required constructors/assignment operators.
   explicit HashCacheStore(const CacheOptions &opts) {
@@ -401,7 +447,7 @@ class HashCacheStore {
 
   ~HashCacheStore() { Clear(); }
 
-  HashCacheStore<State> &operator=(const HashCacheStore<State> &store) {
+  HashCacheStore &operator=(const HashCacheStore &store) {
     if (this != &store) {
       CopyStates(store);
       Reset();
@@ -443,6 +489,8 @@ class HashCacheStore {
     state_map_.clear();
   }
 
+  StateId CountStates() const { return state_map_.size(); }
+
   // Iterates over cached states (in an arbitrary order).
   bool Done() const { return iter_ == state_map_.end(); }
 
@@ -575,6 +623,8 @@ class FirstCacheStore {
     cache_first_state_ = nullptr;
   }
 
+  StateId CountStates() const { return store_.CountStates(); }
+
   // Iterates over cached states (in an arbitrary order). Only needed if GC is
   // enabled.
   bool Done() const { return store_.Done(); }
@@ -684,6 +734,8 @@ class GCCacheStore {
     cache_size_ = 0;
   }
 
+  StateId CountStates() const { return store_.CountStates(); }
+
   // Iterates over cached states (in an arbitrary order); only needed if GC is
   // enabled.
   bool Done() const { return store_.Done(); }
@@ -830,8 +882,9 @@ class CacheBaseImpl : public FstImpl<typename State::Arc> {
         max_expanded_state_id_(-1),
         cache_gc_(opts.gc),
         cache_limit_(opts.gc_limit),
-        cache_store_(opts.store ? opts.store : new CacheStore(CacheOptions(
-                                                   opts.gc, opts.gc_limit))),
+        cache_store_(
+            opts.store ? opts.store
+                       : new CacheStore(CacheOptions(opts.gc, opts.gc_limit))),
         new_cache_store_(!opts.store),
         own_cache_store_(opts.store ? opts.own_store : true) {}
 
@@ -861,7 +914,9 @@ class CacheBaseImpl : public FstImpl<typename State::Arc> {
     }
   }
 
-  ~CacheBaseImpl() override { if (own_cache_store_) delete cache_store_; }
+  ~CacheBaseImpl() override {
+    if (own_cache_store_) delete cache_store_;
+  }
 
   void SetStart(StateId s) {
     cache_start_ = s;
@@ -869,38 +924,35 @@ class CacheBaseImpl : public FstImpl<typename State::Arc> {
     if (s >= nknown_states_) nknown_states_ = s + 1;
   }
 
-  void SetFinal(StateId s, Weight weight) {
+  void SetFinal(StateId s, Weight weight = Weight::One()) {
     auto *state = cache_store_->GetMutableState(s);
     state->SetFinal(std::move(weight));
     static constexpr auto flags = kCacheFinal | kCacheRecent;
     state->SetFlags(flags, flags);
   }
 
-// Disabled to ensure PushArc not AddArc is used in existing code
-// TODO(sorenj): re-enable for backing store
-#if 0
-  // AddArc adds a single arc to a state and does incremental cache
-  // book-keeping. For efficiency, prefer PushArc and SetArcs below
-  // when possible.
-  void AddArc(StateId s, const Arc &arc) {
-    auto *state = cache_store_->GetMutableState(s);
-    cache_store_->AddArc(state, arc);
-    if (arc.nextstate >= nknown_states_)
-      nknown_states_ = arc.nextstate + 1;
-    SetExpandedState(s);
-    static constexpr auto flags = kCacheArcs | kCacheRecent;
-    state->SetFlags(flags, flags);
-  }
-#endif
-
   // Adds a single arc to a state but delays cache book-keeping. SetArcs must
-  // be called when all PushArc calls at a state are complete. Do not mix with
-  // calls to AddArc.
+  // be called when all PushArc and EmplaceArc calls at a state are complete.
+  // Do not mix with calls to AddArc.
   void PushArc(StateId s, const Arc &arc) {
     auto *state = cache_store_->GetMutableState(s);
     state->PushArc(arc);
   }
 
+  void PushArc(StateId s, Arc &&arc) {
+    auto *state = cache_store_->GetMutableState(s);
+    state->PushArc(std::move(arc));
+  }
+
+  // Adds a single arc to a state but delays cache book-keeping. SetArcs must
+  // be called when all PushArc and EmplaceArc calls at a state are complete.
+  // Do not mix with calls to AddArc.
+  template <class... T>
+  void EmplaceArc(StateId s, T &&... ctor_args) {
+    auto *state = cache_store_->GetMutableState(s);
+    state->EmplaceArc(std::forward<T>(ctor_args)...);
+  }
+
   // Marks arcs of a state as cached and does cache book-keeping after all
   // calls to PushArc have been completed. Do not mix with calls to AddArc.
   void SetArcs(StateId s) {
@@ -1025,7 +1077,8 @@ class CacheBaseImpl : public FstImpl<typename State::Arc> {
     if (s < min_unexpanded_state_id_) return;
     if (s == min_unexpanded_state_id_) ++min_unexpanded_state_id_;
     if (cache_gc_ || cache_limit_ == 0) {
-      if (expanded_states_.size() <= s) expanded_states_.resize(s + 1, false);
+      if (expanded_states_.size() <= static_cast<size_t>(s))
+        expanded_states_.resize(s + 1, false);
       expanded_states_[s] = true;
     }
   }
@@ -1170,9 +1223,9 @@ class CacheArcIterator {
 
   void Seek(size_t a) { i_ = a; }
 
-  constexpr uint32 Flags() const { return kArcValueFlags; }
+  constexpr uint8 Flags() const { return kArcValueFlags; }
 
-  void SetFlags(uint32 flags, uint32 mask) {}
+  void SetFlags(uint8 flags, uint8 mask) {}
 
  private:
   const State *state_;
@@ -1218,9 +1271,9 @@ class CacheMutableArcIterator
 
   void SetValue(const Arc &arc) final { state_->SetArc(arc, i_); }
 
-  uint32 Flags() const final { return kArcValueFlags; }
+  uint8 Flags() const final { return kArcValueFlags; }
 
-  void SetFlags(uint32, uint32) final {}
+  void SetFlags(uint8, uint8) final {}
 
  private:
   size_t i_;
@@ -1245,7 +1298,7 @@ class ExpanderCacheStore {
       : store_(opts) {}
 
   template <class Expander>
-  State *FindOrExpand(Expander &expander, StateId s) {  // NOLINT
+  State *FindOrExpand(Expander &expander, StateId s) {
     auto *state = store_.GetMutableState(s);
     if (state->Flags()) {
       state->SetFlags(kCacheRecent, kCacheRecent);
@@ -1268,7 +1321,11 @@ class ExpanderCacheStore {
 
     void AddArc(const Arc &arc) { state->PushArc(arc); }
 
-    void SetFinal(Weight weight) { state->SetFinal(std::move(weight)); }
+    void AddArc(Arc &&arc) { state->PushArc(std::move(arc)); }
+
+    void SetFinal(Weight weight = Weight::One()) {
+      state->SetFinal(std::move(weight));
+    }
   };
 };
 
