@@ -1,3 +1,17 @@
+// Copyright 2005-2020 Google LLC
+//
+// Licensed under the Apache License, Version 2.0 (the 'License');
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+//
+//     http://www.apache.org/licenses/LICENSE-2.0
+//
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an 'AS IS' BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
+//
 // See www.openfst.org for extensive documentation on this weighted
 // finite-state transducer library.
 //
@@ -7,6 +21,8 @@
 #ifndef FST_COMPOSE_FILTER_H_
 #define FST_COMPOSE_FILTER_H_
 
+#include <fst/types.h>
+
 #include <fst/filter-state.h>
 #include <fst/fst-decl.h>  // For optional argument declarations
 #include <fst/fst.h>
@@ -96,7 +112,7 @@ class NullComposeFilter {
         fst1_(matcher1_->GetFst()),
         fst2_(matcher2_->GetFst()) {}
 
-  NullComposeFilter(const NullComposeFilter<M1, M2> &filter, bool safe = false)
+  NullComposeFilter(const NullComposeFilter &filter, bool safe = false)
       : matcher1_(filter.matcher1_->Copy(safe)),
         matcher2_(filter.matcher2_->Copy(safe)),
         fst1_(matcher1_->GetFst()),
@@ -237,8 +253,8 @@ class SequenceComposeFilter {
 
   FilterState FilterArc(Arc *arc1, Arc *arc2) const {
     if (arc1->olabel == kNoLabel) {
-      return alleps1_ ? FilterState::NoState() : noeps1_ ? FilterState(0)
-                                                         : FilterState(1);
+      return alleps1_ ? FilterState::NoState()
+                      : noeps1_ ? FilterState(0) : FilterState(1);
     } else if (arc2->ilabel == kNoLabel) {
       return fs_ != FilterState(0) ? FilterState::NoState() : FilterState(0);
     } else {
@@ -261,8 +277,8 @@ class SequenceComposeFilter {
   StateId s1_;      // Current fst1_ state.
   StateId s2_;      // Current fst2_ state.
   FilterState fs_;  // Current filter state.
-  bool alleps1_;   // Only epsilons (and non-final) leaving s1_?
-  bool noeps1_;    // No epsilons leaving s1_?
+  bool alleps1_;    // Only epsilons (and non-final) leaving s1_?
+  bool noeps1_;     // No epsilons leaving s1_?
 };
 
 // This filter requires epsilons on FST2 to be read before epsilons on FST1.
@@ -316,8 +332,8 @@ class AltSequenceComposeFilter {
 
   FilterState FilterArc(Arc *arc1, Arc *arc2) const {
     if (arc2->ilabel == kNoLabel) {
-      return alleps2_ ? FilterState::NoState() : noeps2_ ? FilterState(0)
-                                                         : FilterState(1);
+      return alleps2_ ? FilterState::NoState()
+                      : noeps2_ ? FilterState(0) : FilterState(1);
     } else if (arc1->olabel == kNoLabel) {
       return fs_ == FilterState(1) ? FilterState::NoState() : FilterState(0);
     } else {
@@ -443,6 +459,69 @@ class MatchComposeFilter {
   bool noeps2_;     // No epsilons leaving s2?
 };
 
+// This filter disallows matching epsilons on FST1 with epsilons on FST2,
+// but allows all other matches, potentially resulting in redundant
+// epsilon paths. The use of this filter gives correct results iff one of the
+// following conditions hold:
+//
+//  (1) The semiring is idempotent,
+//  (2) the first FST is output-epsilon free, or
+//  (3) the second FST is input-epsilon free.
+//
+// For (1), redundant epsilon paths may be created but won't hurt correctness.
+// For (2) and (3), no redundant paths are created.
+template <class M1, class M2 /* = M1 */>
+class NoMatchComposeFilter {
+ public:
+  using Matcher1 = M1;
+  using Matcher2 = M2;
+  using FST1 = typename M1::FST;
+  using FST2 = typename M2::FST;
+  using FilterState = TrivialFilterState;
+
+  using Arc = typename FST1::Arc;
+  using Label = typename Arc::Label;
+  using StateId = typename Arc::StateId;
+  using Weight = typename Arc::Weight;
+
+  NoMatchComposeFilter(const FST1 &fst1, const FST2 &fst2,
+                       Matcher1 *matcher1 = nullptr,
+                       Matcher2 *matcher2 = nullptr)
+      : matcher1_(matcher1 ? matcher1 : new Matcher1(fst1, MATCH_OUTPUT)),
+        matcher2_(matcher2 ? matcher2 : new Matcher2(fst2, MATCH_INPUT)),
+        fst1_(matcher1_->GetFst()),
+        fst2_(matcher2_->GetFst()) {}
+
+  NoMatchComposeFilter(const NoMatchComposeFilter<Matcher1, Matcher2> &filter,
+                       bool safe = false)
+      : matcher1_(filter.matcher1_->Copy(safe)),
+        matcher2_(filter.matcher2_->Copy(safe)),
+        fst1_(matcher1_->GetFst()),
+        fst2_(matcher2_->GetFst()) {}
+
+  FilterState Start() const { return FilterState(true); }
+
+  void SetState(StateId, StateId, const FilterState &) {}
+
+  FilterState FilterArc(Arc *arc1, Arc *arc2) const {
+    return FilterState(arc1->olabel != 0 || arc2->ilabel != 0);
+  }
+
+  void FilterFinal(Weight *, Weight *) const {}
+
+  Matcher1 *GetMatcher1() { return matcher1_.get(); }
+
+  Matcher2 *GetMatcher2() { return matcher2_.get(); }
+
+  uint64 Properties(uint64 props) const { return props; }
+
+ private:
+  std::unique_ptr<Matcher1> matcher1_;
+  std::unique_ptr<Matcher2> matcher2_;
+  const FST1 &fst1_;
+  const FST2 &fst2_;
+};
+
 // This filter works with the MultiEpsMatcher to determine if multi-epsilons are
 // preserved in the composition output (rather than rewritten as 0) and
 // ensures correct properties.
@@ -466,7 +545,7 @@ class MultiEpsFilter {
       : filter_(fst1, fst2, matcher1, matcher2),
         keep_multi_eps_(keep_multi_eps) {}
 
-  MultiEpsFilter(const MultiEpsFilter<Filter> &filter, bool safe = false)
+  MultiEpsFilter(const MultiEpsFilter &filter, bool safe = false)
       : filter_(filter.filter_, safe),
         keep_multi_eps_(filter.keep_multi_eps_) {}
 
