@@ -1,3 +1,17 @@
+// Copyright 2005-2020 Google LLC
+//
+// Licensed under the Apache License, Version 2.0 (the 'License');
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+//
+//     http://www.apache.org/licenses/LICENSE-2.0
+//
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an 'AS IS' BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
+//
 // See www.openfst.org for extensive documentation on this weighted
 // finite-state transducer library.
 //
@@ -11,9 +25,12 @@
 #include <forward_list>
 #include <map>
 #include <string>
+#include <utility>
 #include <vector>
 
+#include <fst/types.h>
 #include <fst/log.h>
+#include <fst/const-fst.h>
 
 #include <fst/arc-map.h>
 #include <fst/bi-table.h>
@@ -23,7 +40,6 @@
 #include <fst/prune.h>
 #include <fst/test-properties.h>
 
-
 namespace fst {
 
 // Common divisors are used in determinization to compute transition weights.
@@ -230,14 +246,14 @@ class DefaultDeterminizeFilter {
   // Filters transition, possibly modifying label map. Returns true if arc is
   // added to the label map.
   bool FilterArc(const Arc &arc, const Element &src_element,
-                 const Element &dest_element, LabelMap *label_map) const {
+                 Element &&dest_element, LabelMap *label_map) const {
     // Adds element to unique state tuple for arc label.
     auto &det_arc = (*label_map)[arc.ilabel];
     if (det_arc.label == kNoLabel) {
       det_arc = internal::DeterminizeArc<StateTuple>(arc);
       det_arc.dest_tuple->filter_state = FilterState(0);
     }
-    det_arc.dest_tuple->subset.push_front(dest_element);
+    det_arc.dest_tuple->subset.push_front(std::move(dest_element));
     return true;
   }
 
@@ -460,7 +476,7 @@ class DeterminizeFstImplBase : public CacheImpl<Arc> {
     SetOutputSymbols(fst.OutputSymbols());
   }
 
-  DeterminizeFstImplBase(const DeterminizeFstImplBase<Arc> &impl)
+  DeterminizeFstImplBase(const DeterminizeFstImplBase &impl)
       : CacheImpl<Arc>(impl), fst_(impl.fst_->Copy(true)) {
     SetType("determinize");
     SetProperties(impl.Properties(), kCopyProperties);
@@ -468,7 +484,7 @@ class DeterminizeFstImplBase : public CacheImpl<Arc> {
     SetOutputSymbols(impl.OutputSymbols());
   }
 
-  virtual DeterminizeFstImplBase<Arc> *Copy() const = 0;
+  virtual DeterminizeFstImplBase *Copy() const = 0;
 
   StateId Start() {
     if (!HasStart()) {
@@ -555,8 +571,7 @@ class DeterminizeFsaImpl : public DeterminizeFstImplBase<Arc> {
     if (out_dist_) out_dist_->clear();
   }
 
-  DeterminizeFsaImpl(
-      const DeterminizeFsaImpl<Arc, CommonDivisor, Filter, StateTable> &impl)
+  DeterminizeFsaImpl(const DeterminizeFsaImpl &impl)
       : DeterminizeFstImplBase<Arc>(impl),
         delta_(impl.delta_),
         in_dist_(nullptr),
@@ -569,10 +584,8 @@ class DeterminizeFsaImpl : public DeterminizeFstImplBase<Arc> {
     }
   }
 
-  DeterminizeFsaImpl<Arc, CommonDivisor, Filter, StateTable> *Copy()
-      const override {
-    return new DeterminizeFsaImpl<Arc, CommonDivisor, Filter, StateTable>(
-        *this);
+  DeterminizeFsaImpl *Copy() const override {
+    return new DeterminizeFsaImpl(*this);
   }
 
   uint64 Properties() const override { return Properties(kFstProperties); }
@@ -588,9 +601,8 @@ class DeterminizeFsaImpl : public DeterminizeFstImplBase<Arc> {
   StateId ComputeStart() override {
     const auto s = GetFst().Start();
     if (s == kNoStateId) return kNoStateId;
-    const Element element(s, Weight::One());
     auto *tuple = new StateTuple;
-    tuple->subset.push_front(element);
+    tuple->subset.emplace_front(s, Weight::One());
     tuple->filter_state = filter_->Start();
     return FindState(tuple);
   }
@@ -638,7 +650,7 @@ class DeterminizeFsaImpl : public DeterminizeFstImplBase<Arc> {
     LabelMap label_map;
     GetLabelMap(s, &label_map);
     for (auto it = label_map.begin(); it != label_map.end(); ++it) {
-      AddArc(s, it->second);
+      AddArc(s, std::move(it->second));
     }
     SetArcs(s);
   }
@@ -657,9 +669,10 @@ class DeterminizeFsaImpl : public DeterminizeFstImplBase<Arc> {
       for (ArcIterator<Fst<Arc>> aiter(GetFst(), src_element.state_id);
            !aiter.Done(); aiter.Next()) {
         const auto &arc = aiter.Value();
-        const Element dest_element(arc.nextstate,
-                                   Times(src_element.weight, arc.weight));
-        filter_->FilterArc(arc, src_element, dest_element, label_map);
+        Element dest_element(arc.nextstate,
+                             Times(src_element.weight, arc.weight));
+        filter_->FilterArc(arc, src_element, std::move(dest_element),
+                           label_map);
       }
     }
     for (auto it = label_map->begin(); it != label_map->end(); ++it) {
@@ -703,22 +716,25 @@ class DeterminizeFsaImpl : public DeterminizeFstImplBase<Arc> {
 
   // Adds an arc from state S to the destination state associated with state
   // tuple in det_arc as created by GetLabelMap.
-  void AddArc(StateId s, const DetArc &det_arc) {
-    const Arc arc(det_arc.label, det_arc.label, det_arc.weight,
-                  FindState(det_arc.dest_tuple));
-    CacheImpl<Arc>::PushArc(s, arc);
+  void AddArc(StateId s, DetArc &&det_arc) {
+    CacheImpl<Arc>::EmplaceArc(s, det_arc.label, det_arc.label,
+                               std::move(det_arc.weight),
+                               FindState(det_arc.dest_tuple));
   }
 
   float delta_;                         // Quantization delta for weights.
   const std::vector<Weight> *in_dist_;  // Distance to final NFA states.
   std::vector<Weight> *out_dist_;       // Distance to final DFA states.
 
-  // FIXME(kbg): Ought to be static const?
-  CommonDivisor common_divisor_;
+  static const CommonDivisor common_divisor_;
   std::unique_ptr<Filter> filter_;
   std::unique_ptr<StateTable> state_table_;
 };
 
+template <class Arc, class CommonDivisor, class Filter, class StateTable>
+const CommonDivisor DeterminizeFsaImpl<Arc, CommonDivisor, Filter,
+                                       StateTable>::common_divisor_{};
+
 // Implementation of delayed determinization for transducers. Transducer
 // determinization is implemented by mapping the input to the Gallic semiring as
 // an acceptor whose weights contain the output strings and using acceptor
@@ -765,8 +781,7 @@ class DeterminizeFstImpl : public DeterminizeFstImplBase<Arc> {
     Init(GetFst(), opts.filter);
   }
 
-  DeterminizeFstImpl(
-      const DeterminizeFstImpl<Arc, G, CommonDivisor, Filter, StateTable> &impl)
+  DeterminizeFstImpl(const DeterminizeFstImpl &impl)
       : DeterminizeFstImplBase<Arc>(impl),
         delta_(impl.delta_),
         subsequential_label_(impl.subsequential_label_),
@@ -774,10 +789,8 @@ class DeterminizeFstImpl : public DeterminizeFstImplBase<Arc> {
     Init(GetFst(), nullptr);
   }
 
-  DeterminizeFstImpl<Arc, G, CommonDivisor, Filter, StateTable> *Copy()
-      const override {
-    return new DeterminizeFstImpl<Arc, G, CommonDivisor, Filter, StateTable>(
-        *this);
+  DeterminizeFstImpl *Copy() const override {
+    return new DeterminizeFstImpl(*this);
   }
 
   uint64 Properties() const override { return Properties(kFstProperties); }
@@ -863,7 +876,7 @@ class DeterminizeFst : public ImplToFst<internal::DeterminizeFstImplBase<A>> {
       : ImplToFst<Impl>(CreateImpl(fst)) {}
 
   template <class CommonDivisor, class Filter, class StateTable>
-  DeterminizeFst(
+  explicit DeterminizeFst(
       const Fst<Arc> &fst,
       const DeterminizeFstOptions<Arc, CommonDivisor, Filter, StateTable>
           &opts =
@@ -892,13 +905,13 @@ class DeterminizeFst : public ImplToFst<internal::DeterminizeFstImplBase<A>> {
   }
 
   // See Fst<>::Copy() for doc.
-  DeterminizeFst(const DeterminizeFst<Arc> &fst, bool safe = false)
+  DeterminizeFst(const DeterminizeFst &fst, bool safe = false)
       : ImplToFst<Impl>(safe ? std::shared_ptr<Impl>(fst.GetImpl()->Copy())
                              : fst.GetSharedImpl()) {}
 
   // Get a copy of this DeterminizeFst. See Fst<>::Copy() for further doc.
-  DeterminizeFst<Arc> *Copy(bool safe = false) const override {
-    return new DeterminizeFst<Arc>(*this, safe);
+  DeterminizeFst *Copy(bool safe = false) const override {
+    return new DeterminizeFst(*this, safe);
   }
 
   inline void InitStateIterator(StateIteratorData<Arc> *data) const override;
@@ -930,16 +943,20 @@ class DeterminizeFst : public ImplToFst<internal::DeterminizeFstImplBase<A>> {
           internal::DeterminizeFsaImpl<Arc, CommonDivisor, Filter, StateTable>>(
           fst, nullptr, nullptr, opts);
     } else if (opts.type == DETERMINIZE_DISAMBIGUATE) {
-      auto rv = std::make_shared<internal::DeterminizeFstImpl<
-          Arc, GALLIC_MIN, CommonDivisor, Filter, StateTable>>(fst, opts);
-      if (!(Weight::Properties() & kPath)) {
-        FSTERROR() << "DeterminizeFst: Weight needs to have the "
-                   << "path property to disambiguate output: "
-                   << Weight::Type();
+      if constexpr (IsPath<Weight>::value) {
+        // Calls disambiguating implementation for non-functional transducers.
+        return std::make_shared<internal::DeterminizeFstImpl<
+            Arc, GALLIC_MIN, CommonDivisor, Filter, StateTable>>(fst, opts);
+      } else {
+        FSTERROR() << "DeterminizeFst: Weight needs to have the path "
+                   << "property to disambiguate output: " << Weight::Type();
+        // Return an error Impl.
+        const ConstFst<Arc> empty_fst;
+        auto rv = std::make_shared<internal::DeterminizeFstImpl<
+            Arc, GALLIC, CommonDivisor, Filter, StateTable>>(empty_fst, opts);
         rv->SetProperties(kError, kError);
+        return rv;
       }
-      // Calls disambiguating implementation for non-functional transducers.
-      return rv;
     } else if (opts.type == DETERMINIZE_FUNCTIONAL) {
       // Calls implementation for functional transducers.
       return std::make_shared<internal::DeterminizeFstImpl<
@@ -976,7 +993,8 @@ void DeterminizeFstImpl<A, G, D, F, T>::Init(const Fst<A> &fst, F *filter) {
       subsequential_label_, increment_subsequential_label_,
       increment_subsequential_label_);
   const FactorWeightFst<ToArc, FactorIterator> factored_fst(det_fsa, fopts);
-  from_fst_.reset(new FromFst(factored_fst, FromMapper(subsequential_label_)));
+  from_fst_ = std::make_unique<FromFst>(factored_fst,
+                                         FromMapper(subsequential_label_));
 }
 
 }  // namespace internal
@@ -1006,7 +1024,7 @@ class ArcIterator<DeterminizeFst<Arc>>
 template <class Arc>
 inline void DeterminizeFst<Arc>::InitStateIterator(
     StateIteratorData<Arc> *data) const {
-  data->base = new StateIterator<DeterminizeFst<Arc>>(*this);
+  data->base = std::make_unique<StateIterator<DeterminizeFst<Arc>>>(*this);
 }
 
 // Useful aliases when using StdArc.
@@ -1069,18 +1087,24 @@ void Determinize(
   nopts.gc_limit = 0;  // Caches only the last state for fastest copy.
   if (opts.weight_threshold != Weight::Zero() ||
       opts.state_threshold != kNoStateId) {
-    if (ifst.Properties(kAcceptor, false)) {
-      std::vector<Weight> idistance;
-      std::vector<Weight> odistance;
-      ShortestDistance(ifst, &idistance, true);
-      DeterminizeFst<Arc> dfst(ifst, &idistance, &odistance, nopts);
-      PruneOptions<Arc, AnyArcFilter<Arc>> popts(
-          opts.weight_threshold, opts.state_threshold, AnyArcFilter<Arc>(),
-          &odistance);
-      Prune(dfst, ofst, popts);
+    if constexpr (IsPath<Weight>::value) {
+      if (ifst.Properties(kAcceptor, false)) {
+        std::vector<Weight> idistance;
+        std::vector<Weight> odistance;
+        ShortestDistance(ifst, &idistance, true);
+        DeterminizeFst<Arc> dfst(ifst, &idistance, &odistance, nopts);
+        PruneOptions<Arc, AnyArcFilter<Arc>> popts(
+            opts.weight_threshold, opts.state_threshold, AnyArcFilter<Arc>(),
+            &odistance);
+        Prune(dfst, ofst, popts);
+      } else {
+        *ofst = DeterminizeFst<Arc>(ifst, nopts);
+        Prune(ofst, opts.weight_threshold, opts.state_threshold);
+      }
     } else {
-      *ofst = DeterminizeFst<Arc>(ifst, nopts);
-      Prune(ofst, opts.weight_threshold, opts.state_threshold);
+      FSTERROR() << "Determinize: Weight needs to have the path "
+                 << "property to use pruning options: " << Weight::Type();
+      ofst->SetProperties(kError, kError);
     }
   } else {
     *ofst = DeterminizeFst<Arc>(ifst, nopts);
