@@ -1,3 +1,17 @@
+// Copyright 2005-2020 Google LLC
+//
+// Licensed under the Apache License, Version 2.0 (the 'License');
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+//
+//     http://www.apache.org/licenses/LICENSE-2.0
+//
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an 'AS IS' BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
+//
 // See www.openfst.org for extensive documentation on this weighted
 // finite-state transducer library.
 //
@@ -12,6 +26,8 @@
 #include <utility>
 #include <vector>
 
+#include <fst/types.h>
+
 #include <fst/arcsort.h>
 #include <fst/compose.h>
 #include <fst/connect.h>
@@ -24,7 +40,6 @@
 #include <fst/union-find.h>
 #include <fst/verify.h>
 
-
 namespace fst {
 
 template <class Arc>
@@ -174,7 +189,7 @@ void RelationDeterminizeFilter<Arc, Relation>::InitLabelMap(
     if (arc.ilabel == label && arc.nextstate == nextstate) continue;
     DeterminizeArc<StateTuple> det_arc(arc);
     det_arc.dest_tuple->filter_state = FilterState(arc.nextstate);
-    label_map->insert(std::make_pair(arc.ilabel, det_arc));
+    label_map->emplace(arc.ilabel, det_arc);
     label = arc.ilabel;
     nextstate = arc.nextstate;
   }
@@ -267,7 +282,7 @@ class Disambiguator {
       // Ensures composition is between acceptors.
       const bool trans = ifst.Properties(kNotAcceptor, true);
       const auto *fsa =
-          trans ? new ProjectFst<Arc>(ifst, PROJECT_INPUT) : &ifst;
+          trans ? new ProjectFst<Arc>(ifst, ProjectType::INPUT) : &ifst;
       opts.state_table = new StateTable(*fsa, *fsa);
       const ComposeFst<Arc> cfst(*fsa, *fsa, opts);
       std::vector<bool> coaccess;
@@ -383,19 +398,26 @@ void Disambiguator<Arc>::PreDisambiguate(const ExpandedFst<Arc> &ifst,
   nopts.gc_limit = 0;  // Cache only the last state for fastest copy.
   if (opts.weight_threshold != Weight::Zero() ||
       opts.state_threshold != kNoStateId) {
-    /* TODO(riley): fails regression test; understand why
-    if (ifst.Properties(kAcceptor, true)) {
-      std::vector<Weight> idistance, odistance;
-      ShortestDistance(ifst, &idistance, true);
-      DeterminizeFst<Arc> dfst(ifst, &idistance, &odistance, nopts);
-      PruneOptions< Arc, AnyArcFilter<Arc>> popts(opts.weight_threshold,
-                                                   opts.state_threshold,
-                                                   AnyArcFilter<Arc>(),
-                                                   &odistance);
-      Prune(dfst, ofst, popts);
-      } else */ {
-      *ofst = DeterminizeFst<Arc>(ifst, nopts);
-      Prune(ofst, opts.weight_threshold, opts.state_threshold);
+    if constexpr (IsPath<Weight>::value) {
+      /* TODO(riley): fails regression test; understand why
+      if (ifst.Properties(kAcceptor, true)) {
+        std::vector<Weight> idistance, odistance;
+        ShortestDistance(ifst, &idistance, true);
+        DeterminizeFst<Arc> dfst(ifst, &idistance, &odistance, nopts);
+        PruneOptions< Arc, AnyArcFilter<Arc>> popts(opts.weight_threshold,
+                                                     opts.state_threshold,
+                                                     AnyArcFilter<Arc>(),
+                                                     &odistance);
+        Prune(dfst, ofst, popts);
+        } else */
+      {
+        *ofst = DeterminizeFst<Arc>(ifst, nopts);
+        Prune(ofst, opts.weight_threshold, opts.state_threshold);
+      }
+    } else {
+      FSTERROR() << "Disambiguate: Weight must have path property to use "
+                 << "pruning options: " << Weight::Type();
+      error_ = true;
     }
   } else {
     *ofst = DeterminizeFst<Arc>(ifst, nopts);
@@ -406,7 +428,7 @@ void Disambiguator<Arc>::PreDisambiguate(const ExpandedFst<Arc> &ifst,
 template <class Arc>
 void Disambiguator<Arc>::FindAmbiguities(const ExpandedFst<Arc> &fst) {
   if (fst.Start() == kNoStateId) return;
-  candidates_.reset(new ArcIdMap(ArcIdCompare(head_)));
+  candidates_ = std::make_unique<ArcIdMap>(ArcIdCompare(head_));
   const auto start_pr = std::make_pair(fst.Start(), fst.Start());
   coreachable_.insert(start_pr);
   queue_.push_back(start_pr);
@@ -447,7 +469,8 @@ void Disambiguator<Arc>::FindAmbiguousPairs(const ExpandedFst<Arc> &fst,
           if (spr.first != spr.second &&
               head_[spr.first] == head_[spr.second]) {
             if (!merge_) {
-              merge_.reset(new UnionFind<StateId>(fst.NumStates(), kNoStateId));
+              merge_ = std::make_unique<UnionFind<StateId>>(fst.NumStates(),
+                                                             kNoStateId);
               merge_->MakeAllSet(fst.NumStates());
             }
             merge_->Union(spr.first, spr.second);
