@@ -1,3 +1,17 @@
+// Copyright 2005-2020 Google LLC
+//
+// Licensed under the Apache License, Version 2.0 (the 'License');
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+//
+//     http://www.apache.org/licenses/LICENSE-2.0
+//
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an 'AS IS' BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
+//
 // See www.openfst.org for extensive documentation on this weighted
 // finite-state transducer library.
 //
@@ -8,35 +22,36 @@
 
 #include <algorithm>
 #include <string>
-#include <unordered_map>
 #include <utility>
 #include <vector>
 
+#include <fst/types.h>
 #include <fst/log.h>
 
 #include <fst/cache.h>
 #include <fst/test-properties.h>
 
+#include <unordered_map>
 
 namespace fst {
 
-constexpr uint32 kFactorFinalWeights = 0x00000001;
-constexpr uint32 kFactorArcWeights = 0x00000002;
+constexpr uint8 kFactorFinalWeights = 0x01;
+constexpr uint8 kFactorArcWeights = 0x02;
 
 template <class Arc>
 struct FactorWeightOptions : CacheOptions {
   using Label = typename Arc::Label;
 
   float delta;
-  uint32 mode;         // Factor arc weights and/or final weights.
+  uint8 mode;          // Factor arc weights and/or final weights.
   Label final_ilabel;  // Input label of arc when factoring final weights.
   Label final_olabel;  // Output label of arc when factoring final weights.
   bool increment_final_ilabel;  // When factoring final w' results in > 1 arcs
   bool increment_final_olabel;  // at state, increment labels to make distinct?
 
   explicit FactorWeightOptions(const CacheOptions &opts, float delta = kDelta,
-                               uint32 mode = kFactorArcWeights |
-                                             kFactorFinalWeights,
+                               uint8 mode = kFactorArcWeights |
+                                            kFactorFinalWeights,
                                Label final_ilabel = 0, Label final_olabel = 0,
                                bool increment_final_ilabel = false,
                                bool increment_final_olabel = false)
@@ -49,8 +64,8 @@ struct FactorWeightOptions : CacheOptions {
         increment_final_olabel(increment_final_olabel) {}
 
   explicit FactorWeightOptions(float delta = kDelta,
-                               uint32 mode = kFactorArcWeights |
-                                             kFactorFinalWeights,
+                               uint8 mode = kFactorArcWeights |
+                                            kFactorFinalWeights,
                                Label final_ilabel = 0, Label final_olabel = 0,
                                bool increment_final_ilabel = false,
                                bool increment_final_olabel = false)
@@ -95,6 +110,27 @@ class IdentityFactor {
   void Reset() {}
 };
 
+// Factor the Fst to unfold it as needed so that every two paths leading to the
+// same state have the same weight. Requires applying only to arc weights
+// (FactorWeightOptions::mode == kFactorArcWeights).
+template <class W>
+class OneFactor {
+ public:
+  explicit OneFactor(const W &w) : weight_(w), done_(w == W::One()) {}
+
+  bool Done() const { return done_; }
+
+  void Next() { done_ = true; }
+
+  std::pair<W, W> Value() const { return std::make_pair(W::One(), weight_); }
+
+  void Reset() { done_ = weight_ == W::One(); }
+
+ private:
+  W weight_;
+  bool done_;
+};
+
 // Factors a StringWeight w as 'ab' where 'a' is a label.
 template <typename Label, StringType S = STRING_LEFT>
 class StringFactor {
@@ -196,10 +232,10 @@ class FactorWeightFstImpl : public CacheImpl<Arc> {
   using FstImpl<Arc>::SetInputSymbols;
   using FstImpl<Arc>::SetOutputSymbols;
 
-  using CacheBaseImpl<CacheState<Arc>>::PushArc;
-  using CacheBaseImpl<CacheState<Arc>>::HasStart;
-  using CacheBaseImpl<CacheState<Arc>>::HasFinal;
+  using CacheBaseImpl<CacheState<Arc>>::EmplaceArc;
   using CacheBaseImpl<CacheState<Arc>>::HasArcs;
+  using CacheBaseImpl<CacheState<Arc>>::HasFinal;
+  using CacheBaseImpl<CacheState<Arc>>::HasStart;
   using CacheBaseImpl<CacheState<Arc>>::SetArcs;
   using CacheBaseImpl<CacheState<Arc>>::SetFinal;
   using CacheBaseImpl<CacheState<Arc>>::SetStart;
@@ -260,11 +296,10 @@ class FactorWeightFstImpl : public CacheImpl<Arc> {
   Weight Final(StateId s) {
     if (!HasFinal(s)) {
       const auto &element = elements_[s];
-      // TODO(sorenj): fix so cast is unnecessary
       const auto weight =
           element.state == kNoStateId
               ? element.weight
-              : (Weight)Times(element.weight, fst_->Final(element.state));
+              : Times(element.weight, fst_->Final(element.state));
       FactorIterator siter(weight);
       if (!(mode_ & kFactorFinalWeights) || siter.Done()) {
         SetFinal(s, weight);
@@ -319,7 +354,7 @@ class FactorWeightFstImpl : public CacheImpl<Arc> {
       return unfactored_[element.state];
     } else {
       const auto insert_result =
-          element_map_.insert(std::make_pair(element, elements_.size()));
+          element_map_.emplace(element, elements_.size());
       if (insert_result.second) {
         elements_.push_back(element);
       }
@@ -335,17 +370,17 @@ class FactorWeightFstImpl : public CacheImpl<Arc> {
       for (ArcIterator<Fst<Arc>> ait(*fst_, element.state); !ait.Done();
            ait.Next()) {
         const auto &arc = ait.Value();
-        const auto weight = Times(element.weight, arc.weight);
+        auto weight = Times(element.weight, arc.weight);
         FactorIterator fiter(weight);
         if (!(mode_ & kFactorArcWeights) || fiter.Done()) {
           const auto dest = FindState(Element(arc.nextstate, Weight::One()));
-          PushArc(s, Arc(arc.ilabel, arc.olabel, weight, dest));
+          EmplaceArc(s, arc.ilabel, arc.olabel, std::move(weight), dest);
         } else {
           for (; !fiter.Done(); fiter.Next()) {
-            const auto &pair = fiter.Value();
+            auto pair = fiter.Value();
             const auto dest =
                 FindState(Element(arc.nextstate, pair.second.Quantize(delta_)));
-            PushArc(s, Arc(arc.ilabel, arc.olabel, pair.first, dest));
+            EmplaceArc(s, arc.ilabel, arc.olabel, std::move(pair.first), dest);
           }
         }
       }
@@ -360,10 +395,10 @@ class FactorWeightFstImpl : public CacheImpl<Arc> {
       auto ilabel = final_ilabel_;
       auto olabel = final_olabel_;
       for (FactorIterator fiter(weight); !fiter.Done(); fiter.Next()) {
-        const auto &pair = fiter.Value();
+        auto pair = fiter.Value();
         const auto dest =
             FindState(Element(kNoStateId, pair.second.Quantize(delta_)));
-        PushArc(s, Arc(ilabel, olabel, pair.first, dest));
+        EmplaceArc(s, ilabel, olabel, std::move(pair.first), dest);
         if (increment_final_ilabel_) ++ilabel;
         if (increment_final_olabel_) ++olabel;
       }
@@ -394,13 +429,13 @@ class FactorWeightFstImpl : public CacheImpl<Arc> {
 
   std::unique_ptr<const Fst<Arc>> fst_;
   float delta_;
-  uint32 mode_;         // Factoring arc and/or final weights.
+  uint8 mode_;          // Factoring arc and/or final weights.
   Label final_ilabel_;  // ilabel of arc created when factoring final weights.
   Label final_olabel_;  // olabel of arc created when factoring final weights.
   bool increment_final_ilabel_;    // When factoring final weights results in
   bool increment_final_olabel_;    // mutiple arcs, increment labels?
-  std::vector<Element> elements_;  // mapping from FST state to Element.
-  ElementMap element_map_;         // mapping from Element to FST state.
+  std::vector<Element> elements_;  // Mapping from FST state to Element.
+  ElementMap element_map_;         // Mapping from Element to FST state.
   // Mapping between old/new StateId for states that do not need to be factored
   // when mode_ is 0 or kFactorFinalWeights.
   std::vector<StateId> unfactored_;
@@ -440,12 +475,12 @@ class FactorWeightFst
       : ImplToFst<Impl>(std::make_shared<Impl>(fst, opts)) {}
 
   // See Fst<>::Copy() for doc.
-  FactorWeightFst(const FactorWeightFst<Arc, FactorIterator> &fst, bool copy)
+  FactorWeightFst(const FactorWeightFst &fst, bool copy)
       : ImplToFst<Impl>(fst, copy) {}
 
   // Get a copy of this FactorWeightFst. See Fst<>::Copy() for further doc.
-  FactorWeightFst<Arc, FactorIterator> *Copy(bool copy = false) const override {
-    return new FactorWeightFst<Arc, FactorIterator>(*this, copy);
+  FactorWeightFst *Copy(bool copy = false) const override {
+    return new FactorWeightFst(*this, copy);
   }
 
   inline void InitStateIterator(StateIteratorData<Arc> *data) const override;
@@ -488,7 +523,9 @@ class ArcIterator<FactorWeightFst<Arc, FactorIterator>>
 template <class Arc, class FactorIterator>
 inline void FactorWeightFst<Arc, FactorIterator>::InitStateIterator(
     StateIteratorData<Arc> *data) const {
-  data->base = new StateIterator<FactorWeightFst<Arc, FactorIterator>>(*this);
+  data->base =
+      std::make_unique<StateIterator<FactorWeightFst<Arc, FactorIterator>>>(
+          *this);
 }
 
 }  // namespace fst
