@@ -1,3 +1,17 @@
+// Copyright 2005-2020 Google LLC
+//
+// Licensed under the Apache License, Version 2.0 (the 'License');
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+//
+//     http://www.apache.org/licenses/LICENSE-2.0
+//
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an 'AS IS' BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
+//
 // See www.openfst.org for extensive documentation on this weighted
 // finite-state transducer library.
 //
@@ -9,9 +23,9 @@
 
 #include <sys/types.h>
 
+#include <atomic>
 #include <cmath>
 #include <cstddef>
-
 #include <iostream>
 #include <memory>
 #include <sstream>
@@ -20,6 +34,7 @@
 
 #include <fst/compat.h>
 #include <fst/flags.h>
+#include <fst/types.h>
 #include <fst/log.h>
 #include <fstream>
 
@@ -29,13 +44,15 @@
 #include <fst/register.h>
 #include <fst/symbol-table.h>
 #include <fst/util.h>
+#include <string_view>
 
 
 DECLARE_bool(fst_align);
 
 namespace fst {
 
-bool IsFstHeader(std::istream &, const string &);
+// Identifies stream data as an FST (and its endianity).
+constexpr int32 kFstMagicNumber = 2125659606;
 
 class FstHeader;
 
@@ -54,7 +71,7 @@ struct FstReadOptions {
   // a warning indicating why it was chosen.
   enum FileReadMode { READ, MAP };
 
-  string source;                // Where you're reading from.
+  std::string source;           // Where you're reading from.
   const FstHeader *header;      // Pointer to FST header; if non-zero, use
                                 // this info (don't read a stream header).
   const SymbolTable *isymbols;  // Pointer to input symbols; if non-zero, use
@@ -65,33 +82,34 @@ struct FstReadOptions {
   bool read_isymbols;           // Read isymbols, if any (default: true).
   bool read_osymbols;           // Read osymbols, if any (default: true).
 
-  explicit FstReadOptions(const string &source = "<unspecified>",
+  explicit FstReadOptions(const std::string_view source = "<unspecified>",
                           const FstHeader *header = nullptr,
                           const SymbolTable *isymbols = nullptr,
                           const SymbolTable *osymbols = nullptr);
 
-  explicit FstReadOptions(const string &source, const SymbolTable *isymbols,
+  explicit FstReadOptions(const std::string_view source,
+                          const SymbolTable *isymbols,
                           const SymbolTable *osymbols = nullptr);
 
   // Helper function to convert strings FileReadModes into their enum value.
-  static FileReadMode ReadMode(const string &mode);
+  static FileReadMode ReadMode(const std::string &mode);
 
   // Outputs a debug string for the FstReadOptions object.
-  string DebugString() const;
+  std::string DebugString() const;
 };
 
 struct FstWriteOptions {
-  string source;        // Where you're writing to.
+  std::string source;   // Where you're writing to.
   bool write_header;    // Write the header?
   bool write_isymbols;  // Write input symbols?
   bool write_osymbols;  // Write output symbols?
   bool align;           // Write data aligned (may fail on pipes)?
   bool stream_write;    // Avoid seek operations in writing.
 
-  explicit FstWriteOptions(const string &source = "<unspecifed>",
+  explicit FstWriteOptions(std::string_view source = "<unspecified>",
                            bool write_header = true, bool write_isymbols = true,
                            bool write_osymbols = true,
-                           bool align = FLAGS_fst_align,
+                           bool align = FST_FLAGS_fst_align,
                            bool stream_write = false)
       : source(source),
         write_header(write_header),
@@ -107,22 +125,27 @@ struct FstWriteOptions {
 
 class FstHeader {
  public:
-  enum {
+  enum Flags {
     HAS_ISYMBOLS = 0x1,  // Has input symbol table.
     HAS_OSYMBOLS = 0x2,  // Has output symbol table.
     IS_ALIGNED = 0x4,    // Memory-aligned (where appropriate).
-  } Flags;
+  };
 
-  FstHeader() : version_(0), flags_(0), properties_(0), start_(-1),
-      numstates_(0), numarcs_(0) {}
+  FstHeader()
+      : version_(0),
+        flags_(0),
+        properties_(0),
+        start_(-1),
+        numstates_(0),
+        numarcs_(0) {}
 
-  const string &FstType() const { return fsttype_; }
+  const std::string &FstType() const { return fsttype_; }
 
-  const string &ArcType() const { return arctype_; }
+  const std::string &ArcType() const { return arctype_; }
 
   int32 Version() const { return version_; }
 
-  int32 GetFlags() const { return flags_; }
+  uint32 GetFlags() const { return flags_; }
 
   uint64 Properties() const { return properties_; }
 
@@ -132,13 +155,13 @@ class FstHeader {
 
   int64 NumArcs() const { return numarcs_; }
 
-  void SetFstType(const string &type) { fsttype_ = type; }
+  void SetFstType(const std::string &type) { fsttype_ = type; }
 
-  void SetArcType(const string &type) { arctype_ = type; }
+  void SetArcType(const std::string &type) { arctype_ = type; }
 
   void SetVersion(int32 version) { version_ = version; }
 
-  void SetFlags(int32 flags) { flags_ = flags; }
+  void SetFlags(uint32 flags) { flags_ = flags; }
 
   void SetProperties(uint64 properties) { properties_ = properties; }
 
@@ -148,23 +171,22 @@ class FstHeader {
 
   void SetNumArcs(int64 numarcs) { numarcs_ = numarcs; }
 
-  bool Read(std::istream &strm, const string &source,
-            bool rewind = false);
+  bool Read(std::istream &strm, const std::string &source, bool rewind = false);
 
-  bool Write(std::ostream &strm, const string &source) const;
+  bool Write(std::ostream &strm, const std::string &source) const;
 
   // Outputs a debug string for the FstHeader object.
-  string DebugString() const;
+  std::string DebugString() const;
 
  private:
-  string fsttype_;     // E.g. "vector".
-  string arctype_;     // E.g. "standard".
-  int32 version_;      // Type version number.
-  int32 flags_;        // File format bits.
-  uint64 properties_;  // FST property bits.
-  int64 start_;        // Start state.
-  int64 numstates_;    // # of states.
-  int64 numarcs_;      // # of arcs.
+  std::string fsttype_;  // E.g. "vector".
+  std::string arctype_;  // E.g. "standard".
+  int32 version_;        // Type version number.
+  uint32 flags_;         // File format bits.
+  uint64 properties_;    // FST property bits.
+  int64 start_;          // Start state.
+  int64 numstates_;      // # of states.
+  int64 numarcs_;        // # of arcs.
 };
 
 // Specifies matcher action.
@@ -181,7 +203,8 @@ constexpr int kNoStateId = -1;  // Not a valid state ID.
 
 // A generic FST, templated on the arc definition, with common-demoninator
 // methods (use StateIterator and ArcIterator to iterate over its states and
-// arcs).
+// arcs). Derived classes should be assumed to be thread-unsafe unless
+// otherwise specified.
 template <class A>
 class Fst {
  public:
@@ -212,7 +235,7 @@ class Fst {
   virtual uint64 Properties(uint64 mask, bool test) const = 0;
 
   // FST type name.
-  virtual const string &Type() const = 0;
+  virtual const std::string &Type() const = 0;
 
   // Gets a copy of this Fst. The copying behaves as follows:
   //
@@ -228,10 +251,10 @@ class Fst {
   // (3) If a MutableFst is copied and then mutated, then the original is
   // unmodified and vice versa (often by a copy-on-write on the initial
   // mutation, which may not be constant time).
-  virtual Fst<Arc> *Copy(bool safe = false) const = 0;
+  virtual Fst *Copy(bool safe = false) const = 0;
 
   // Reads an FST from an input stream; returns nullptr on error.
-  static Fst<Arc> *Read(std::istream &strm, const FstReadOptions &opts) {
+  static Fst *Read(std::istream &strm, const FstReadOptions &opts) {
     FstReadOptions ropts(opts);
     FstHeader hdr;
     if (ropts.header) {
@@ -250,17 +273,17 @@ class Fst {
     return reader(strm, ropts);
   }
 
-  // Reads an FST from a file; returns nullptr on error. An empty filename
+  // Reads an FST from a file; returns nullptr on error. An empty source
   // results in reading from standard input.
-  static Fst<Arc> *Read(const string &filename) {
-    if (!filename.empty()) {
-      std::ifstream strm(filename,
+  static Fst *Read(const std::string &source) {
+    if (!source.empty()) {
+      std::ifstream strm(source,
                               std::ios_base::in | std::ios_base::binary);
       if (!strm) {
-        LOG(ERROR) << "Fst::Read: Can't open file: " << filename;
+        LOG(ERROR) << "Fst::Read: Can't open file: " << source;
         return nullptr;
       }
-      return Read(strm, FstReadOptions(filename));
+      return Read(strm, FstReadOptions(source));
     } else {
       return Read(std::cin, FstReadOptions("standard input"));
     }
@@ -273,14 +296,20 @@ class Fst {
     return false;
   }
 
-  // Writes an FST to a file; returns false on error; an empty filename
+  // Writes an FST to a file; returns false on error; an empty source
   // results in writing to standard output.
-  virtual bool Write(const string &filename) const {
-    LOG(ERROR) << "Fst::Write: No write filename method for " << Type()
+  virtual bool Write(const std::string &source) const {
+    LOG(ERROR) << "Fst::Write: No write source method for " << Type()
                << " FST type";
     return false;
   }
 
+  // Some Fst implementations support
+  //   template <class Fst2>
+  //   static bool Fst1::WriteFst(const Fst2 &fst2, ...);
+  // which is equivalent to Fst1(fst2).Write(...), but uses less memory.
+  // WriteFst is not part of the general Fst interface.
+
   // Returns input label symbol table; return nullptr if not specified.
   virtual const SymbolTable *InputSymbols() const = 0;
 
@@ -300,17 +329,19 @@ class Fst {
   virtual MatcherBase<Arc> *InitMatcher(MatchType match_type) const;
 
  protected:
-  bool WriteFile(const string &filename) const {
-    if (!filename.empty()) {
-      std::ofstream strm(filename,
+  bool WriteFile(const std::string &source) const {
+    if (!source.empty()) {
+      std::ofstream strm(source,
                                std::ios_base::out | std::ios_base::binary);
       if (!strm) {
-        LOG(ERROR) << "Fst::Write: Can't open file: " << filename;
+        LOG(ERROR) << "Fst::WriteFile: Can't open file: " << source;
         return false;
       }
-      bool val = Write(strm, FstWriteOptions(filename));
-      if (!val) LOG(ERROR) << "Fst::Write failed: " << filename;
-      return val;
+      if (!Write(strm, FstWriteOptions(source))) {
+        LOG(ERROR) << "Fst::WriteFile: Write failed: " << source;
+        return false;
+      }
+      return true;
     } else {
       return Write(std::cout, FstWriteOptions("standard output"));
     }
@@ -347,8 +378,8 @@ template <class Arc>
 struct StateIteratorData {
   using StateId = typename Arc::StateId;
 
-  // Specialized iterator if non-zero.
-  StateIteratorBase<Arc> *base;
+  // Specialized iterator if non-null.
+  std::unique_ptr<StateIteratorBase<Arc>> base;
   // Otherwise, the total number of states.
   StateId nstates;
 
@@ -368,6 +399,9 @@ struct StateIteratorData {
 //     ...
 //   }
 // There is no copying of the FST.
+//
+// Specializations may exist for some FST types.
+// StateIterators are thread-unsafe unless otherwise specified.
 template <class FST>
 class StateIterator {
  public:
@@ -378,8 +412,6 @@ class StateIterator {
     fst.InitStateIterator(&data_);
   }
 
-  ~StateIterator() { delete data_.base; }
-
   bool Done() const {
     return data_.base ? data_.base->Done() : s_ >= data_.nstates;
   }
@@ -407,19 +439,20 @@ class StateIterator {
   StateId s_;
 };
 
-// Flags to control the behavior on an arc iterator.
-static constexpr uint32 kArcILabelValue =
-    0x0001;  // Value() gives valid ilabel.
-static constexpr uint32 kArcOLabelValue = 0x0002;  //  "       "     " olabel.
-static constexpr uint32 kArcWeightValue = 0x0004;  //  "       "     " weight.
-static constexpr uint32 kArcNextStateValue =
-    0x0008;                                    //  "       "     " nextstate.
-static constexpr uint32 kArcNoCache = 0x0010;  // No need to cache arcs.
-
-static constexpr uint32 kArcValueFlags =
+// Flags to control the behavior on an arc iterator via SetFlags().
+// Value() gives valid ilabel.
+static constexpr uint8 kArcILabelValue = 0x01;
+// Value() call gives valid olabel.
+static constexpr uint8 kArcOLabelValue = 0x02;
+// Value() call gives valid weight.
+static constexpr uint8 kArcWeightValue = 0x04;
+// Value() call gives valid nextstate.
+static constexpr uint8 kArcNextStateValue = 0x08;
+// Arcs need not be cached.
+static constexpr uint8 kArcNoCache = 0x10;
+static constexpr uint8 kArcValueFlags =
     kArcILabelValue | kArcOLabelValue | kArcWeightValue | kArcNextStateValue;
-
-static constexpr uint32 kArcFlags = kArcValueFlags | kArcNoCache;
+static constexpr uint8 kArcFlags = kArcValueFlags | kArcNoCache;
 
 // Arc iterator interface, templated on the arc definition; used for arc
 // iterator specializations that are returned by the InitArcIterator FST method.
@@ -442,10 +475,10 @@ class ArcIteratorBase {
   virtual void Reset() = 0;
   // Advances to arbitrary arc by position.
   virtual void Seek(size_t) = 0;
-  // Returns current behavorial flags
-  virtual uint32 Flags() const = 0;
-  // Sets behavorial flags.
-  virtual void SetFlags(uint32, uint32) = 0;
+  // Returns current behavorial flags, a bitmask of kArcFlags.
+  virtual uint8 Flags() const = 0;
+  // Sets behavorial flags, a bitmask of kArcFlags.
+  virtual void SetFlags(uint8, uint8) = 0;
 };
 
 // ArcIterator initialization data.
@@ -458,10 +491,12 @@ struct ArcIteratorData {
 
   ArcIteratorData &operator=(const ArcIteratorData &) = delete;
 
-  ArcIteratorBase<Arc> *base;  // Specialized iterator if non-zero.
-  const Arc *arcs;             // O.w. arcs pointer
-  size_t narcs;                // ... and arc count.
-  int *ref_count;              // ... and reference count if non-zero.
+  std::unique_ptr<ArcIteratorBase<Arc>>
+      base;         // Specialized iterator if non-null.
+  const Arc *arcs;  // O.w. arcs pointer
+  size_t narcs;     // ... and arc count.
+  int *ref_count;   // ... and a reference count of the `narcs`-length `arcs`
+                    //     array if non-null.
 };
 
 // Generic arc iterator, templated on the FST definition (a wrapper around a
@@ -474,6 +509,9 @@ struct ArcIteratorData {
 //     ...
 //   }
 // There is no copying of the FST.
+//
+// Specializations may exist for some FST types.
+// ArcIterators are thread-unsafe unless otherwise specified.
 template <class FST>
 class ArcIterator {
  public:
@@ -484,14 +522,10 @@ class ArcIterator {
     fst.InitArcIterator(s, &data_);
   }
 
-  explicit ArcIterator(const ArcIteratorData<Arc> &data) : data_(data), i_(0) {
-    if (data_.ref_count) ++(*data_.ref_count);
-  }
+  explicit ArcIterator(const ArcIteratorData<Arc> &data) = delete;
 
   ~ArcIterator() {
-    if (data_.base) {
-      delete data_.base;
-    } else if (data_.ref_count) {
+    if (data_.ref_count) {
       --(*data_.ref_count);
     }
   }
@@ -530,15 +564,11 @@ class ArcIterator {
 
   size_t Position() const { return data_.base ? data_.base->Position() : i_; }
 
-  uint32 Flags() const {
-    if (data_.base) {
-      return data_.base->Flags();
-    } else {
-      return kArcValueFlags;
-    }
+  uint8 Flags() const {
+    return data_.base ? data_.base->Flags() : kArcValueFlags;
   }
 
-  void SetFlags(uint32 flags, uint32 mask) {
+  void SetFlags(uint8 flags, uint8 mask) {
     if (data_.base) data_.base->SetFlags(flags, mask);
   }
 
@@ -631,6 +661,11 @@ inline size_t NumOutputEpsilons(const Fst<Arc> &fst, typename Arc::StateId s) {
 //
 // Users are discouraged, but not prohibited, from subclassing this outside the
 // FST library.
+//
+// This class is thread-compatible except for the const SetProperties
+// overload. Derived classes should be assumed to be thread-unsafe unless
+// otherwise specified. Derived-class copy constructors must produce a
+// thread-safe copy.
 template <class Arc>
 class FstImpl {
  public:
@@ -640,29 +675,53 @@ class FstImpl {
   FstImpl() : properties_(0), type_("null") {}
 
   FstImpl(const FstImpl<Arc> &impl)
-      : properties_(impl.properties_),
+      : properties_(impl.properties_.load(std::memory_order_relaxed)),
         type_(impl.type_),
         isymbols_(impl.isymbols_ ? impl.isymbols_->Copy() : nullptr),
         osymbols_(impl.osymbols_ ? impl.osymbols_->Copy() : nullptr) {}
 
+  FstImpl(FstImpl<Arc> &&impl) noexcept;
+
   virtual ~FstImpl() {}
 
-  const string &Type() const { return type_; }
+  FstImpl &operator=(const FstImpl &impl) {
+    properties_.store(impl.properties_.load(std::memory_order_relaxed),
+                      std::memory_order_relaxed);
+    type_ = impl.type_;
+    isymbols_ = impl.isymbols_ ? impl.isymbols_->Copy() : nullptr;
+    osymbols_ = impl.osymbols_ ? impl.osymbols_->Copy() : nullptr;
+    return *this;
+  }
 
-  void SetType(const string &type) { type_ = type; }
+  FstImpl &operator=(FstImpl &&impl) noexcept;
 
-  virtual uint64 Properties() const { return properties_; }
+  const std::string &Type() const { return type_; }
 
-  virtual uint64 Properties(uint64 mask) const { return properties_ & mask; }
+  void SetType(const std::string &type) { type_ = type; }
+
+  virtual uint64 Properties() const {
+    return properties_.load(std::memory_order_relaxed);
+  }
+
+  virtual uint64 Properties(uint64 mask) const {
+    return properties_.load(std::memory_order_relaxed) & mask;
+  }
 
   void SetProperties(uint64 props) {
-    properties_ &= kError;  // kError can't be cleared.
-    properties_ |= props;
+    uint64 properties = properties_.load(std::memory_order_relaxed);
+    properties &= kError;  // kError can't be cleared.
+    properties |= props;
+    properties_.store(properties, std::memory_order_relaxed);
   }
 
   void SetProperties(uint64 props, uint64 mask) {
-    properties_ &= ~mask | kError;  // kError can't be cleared.
-    properties_ |= props & mask;
+    // Unlike UpdateProperties, does not require compatibility between props
+    // and properties_, since it may be used to update properties after
+    // a mutation.
+    uint64 properties = properties_.load(std::memory_order_relaxed);
+    properties &= ~mask | kError;  // kError can't be cleared.
+    properties |= props & mask;
+    properties_.store(properties, std::memory_order_relaxed);
   }
 
   // Allows (only) setting error bit on const FST implementations.
@@ -670,7 +729,32 @@ class FstImpl {
     if (mask != kError) {
       FSTERROR() << "FstImpl::SetProperties() const: Can only set kError";
     }
-    properties_ |= kError;
+    properties_.fetch_or(kError, std::memory_order_relaxed);
+  }
+
+  // Sets the subset of the properties that have changed, in a thread-safe
+  // manner via atomic bitwise-or..
+  void UpdateProperties(uint64 props, uint64 mask) {
+    // If properties_ and props are compatible (for example kAcceptor and
+    // kNoAcceptor cannot both be set), the props can be or-ed in.
+    // Compatibility is ensured if props comes from ComputeProperties
+    // and properties_ is set correctly initially. However
+    // relying on properties to be set correctly is too large an
+    // assumption, as many places set them incorrectly.
+    // Therefore, we or in only the newly discovered properties.
+    // These cannot become inconsistent, but this means that
+    // incorrectly set properties will remain incorrect.
+    const uint64 properties = properties_.load(std::memory_order_relaxed);
+    DCHECK(internal::CompatProperties(properties, props));
+    const uint64 old_props = properties & mask;
+    const uint64 old_mask = internal::KnownProperties(old_props);
+    const uint64 discovered_mask = mask & ~old_mask;
+    const uint64 discovered_props = props & discovered_mask;
+    // It is always correct to or these bits in, but do this only when
+    // necessary to avoid extra stores and possible cache flushes.
+    if (discovered_props != 0) {
+      properties_.fetch_or(discovered_props, std::memory_order_relaxed);
+    }
   }
 
   const SymbolTable *InputSymbols() const { return isymbols_.get(); }
@@ -699,13 +783,13 @@ class FstImpl {
   // Writes header and symbols to output stream. If opts.header is false, skips
   // writing header. If opts.[io]symbols is false, skips writing those symbols.
   // This method is needed for implementations that implement Write methods.
-  void WriteHeader(std::ostream &strm, const FstWriteOptions &opts,
-                   int version, FstHeader *hdr) const {
+  void WriteHeader(std::ostream &strm, const FstWriteOptions &opts, int version,
+                   FstHeader *hdr) const {
     if (opts.write_header) {
       hdr->SetFstType(type_);
       hdr->SetArcType(Arc::Type());
       hdr->SetVersion(version);
-      hdr->SetProperties(properties_);
+      hdr->SetProperties(properties_.load(std::memory_order_relaxed));
       int32 file_flags = 0;
       if (isymbols_ && opts.write_isymbols) {
         file_flags |= FstHeader::HAS_ISYMBOLS;
@@ -727,7 +811,7 @@ class FstImpl {
   // cross-type serialization methods Fst::WriteFst.
   static void WriteFstHeader(const Fst<Arc> &fst, std::ostream &strm,
                              const FstWriteOptions &opts, int version,
-                             const string &type, uint64 properties,
+                             const std::string &type, uint64 properties,
                              FstHeader *hdr) {
     if (opts.write_header) {
       hdr->SetFstType(type);
@@ -760,7 +844,7 @@ class FstImpl {
   // success, false on failure.
   static bool UpdateFstHeader(const Fst<Arc> &fst, std::ostream &strm,
                               const FstWriteOptions &opts, int version,
-                              const string &type, uint64 properties,
+                              const std::string &type, uint64 properties,
                               FstHeader *hdr, size_t header_offset) {
     strm.seekp(header_offset);
     if (!strm) {
@@ -781,14 +865,24 @@ class FstImpl {
   }
 
  protected:
-  mutable uint64 properties_;  // Property bits.
+  // Use atomic so that UpdateProperties() can be thread-safe.
+  // This is always used with memory_order_relaxed because it's only used
+  // as a cache and not used to synchronize other operations.
+  mutable std::atomic<uint64> properties_;  // Property bits.
 
  private:
-  string type_;  // Unique name of FST class.
+  std::string type_;  // Unique name of FST class.
   std::unique_ptr<SymbolTable> isymbols_;
   std::unique_ptr<SymbolTable> osymbols_;
 };
 
+template <class Arc>
+inline FstImpl<Arc>::FstImpl(FstImpl<Arc> &&) noexcept = default;
+
+template <class Arc>
+inline FstImpl<Arc> &FstImpl<Arc>::operator=(FstImpl<Arc> &&) noexcept =
+    default;
+
 template <class Arc>
 bool FstImpl<Arc>::ReadHeader(std::istream &strm, const FstReadOptions &opts,
                               int min_version, FstHeader *hdr) {
@@ -797,29 +891,26 @@ bool FstImpl<Arc>::ReadHeader(std::istream &strm, const FstReadOptions &opts,
   } else if (!hdr->Read(strm, opts.source)) {
     return false;
   }
-  if (FLAGS_v >= 2) {
-    LOG(INFO) << "FstImpl::ReadHeader: source: " << opts.source
-              << ", fst_type: " << hdr->FstType()
-              << ", arc_type: " << Arc::Type()
-              << ", version: " << hdr->Version()
-              << ", flags: " << hdr->GetFlags();
-  }
+  VLOG(2) << "FstImpl::ReadHeader: source: " << opts.source
+          << ", fst_type: " << hdr->FstType() << ", arc_type: " << Arc::Type()
+          << ", version: " << hdr->Version() << ", flags: " << hdr->GetFlags();
   if (hdr->FstType() != type_) {
-    LOG(ERROR) << "FstImpl::ReadHeader: FST not of type " << type_
-               << ": " << opts.source;
+    LOG(ERROR) << "FstImpl::ReadHeader: FST not of type " << type_ << ", found "
+               << hdr->FstType() << ": " << opts.source;
     return false;
   }
   if (hdr->ArcType() != Arc::Type()) {
     LOG(ERROR) << "FstImpl::ReadHeader: Arc not of type " << Arc::Type()
-               << ": " << opts.source;
+               << ", found " << hdr->ArcType() << ": " << opts.source;
     return false;
   }
   if (hdr->Version() < min_version) {
-    LOG(ERROR) << "FstImpl::ReadHeader: Obsolete " << type_
-               << " FST version: " << opts.source;
+    LOG(ERROR) << "FstImpl::ReadHeader: Obsolete " << type_ << " FST version "
+               << hdr->Version() << ", min_version=" << min_version << ": "
+               << opts.source;
     return false;
   }
-  properties_ = hdr->Properties();
+  properties_.store(hdr->Properties(), std::memory_order_relaxed);
   if (hdr->GetFlags() & FstHeader::HAS_ISYMBOLS) {
     isymbols_.reset(SymbolTable::Read(strm, opts.source));
   }
@@ -839,20 +930,22 @@ bool FstImpl<Arc>::ReadHeader(std::istream &strm, const FstReadOptions &opts,
   return true;
 }
 
-}  // namespace internal
-
 template <class Arc>
 uint64 TestProperties(const Fst<Arc> &fst, uint64 mask, uint64 *known);
 
+}  // namespace internal
+
 // This is a helper class template useful for attaching an FST interface to
 // its implementation, handling reference counting.
+// Thread-unsafe due to Properties (a const function) calling
+// Impl::SetProperties. TODO(jrosenstock): Make thread-compatible.
+// Impl's copy constructor must produce a thread-safe copy.
 template <class Impl, class FST = Fst<typename Impl::Arc>>
 class ImplToFst : public FST {
  public:
   using Arc = typename Impl::Arc;
   using StateId = typename Arc::StateId;
   using Weight = typename Arc::Weight;
-  using FST::operator=;
 
   StateId Start() const override { return impl_->Start(); }
 
@@ -868,17 +961,22 @@ class ImplToFst : public FST {
     return impl_->NumOutputEpsilons(s);
   }
 
+  // Note that this is a const function, but can set the Impl's properties
+  // when test is true.
   uint64 Properties(uint64 mask, bool test) const override {
     if (test) {
-      uint64 knownprops, testprops = TestProperties(*this, mask, &knownprops);
-      impl_->SetProperties(testprops, knownprops);
+      uint64 knownprops,
+          testprops = internal::TestProperties(*this, mask, &knownprops);
+      // Properties is a const member function, but can set the cached
+      // properties. UpdateProperties does this thread-safely via atomics.
+      impl_->UpdateProperties(testprops, knownprops);
       return testprops & mask;
     } else {
       return impl_->Properties(mask);
     }
   }
 
-  const string &Type() const override { return impl_->Type(); }
+  const std::string &Type() const override { return impl_->Type(); }
 
   const SymbolTable *InputSymbols() const override {
     return impl_->InputSymbols();
@@ -891,9 +989,11 @@ class ImplToFst : public FST {
  protected:
   explicit ImplToFst(std::shared_ptr<Impl> impl) : impl_(std::move(impl)) {}
 
+  // The object is thread-compatible if constructed with safe=true,
+  // otherwise thread-unsafe.
   // This constructor presumes there is a copy constructor for the
-  // implementation.
-  ImplToFst(const ImplToFst<Impl, FST> &fst, bool safe) {
+  // implementation that produces a thread-safe copy.
+  ImplToFst(const ImplToFst &fst, bool safe) {
     if (safe) {
       impl_ = std::make_shared<Impl>(*(fst.impl_));
     } else {
@@ -901,6 +1001,27 @@ class ImplToFst : public FST {
     }
   }
 
+  ImplToFst() = delete;
+
+  ImplToFst(const ImplToFst &fst) : impl_(fst.impl_) {}
+
+  ImplToFst(ImplToFst &&fst) noexcept : impl_(std::move(fst.impl_)) {
+    fst.impl_ = std::make_shared<Impl>();
+  }
+
+  ImplToFst &operator=(const ImplToFst &fst) {
+    impl_ = fst.impl_;
+    return *this;
+  }
+
+  ImplToFst &operator=(ImplToFst &&fst) noexcept {
+    if (this != &fst) {
+      impl_ = std::move(fst.impl_);
+      fst.impl_ = std::make_shared<Impl>();
+    }
+    return *this;
+  }
+
   // Returns raw pointers to the shared object.
   const Impl *GetImpl() const { return impl_.get(); }
 
@@ -911,7 +1032,7 @@ class ImplToFst : public FST {
 
   bool Unique() const { return impl_.unique(); }
 
-  void SetImpl(std::shared_ptr<Impl> impl) { impl_ = impl; }
+  void SetImpl(std::shared_ptr<Impl> impl) { impl_ = std::move(impl); }
 
  private:
   template <class IFST, class OFST>
@@ -930,34 +1051,34 @@ class ImplToFst : public FST {
 template <class IFST, class OFST>
 void Cast(const IFST &ifst, OFST *ofst) {
   using OImpl = typename OFST::Impl;
-  ofst->impl_ = std::shared_ptr<OImpl>(ifst.impl_,
-      reinterpret_cast<OImpl *>(ifst.impl_.get()));
+  ofst->impl_ = std::shared_ptr<OImpl>(
+      ifst.impl_, reinterpret_cast<OImpl *>(ifst.impl_.get()));
 }
 
 // FST serialization.
 
 template <class Arc>
-string FstToString(const Fst<Arc> &fst,
-                   const FstWriteOptions &options =
-                       FstWriteOptions("FstToString")) {
+std::string FstToString(
+    const Fst<Arc> &fst,
+    const FstWriteOptions &options = FstWriteOptions("FstToString")) {
   std::ostringstream ostrm;
   fst.Write(ostrm, options);
   return ostrm.str();
 }
 
 template <class Arc>
-void FstToString(const Fst<Arc> &fst, string *result) {
+void FstToString(const Fst<Arc> &fst, std::string *result) {
   *result = FstToString(fst);
 }
 
 template <class Arc>
-void FstToString(const Fst<Arc> &fst, string *result,
+void FstToString(const Fst<Arc> &fst, std::string *result,
                  const FstWriteOptions &options) {
   *result = FstToString(fst, options);
 }
 
 template <class Arc>
-Fst<Arc> *StringToFst(const string &s) {
+Fst<Arc> *StringToFst(const std::string &s) {
   std::istringstream istrm(s);
   return Fst<Arc>::Read(istrm, FstReadOptions("StringToFst"));
 }
