@@ -1,3 +1,17 @@
+// Copyright 2005-2020 Google LLC
+//
+// Licensed under the Apache License, Version 2.0 (the 'License');
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+//
+//     http://www.apache.org/licenses/LICENSE-2.0
+//
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an 'AS IS' BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
+//
 // See www.openfst.org for extensive documentation on this weighted
 // finite-state transducer library.
 //
@@ -7,6 +21,7 @@
 #define FST_INTERVAL_SET_H_
 
 #include <algorithm>
+#include <initializer_list>
 #include <iostream>
 #include <vector>
 
@@ -65,6 +80,8 @@ class VectorIntervalStore {
   using Iterator = typename std::vector<Interval>::const_iterator;
 
   VectorIntervalStore() : count_(-1) {}
+  VectorIntervalStore(std::initializer_list<Interval> intervals_init)
+      : intervals_(intervals_init), count_(-1) {}
 
   std::vector<Interval> *MutableIntervals() { return &intervals_; }
 
@@ -107,6 +124,9 @@ class IntervalSet {
  public:
   using Interval = IntInterval<T>;
 
+  IntervalSet(std::initializer_list<Interval> intervals_init)
+      : intervals_(intervals_init) {}
+
   template <class... A>
   explicit IntervalSet(A... args) : intervals_(args...) {}
 
