@@ -1,3 +1,17 @@
+// Copyright 2005-2020 Google LLC
+//
+// Licensed under the Apache License, Version 2.0 (the 'License');
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+//
+//     http://www.apache.org/licenses/LICENSE-2.0
+//
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an 'AS IS' BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
+//
 // See www.openfst.org for extensive documentation on this weighted
 // finite-state transducer library.
 //
@@ -9,6 +23,7 @@
 
 #include <vector>
 
+#include <fst/types.h>
 #include <fst/log.h>
 
 #include <fst/filter-state.h>
@@ -308,7 +323,7 @@ class PushWeightsComposeFilter {
     if (!(LookAheadFlags() & kLookAheadWeight)) {
       return FilterState(fs1, FilterState2(Weight::One()));
     }
-    const auto &lweight = filter_.LookAheadArc()
+    const auto &lweight = LookAheadArc()
                               ? Selector().GetMatcher()->LookAheadWeight()
                               : Weight::One();
     const auto &fs2 = fs_.GetState2();
@@ -438,8 +453,7 @@ class PushLabelsComposeFilter {
     }
     const auto &fs1 = filter_.FilterArc(arc1, arc2);
     if (fs1 == FilterState1::NoState()) return FilterState::NoState();
-    if (!filter_.LookAheadArc())
-      return FilterState(fs1, FilterState2(kNoLabel));
+    if (!LookAheadArc()) return FilterState(fs1, FilterState2(kNoLabel));
     return LookAheadOutput() ? PushLabelFilterArc(arc1, arc2, fs1)
                              : PushLabelFilterArc(arc2, arc1, fs1);
   }
