@@ -1,3 +1,17 @@
+// Copyright 2005-2020 Google LLC
+//
+// Licensed under the Apache License, Version 2.0 (the 'License');
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+//
+//     http://www.apache.org/licenses/LICENSE-2.0
+//
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an 'AS IS' BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
+//
 // See www.openfst.org for extensive documentation on this weighted
 // finite-state transducer library.
 //
@@ -12,6 +26,7 @@
 #include <vector>
 
 #include <fst/flags.h>
+#include <fst/types.h>
 #include <fst/log.h>
 
 #include <fst/add-on.h>
@@ -20,7 +35,6 @@
 #include <fst/label-reachable.h>
 #include <fst/matcher.h>
 
-
 DECLARE_string(save_relabel_ipairs);
 DECLARE_string(save_relabel_opairs);
 
@@ -48,7 +62,7 @@ namespace fst {
 //
 //   // If safe = true, the copy is thread-safe (except the lookahead FST is
 //   // preserved). See Fst<>::Copy() for further doc.
-//   LookaheadMatcher<FST> *Copy(bool safe = false) const override;
+//   LookaheadMatcher *Copy(bool safe = false) const override;
 
 //  // Below are methods for looking ahead for a match to a label and more
 //  // generally, to a rational set. Each returns false if there is definitely
@@ -180,12 +194,12 @@ class TrivialLookAheadMatcher
       : matcher_(fst, match_type) {}
 
   // This makes a copy of the FST.
-  TrivialLookAheadMatcher(const TrivialLookAheadMatcher<M> &lmatcher,
+  TrivialLookAheadMatcher(const TrivialLookAheadMatcher &lmatcher,
                           bool safe = false)
       : matcher_(lmatcher.matcher_, safe) {}
 
-  TrivialLookAheadMatcher<M> *Copy(bool safe = false) const override {
-    return new TrivialLookAheadMatcher<M>(*this, safe);
+  TrivialLookAheadMatcher *Copy(bool safe = false) const override {
+    return new TrivialLookAheadMatcher(*this, safe);
   }
 
   MatchType Type(bool test) const override { return matcher_.Type(test); }
@@ -232,9 +246,8 @@ class TrivialLookAheadMatcher
 
 // Look-ahead of one transition. Template argument flags accepts flags to
 // control behavior.
-template <class M,
-          uint32 flags = kLookAheadNonEpsilons | kLookAheadEpsilons |
-                         kLookAheadWeight | kLookAheadPrefix>
+template <class M, uint32 flags = kLookAheadNonEpsilons | kLookAheadEpsilons |
+                                  kLookAheadWeight | kLookAheadPrefix>
 class ArcLookAheadMatcher : public LookAheadMatcherBase<typename M::FST::Arc> {
  public:
   using FST = typename M::FST;
@@ -254,34 +267,31 @@ class ArcLookAheadMatcher : public LookAheadMatcherBase<typename M::FST::Arc> {
   enum : uint32 { kFlags = flags };
 
   // This makes a copy of the FST.
-  ArcLookAheadMatcher(
-      const FST &fst, MatchType match_type,
-      std::shared_ptr<MatcherData> data = std::shared_ptr<MatcherData>())
+  ArcLookAheadMatcher(const FST &fst, MatchType match_type,
+                      std::shared_ptr<MatcherData> data = nullptr)
       : matcher_(fst, match_type),
         fst_(matcher_.GetFst()),
         lfst_(nullptr),
         state_(kNoStateId) {}
 
   // This doesn't copy the FST.
-  ArcLookAheadMatcher(
-      const FST *fst, MatchType match_type,
-      std::shared_ptr<MatcherData> data = std::shared_ptr<MatcherData>())
+  ArcLookAheadMatcher(const FST *fst, MatchType match_type,
+                      std::shared_ptr<MatcherData> data = nullptr)
       : matcher_(fst, match_type),
         fst_(matcher_.GetFst()),
         lfst_(nullptr),
         state_(kNoStateId) {}
 
   // This makes a copy of the FST.
-  ArcLookAheadMatcher(const ArcLookAheadMatcher<M, flags> &lmatcher,
-                      bool safe = false)
+  ArcLookAheadMatcher(const ArcLookAheadMatcher &lmatcher, bool safe = false)
       : matcher_(lmatcher.matcher_, safe),
         fst_(matcher_.GetFst()),
         lfst_(lmatcher.lfst_),
         state_(kNoStateId) {}
 
   // General matcher methods.
-  ArcLookAheadMatcher<M, flags> *Copy(bool safe = false) const override {
-    return new ArcLookAheadMatcher<M, flags>(*this, safe);
+  ArcLookAheadMatcher *Copy(bool safe = false) const override {
+    return new ArcLookAheadMatcher(*this, safe);
   }
 
   MatchType Type(bool test) const override { return matcher_.Type(test); }
@@ -316,9 +326,7 @@ class ArcLookAheadMatcher : public LookAheadMatcherBase<typename M::FST::Arc> {
 
   const MatcherData *GetData() const { return nullptr; }
 
-  std::shared_ptr<MatcherData> GetSharedData() const {
-    return std::shared_ptr<MatcherData>();
-  }
+  std::shared_ptr<MatcherData> GetSharedData() const { return nullptr; }
 
   // Look-ahead methods.
 
@@ -418,11 +426,15 @@ template <class M,
           uint32 flags = kLookAheadEpsilons | kLookAheadWeight |
                          kLookAheadPrefix | kLookAheadNonEpsilonPrefix |
                          kLookAheadKeepRelabelData,
-          class Accumulator = DefaultAccumulator<typename M::Arc>,
-          class Reachable = LabelReachable<typename M::Arc, Accumulator>>
+          class Accum = DefaultAccumulator<typename M::Arc>,
+          class R = LabelReachable<typename M::Arc, Accum>>
 class LabelLookAheadMatcher
     : public LookAheadMatcherBase<typename M::FST::Arc> {
  public:
+  using Matcher = M;
+  using Accumulator = Accum;
+  using Reachable = R;
+
   using FST = typename M::FST;
   using Arc = typename FST::Arc;
   using Label = typename Arc::Label;
@@ -437,36 +449,37 @@ class LabelLookAheadMatcher
   using LookAheadMatcherBase<Arc>::LookAheadPrefix;
   using LookAheadMatcherBase<Arc>::SetLookAheadPrefix;
 
+  static_assert(!(flags & kInputLookAheadMatcher) !=
+                    !(flags & kOutputLookAheadMatcher),
+                "Must include precisely one of kInputLookAheadMatcher and "
+                "kOutputLookAheadMatcher");
   enum : uint32 { kFlags = flags };
 
   // This makes a copy of the FST.
-  LabelLookAheadMatcher(
-      const FST &fst, MatchType match_type,
-      std::shared_ptr<MatcherData> data = std::shared_ptr<MatcherData>(),
-      Accumulator *accumulator = nullptr)
+  LabelLookAheadMatcher(const FST &fst, MatchType match_type,
+                        std::shared_ptr<MatcherData> data = nullptr,
+                        std::unique_ptr<Accumulator> accumulator = nullptr)
       : matcher_(fst, match_type),
         lfst_(nullptr),
         state_(kNoStateId),
         error_(false) {
-    Init(fst, match_type, data, accumulator);
+    Init(fst, match_type, data, std::move(accumulator));
   }
 
   // This doesn't copy the FST.
-  LabelLookAheadMatcher(
-      const FST *fst, MatchType match_type,
-      std::shared_ptr<MatcherData> data = std::shared_ptr<MatcherData>(),
-      Accumulator *accumulator = nullptr)
+  LabelLookAheadMatcher(const FST *fst, MatchType match_type,
+                        std::shared_ptr<MatcherData> data = nullptr,
+                        std::unique_ptr<Accumulator> accumulator = nullptr)
       : matcher_(fst, match_type),
         lfst_(nullptr),
         state_(kNoStateId),
         error_(false) {
-    Init(*fst, match_type, data, accumulator);
+    Init(*fst, match_type, data, std::move(accumulator));
   }
 
   // This makes a copy of the FST.
-  LabelLookAheadMatcher(
-      const LabelLookAheadMatcher<M, flags, Accumulator, Reachable> &lmatcher,
-      bool safe = false)
+  LabelLookAheadMatcher(const LabelLookAheadMatcher &lmatcher,
+                        bool safe = false)
       : matcher_(lmatcher.matcher_, safe),
         lfst_(lmatcher.lfst_),
         label_reachable_(lmatcher.label_reachable_
@@ -475,10 +488,8 @@ class LabelLookAheadMatcher
         state_(kNoStateId),
         error_(lmatcher.error_) {}
 
-  LabelLookAheadMatcher<M, flags, Accumulator, Reachable> *Copy(
-      bool safe = false) const override {
-    return new LabelLookAheadMatcher<M, flags, Accumulator, Reachable>(*this,
-                                                                       safe);
+  LabelLookAheadMatcher *Copy(bool safe = false) const override {
+    return new LabelLookAheadMatcher(*this, safe);
   }
 
   MatchType Type(bool test) const override { return matcher_.Type(test); }
@@ -530,11 +541,10 @@ class LabelLookAheadMatcher
 
   const MatcherData *GetData() const {
     return label_reachable_ ? label_reachable_->GetData() : nullptr;
-  };
+  }
 
   std::shared_ptr<MatcherData> GetSharedData() const {
-    return label_reachable_ ? label_reachable_->GetSharedData()
-                            : std::shared_ptr<MatcherData>();
+    return label_reachable_ ? label_reachable_->GetSharedData() : nullptr;
   }
   // Checks if there is a matching (possibly super-final) transition at
   // (state_, s).
@@ -556,7 +566,7 @@ class LabelLookAheadMatcher
 
   template <class LFST>
   void InitLookAheadFst(const LFST &fst, bool copy = false) {
-    lfst_ = static_cast<const Fst<Arc> *>(&fst);
+    lfst_ = &fst;
     if (label_reachable_) {
       const bool reach_input = Type(false) == MATCH_OUTPUT;
       label_reachable_->ReachInit(fst, reach_input, copy);
@@ -579,20 +589,18 @@ class LabelLookAheadMatcher
  private:
   void Init(const FST &fst, MatchType match_type,
             std::shared_ptr<MatcherData> data,
-            Accumulator *accumulator) {
-    if (!(kFlags & (kInputLookAheadMatcher | kOutputLookAheadMatcher))) {
-      FSTERROR() << "LabelLookaheadMatcher: Bad matcher flags: " << kFlags;
-      error_ = true;
-    }
+            std::unique_ptr<Accumulator> accumulator) {
     const bool reach_input = match_type == MATCH_INPUT;
     if (data) {
       if (reach_input == data->ReachInput()) {
-        label_reachable_.reset(new Reachable(data, accumulator));
+        label_reachable_ =
+            std::make_unique<Reachable>(data, std::move(accumulator));
       }
     } else if ((reach_input && (kFlags & kInputLookAheadMatcher)) ||
                (!reach_input && (kFlags & kOutputLookAheadMatcher))) {
-      label_reachable_.reset(new Reachable(fst, reach_input, accumulator,
-                                           kFlags & kLookAheadKeepRelabelData));
+      label_reachable_ =
+          std::make_unique<Reachable>(fst, reach_input, std::move(accumulator),
+                                       kFlags & kLookAheadKeepRelabelData);
     }
   }
 
@@ -610,7 +618,7 @@ template <class LFST>
 inline bool LabelLookAheadMatcher<M, flags, Accumulator,
                                   Reachable>::LookAheadFst(const LFST &fst,
                                                            StateId s) {
-  if (static_cast<const Fst<Arc> *>(&fst) != lfst_) InitLookAheadFst(fst);
+  if (&fst != lfst_) InitLookAheadFst(fst);
   ClearLookAheadWeight();
   ClearLookAheadPrefix();
   if (!label_reachable_) return true;
@@ -642,6 +650,34 @@ inline bool LabelLookAheadMatcher<M, flags, Accumulator,
   return reach_arc || reach_final;
 }
 
+// Relabels the fst with Reachable::Reachable. Relabels input
+// if data.First() is non-null, otherwise relabels output.
+// Optionally saves the input/output label pairs to a file
+// if save_relabel_ipairs/opairs is non-empty.
+template <class Reachable, class FST, class Data>
+void RelabelForReachable(FST *fst, const Data &data,
+                         const std::string &save_relabel_ipairs,
+                         const std::string &save_relabel_opairs) {
+  using Label = typename FST::Arc::Label;
+  if (data.First() != nullptr) {  // reach_input.
+    Reachable reachable(data.SharedFirst());
+    reachable.Relabel(fst, /*relabel_input=*/true);
+    if (!save_relabel_ipairs.empty()) {
+      std::vector<std::pair<Label, Label>> pairs;
+      reachable.RelabelPairs(&pairs, /*avoid_collisions=*/true);
+      WriteLabelPairs(save_relabel_ipairs, pairs);
+    }
+  } else {
+    Reachable reachable(data.SharedSecond());
+    reachable.Relabel(fst, /*relabel_input=*/false);
+    if (!save_relabel_opairs.empty()) {
+      std::vector<std::pair<Label, Label>> pairs;
+      reachable.RelabelPairs(&pairs, /*avoid_collisions=*/true);
+      WriteLabelPairs(save_relabel_opairs, pairs);
+    }
+  }
+}
+
 // Label-lookahead relabeling class.
 template <class Arc, class Data = LabelReachableData<typename Arc::Label>>
 class LabelLookAheadRelabeler {
@@ -687,28 +723,22 @@ inline LabelLookAheadRelabeler<Arc, Data>::LabelLookAheadRelabeler(
   const bool is_mutable = fst.Properties(kMutable, false);
   std::unique_ptr<MutableFst<Arc>> mfst;
   if (is_mutable) {
-    mfst.reset(static_cast<MutableFst<Arc> *>(&fst));
+    // Borrow pointer from fst without increasing ref count; it will
+    // be released below. We do not want to call Copy() since that would
+    // do a deep copy when the Fst is modified.
+    mfst.reset(fst::down_cast<MutableFst<Arc> *>(&fst));
   } else {
-    mfst.reset(new VectorFst<Arc>(fst));
+    mfst = std::make_unique<VectorFst<Arc>>(fst);
   }
-  if (data->First()) {  // reach_input.
-    Reachable reachable(data->SharedFirst());
-    reachable.Relabel(mfst.get(), true);
-    if (!FLAGS_save_relabel_ipairs.empty()) {
-      std::vector<std::pair<Label, Label>> pairs;
-      reachable.RelabelPairs(&pairs, true);
-      WriteLabelPairs(FLAGS_save_relabel_ipairs, pairs);
-    }
+
+  RelabelForReachable<Reachable>(mfst.get(), *data,
+                                 FST_FLAGS_save_relabel_ipairs,
+                                 FST_FLAGS_save_relabel_opairs);
+
+  if (is_mutable) {
+    // Pointer was just borrowed, don't delete it.
+    mfst.release();
   } else {
-    Reachable reachable(data->SharedSecond());
-    reachable.Relabel(mfst.get(), false);
-    if (!FLAGS_save_relabel_opairs.empty()) {
-      std::vector<std::pair<Label, Label>> pairs;
-      reachable.RelabelPairs(&pairs, true);
-      WriteLabelPairs(FLAGS_save_relabel_opairs, pairs);
-    }
-  }
-  if (!is_mutable) {
     *impl = std::make_shared<Impl>(*mfst, name);
     (*impl)->SetAddOn(data);
   }
@@ -731,28 +761,27 @@ class LookAheadMatcher {
       : owned_fst_(fst.Copy()),
         base_(owned_fst_->InitMatcher(match_type)),
         lookahead_(false) {
-    if (!base_) base_.reset(new SortedMatcher<FST>(owned_fst_.get(),
-                                                   match_type));
+    if (!base_)
+      base_ =
+          std::make_unique<SortedMatcher<FST>>(owned_fst_.get(), match_type);
   }
 
   // This doesn't copy the FST.
   LookAheadMatcher(const FST *fst, MatchType match_type)
-      : base_(fst->InitMatcher(match_type)),
-        lookahead_(false) {
-    if (!base_) base_.reset(new SortedMatcher<FST>(fst, match_type));
+      : base_(fst->InitMatcher(match_type)), lookahead_(false) {
+    if (!base_) base_ = std::make_unique<SortedMatcher<FST>>(fst, match_type);
   }
 
   // This makes a copy of the FST.
-  LookAheadMatcher(const LookAheadMatcher<FST> &matcher, bool safe = false)
-      : base_(matcher.base_->Copy(safe)),
-        lookahead_(matcher.lookahead_) { }
+  LookAheadMatcher(const LookAheadMatcher &matcher, bool safe = false)
+      : base_(matcher.base_->Copy(safe)), lookahead_(matcher.lookahead_) {}
 
   // Takes ownership of base.
   explicit LookAheadMatcher(MatcherBase<Arc> *base)
       : base_(base), lookahead_(false) {}
 
-  LookAheadMatcher<FST> *Copy(bool safe = false) const {
-    return new LookAheadMatcher<FST>(*this, safe);
+  LookAheadMatcher *Copy(bool safe = false) const {
+    return new LookAheadMatcher(*this, safe);
   }
 
   MatchType Type(bool test) const { return base_->Type(test); }
@@ -771,9 +800,7 @@ class LookAheadMatcher {
 
   ssize_t Priority(StateId s) { return base_->Priority(s); }
 
-  const FST &GetFst() const {
-    return static_cast<const FST &>(base_->GetFst());
-  }
+  const FST &GetFst() const { return fst::down_cast<const FST &>(base_->GetFst()); }
 
   uint64 Properties(uint64 props) const { return base_->Properties(props); }
 
@@ -781,7 +808,7 @@ class LookAheadMatcher {
 
   bool LookAheadLabel(Label label) const {
     if (LookAheadCheck()) {
-      return static_cast<LBase *>(base_.get())->LookAheadLabel(label);
+      return fst::down_cast<LBase *>(base_.get())->LookAheadLabel(label);
     } else {
       return true;
     }
@@ -789,7 +816,7 @@ class LookAheadMatcher {
 
   bool LookAheadFst(const Fst<Arc> &fst, StateId s) {
     if (LookAheadCheck()) {
-      return static_cast<LBase *>(base_.get())->LookAheadFst(fst, s);
+      return fst::down_cast<LBase *>(base_.get())->LookAheadFst(fst, s);
     } else {
       return true;
     }
@@ -797,7 +824,7 @@ class LookAheadMatcher {
 
   Weight LookAheadWeight() const {
     if (LookAheadCheck()) {
-      return static_cast<LBase *>(base_.get())->LookAheadWeight();
+      return fst::down_cast<LBase *>(base_.get())->LookAheadWeight();
     } else {
       return Weight::One();
     }
@@ -805,7 +832,7 @@ class LookAheadMatcher {
 
   bool LookAheadPrefix(Arc *arc) const {
     if (LookAheadCheck()) {
-      return static_cast<LBase *>(base_.get())->LookAheadPrefix(arc);
+      return fst::down_cast<LBase *>(base_.get())->LookAheadPrefix(arc);
     } else {
       return false;
     }
@@ -813,7 +840,7 @@ class LookAheadMatcher {
 
   void InitLookAheadFst(const Fst<Arc> &fst, bool copy = false) {
     if (LookAheadCheck()) {
-      static_cast<LBase *>(base_.get())->InitLookAheadFst(fst, copy);
+      fst::down_cast<LBase *>(base_.get())->InitLookAheadFst(fst, copy);
     }
   }
 
