@@ -1,3 +1,17 @@
+// Copyright 2005-2020 Google LLC
+//
+// Licensed under the Apache License, Version 2.0 (the 'License');
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+//
+//     http://www.apache.org/licenses/LICENSE-2.0
+//
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an 'AS IS' BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
+//
 // See www.openfst.org for extensive documentation on this weighted
 // finite-state transducer library.
 //
@@ -9,6 +23,8 @@
 #include <memory>
 #include <string>
 
+#include <fst/types.h>
+
 #include <fst/add-on.h>
 #include <fst/const-fst.h>
 #include <fst/lookahead-matcher.h>
@@ -84,49 +100,49 @@ class MatcherFst : public ImplToExpandedFst<internal::AddOnImpl<F, Data>> {
 
   MatcherFst() : ImplToExpandedFst<Impl>(std::make_shared<Impl>(FST(), Name)) {}
 
+  // Constructs a MatcherFst from an FST, which is the underlying FST type used
+  // by this class. Uses the existing Data if present, and runs Init on it.
+  // Stores fst internally, making a thread-safe copy of it.
   explicit MatcherFst(const FST &fst, std::shared_ptr<Data> data = nullptr)
       : ImplToExpandedFst<Impl>(data ? CreateImpl(fst, Name, data)
                                      : CreateDataAndImpl(fst, Name)) {}
 
-  explicit MatcherFst(const Fst<Arc> &fst)
-      : ImplToExpandedFst<Impl>(CreateDataAndImpl(fst, Name)) {}
+  // Constructs a MatcherFst from an Fst<Arc>, which is *not* the underlying
+  // FST type used by this class. Uses the existing Data if present, and
+  // runs Init on it. Stores fst internally, converting Fst<Arc> to FST and
+  // therefore making a deep copy.
+  explicit MatcherFst(const Fst<Arc> &fst, std::shared_ptr<Data> data = nullptr)
+      : ImplToExpandedFst<Impl>(data ? CreateImpl(fst, Name, data)
+                                     : CreateDataAndImpl(fst, Name)) {}
 
   // See Fst<>::Copy() for doc.
-  MatcherFst(const MatcherFst<FST, FstMatcher, Name, Init, Data> &fst,
-             bool safe = false)
+  MatcherFst(const MatcherFst &fst, bool safe = false)
       : ImplToExpandedFst<Impl>(fst, safe) {}
 
   // Get a copy of this MatcherFst. See Fst<>::Copy() for further doc.
-  MatcherFst<FST, FstMatcher, Name, Init, Data> *Copy(
-      bool safe = false) const override {
-    return new MatcherFst<FST, FstMatcher, Name, Init, Data>(*this, safe);
+  MatcherFst *Copy(bool safe = false) const override {
+    return new MatcherFst(*this, safe);
   }
 
   // Read a MatcherFst from an input stream; return nullptr on error
-  static MatcherFst<FST, M, Name, Init, Data> *Read(
-      std::istream &strm, const FstReadOptions &opts) {
+  static MatcherFst *Read(std::istream &strm, const FstReadOptions &opts) {
     auto *impl = Impl::Read(strm, opts);
-    return impl ? new MatcherFst<FST, FstMatcher, Name, Init, Data>(
-                      std::shared_ptr<Impl>(impl))
-                : nullptr;
+    return impl ? new MatcherFst(std::shared_ptr<Impl>(impl)) : nullptr;
   }
 
   // Read a MatcherFst from a file; return nullptr on error
-  // Empty filename reads from standard input
-  static MatcherFst<FST, FstMatcher, Name, Init, Data> *Read(
-      const string &filename) {
-    auto *impl = ImplToExpandedFst<Impl>::Read(filename);
-    return impl ? new MatcherFst<FST, FstMatcher, Name, Init, Data>(
-                      std::shared_ptr<Impl>(impl))
-                : nullptr;
+  // Empty source reads from standard input
+  static MatcherFst *Read(const std::string &source) {
+    auto *impl = ImplToExpandedFst<Impl>::Read(source);
+    return impl ? new MatcherFst(std::shared_ptr<Impl>(impl)) : nullptr;
   }
 
   bool Write(std::ostream &strm, const FstWriteOptions &opts) const override {
     return GetImpl()->Write(strm, opts);
   }
 
-  bool Write(const string &filename) const override {
-    return Fst<Arc>::WriteFile(filename);
+  bool Write(const std::string &source) const override {
+    return Fst<Arc>::WriteFile(source);
   }
 
   void InitStateIterator(StateIteratorData<Arc> *data) const override {
@@ -163,8 +179,9 @@ class MatcherFst : public ImplToExpandedFst<internal::AddOnImpl<F, Data>> {
  protected:
   using ImplToFst<Impl, ExpandedFst<Arc>>::GetImpl;
 
+  // Makes a thread-safe copy of fst.
   static std::shared_ptr<Impl> CreateDataAndImpl(const FST &fst,
-                                                 const string &name) {
+                                                 const std::string &name) {
     FstMatcher imatcher(fst, MATCH_INPUT);
     FstMatcher omatcher(fst, MATCH_OUTPUT);
     return CreateImpl(fst, name,
@@ -172,13 +189,26 @@ class MatcherFst : public ImplToExpandedFst<internal::AddOnImpl<F, Data>> {
                                              omatcher.GetSharedData()));
   }
 
+  // Makes a deep copy of fst.
   static std::shared_ptr<Impl> CreateDataAndImpl(const Fst<Arc> &fst,
-                                                 const string &name) {
+                                                 const std::string &name) {
     FST result(fst);
     return CreateDataAndImpl(result, name);
   }
 
-  static std::shared_ptr<Impl> CreateImpl(const FST &fst, const string &name,
+  // Makes a thread-safe copy of fst.
+  static std::shared_ptr<Impl> CreateImpl(const FST &fst,
+                                          const std::string &name,
+                                          std::shared_ptr<Data> data) {
+    auto impl = std::make_shared<Impl>(fst, name);
+    impl->SetAddOn(data);
+    Init init(&impl);
+    return impl;
+  }
+
+  // Makes a deep copy of fst.
+  static std::shared_ptr<Impl> CreateImpl(const Fst<Arc> &fst,
+                                          const std::string &name,
                                           std::shared_ptr<Data> data) {
     auto impl = std::make_shared<Impl>(fst, name);
     impl->SetAddOn(data);
@@ -225,9 +255,9 @@ class Matcher<MatcherFst<F, M, Name, Init>> {
   Matcher(const FST &fst, MatchType match_type)
       : matcher_(fst.InitMatcher(match_type)) {}
 
-  Matcher(const Matcher<FST> &matcher) : matcher_(matcher.matcher_->Copy()) {}
+  Matcher(const Matcher &matcher) : matcher_(matcher.matcher_->Copy()) {}
 
-  Matcher<FST> *Copy() const { return new Matcher<FST>(*this); }
+  Matcher *Copy() const { return new Matcher(*this); }
 
   MatchType Type(bool test) const { return matcher_->Type(test); }
 
@@ -262,12 +292,12 @@ class LookAheadMatcher<MatcherFst<F, M, Name, Init>> {
   LookAheadMatcher(const FST &fst, MatchType match_type)
       : matcher_(fst.InitMatcher(match_type)) {}
 
-  LookAheadMatcher(const LookAheadMatcher<FST> &matcher, bool safe = false)
+  LookAheadMatcher(const LookAheadMatcher &matcher, bool safe = false)
       : matcher_(matcher.matcher_->Copy(safe)) {}
 
   // General matcher methods.
-  LookAheadMatcher<FST> *Copy(bool safe = false) const {
-    return new LookAheadMatcher<FST>(*this, safe);
+  LookAheadMatcher *Copy(bool safe = false) const {
+    return new LookAheadMatcher(*this, safe);
   }
 
   MatchType Type(bool test) const { return matcher_->Type(test); }
