@@ -1,3 +1,17 @@
+// Copyright 2005-2020 Google LLC
+//
+// Licensed under the Apache License, Version 2.0 (the 'License');
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+//
+//     http://www.apache.org/licenses/LICENSE-2.0
+//
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an 'AS IS' BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
+//
 // See www.openfst.org for extensive documentation on this weighted
 // finite-state transducer library.
 //
@@ -8,13 +22,14 @@
 
 #include <algorithm>
 #include <memory>
-#include <unordered_map>
 #include <utility>
 
+#include <fst/types.h>
 #include <fst/log.h>
 
 #include <fst/mutable-fst.h>  // for all internal FST accessors.
 
+#include <unordered_map>
 
 namespace fst {
 
@@ -51,14 +66,15 @@ namespace fst {
 //
 //   // If safe = true, the copy is thread-safe. See Fst<>::Copy() for
 //   // further doc.
-//   Matcher<FST> *Copy(bool safe = false) const override;
+//   Matcher *Copy(bool safe = false) const override;
 //
 //   // Returns the match type that can be provided (depending on compatibility
-//   of the input FST). It is either the requested match type, MATCH_NONE, or
-//   MATCH_UNKNOWN. If test is false, a costly testing is avoided, but
-//   MATCH_UNKNOWN may be returned. If test is true, a definite answer is
-//   returned, but may involve more costly computation (e.g., visiting the FST).
-//   MatchType Type(bool test) const override;
+//   // of the input FST). It is either the requested match type, MATCH_NONE,
+//   // or MATCH_UNKNOWN. If test is false, a costly testing is avoided, but
+//   // MATCH_UNKNOWN may be returned. If test is true, a definite answer is
+//   // returned, but may involve more costly computation (e.g., visiting
+//   // the FST).
+//   // MatchType Type(bool test) const override;
 //
 //   // Specifies the current state.
 //   void SetState(StateId s) final;
@@ -69,7 +85,7 @@ namespace fst {
 //   bool Find(Label label) final;
 //
 //   // Iterator methods. Note that initially and after SetState() these have
-//   undefined behavior until Find() is called.
+//   // undefined behavior until Find() is called.
 //
 //   bool Done() const final;
 //
@@ -123,7 +139,7 @@ class MatcherBase {
 
   // Virtual interface.
 
-  virtual MatcherBase<Arc> *Copy(bool safe = false) const = 0;
+  virtual MatcherBase *Copy(bool safe = false) const = 0;
   virtual MatchType Type(bool) const = 0;
   virtual void SetState(StateId) = 0;
   virtual bool Find(Label) = 0;
@@ -196,7 +212,7 @@ class SortedMatcher : public MatcherBase<typename F::Arc> {
   }
 
   // This makes a copy of the FST.
-  SortedMatcher(const SortedMatcher<FST> &matcher, bool safe = false)
+  SortedMatcher(const SortedMatcher &matcher, bool safe = false)
       : owned_fst_(matcher.fst_.Copy(safe)),
         fst_(*owned_fst_),
         state_(kNoStateId),
@@ -211,8 +227,8 @@ class SortedMatcher : public MatcherBase<typename F::Arc> {
 
   ~SortedMatcher() override { Destroy(aiter_, &aiter_pool_); }
 
-  SortedMatcher<FST> *Copy(bool safe = false) const override {
-    return new SortedMatcher<FST>(*this, safe);
+  SortedMatcher *Copy(bool safe = false) const override {
+    return new SortedMatcher(*this, safe);
   }
 
   MatchType Type(bool test) const override {
@@ -280,8 +296,8 @@ class SortedMatcher : public MatcherBase<typename F::Arc> {
     if (current_loop_) return false;
     if (aiter_->Done()) return true;
     if (!exact_match_) return false;
-    aiter_->SetFlags(match_type_ == MATCH_INPUT ?
-        kArcILabelValue : kArcOLabelValue,
+    aiter_->SetFlags(
+        match_type_ == MATCH_INPUT ? kArcILabelValue : kArcOLabelValue,
         kArcValueFlags);
     return GetLabel() != match_label_;
   }
@@ -300,13 +316,9 @@ class SortedMatcher : public MatcherBase<typename F::Arc> {
     }
   }
 
-  Weight Final(StateId s) const final {
-    return MatcherBase<Arc>::Final(s);
-  }
+  Weight Final(StateId s) const final { return MatcherBase<Arc>::Final(s); }
 
-  ssize_t Priority(StateId s) final {
-    return MatcherBase<Arc>::Priority(s);
-  }
+  ssize_t Priority(StateId s) final { return MatcherBase<Arc>::Priority(s); }
 
   const FST &GetFst() const override { return fst_; }
 
@@ -326,40 +338,51 @@ class SortedMatcher : public MatcherBase<typename F::Arc> {
   bool LinearSearch();
   bool Search();
 
-  std::unique_ptr<const FST> owned_fst_;   // FST ptr if owned.
-  const FST &fst_;           // FST for matching.
-  StateId state_;            // Matcher state.
-  ArcIterator<FST> *aiter_;  // Iterator for current state.
-  MatchType match_type_;     // Type of match to perform.
-  Label binary_label_;       // Least label for binary search.
-  Label match_label_;        // Current label to be matched.
-  size_t narcs_;             // Current state arc count.
-  Arc loop_;                 // For non-consuming symbols.
-  bool current_loop_;        // Current arc is the implicit loop.
-  bool exact_match_;         // Exact match or lower bound?
-  bool error_;               // Error encountered?
+  std::unique_ptr<const FST> owned_fst_;  // FST ptr if owned.
+  const FST &fst_;                        // FST for matching.
+  StateId state_;                         // Matcher state.
+  ArcIterator<FST> *aiter_;               // Iterator for current state.
+  MatchType match_type_;                  // Type of match to perform.
+  Label binary_label_;                    // Least label for binary search.
+  Label match_label_;                     // Current label to be matched.
+  size_t narcs_;                          // Current state arc count.
+  Arc loop_;                              // For non-consuming symbols.
+  bool current_loop_;                     // Current arc is the implicit loop.
+  bool exact_match_;                      // Exact match or lower bound?
+  bool error_;                            // Error encountered?
   MemoryPool<ArcIterator<FST>> aiter_pool_;  // Pool of arc iterators.
 };
 
 // Returns true iff match to match_label_. The arc iterator is positioned at the
 // lower bound, that is, the first element greater than or equal to
 // match_label_, or the end if all elements are less than match_label_.
+// If multiple elements are equal to the `match_label_`, returns the rightmost
+// one.
 template <class FST>
 inline bool SortedMatcher<FST>::BinarySearch() {
-  size_t low = 0;
-  size_t high = narcs_;
-  while (low < high) {
-    const size_t mid = low + (high - low) / 2;
+  size_t size = narcs_;
+  if (size == 0) {
+    return false;
+  }
+  size_t high = size - 1;
+  while (size > 1) {
+    const size_t half = size / 2;
+    const size_t mid = high - half;
     aiter_->Seek(mid);
-    if (GetLabel() < match_label_) {
-      low = mid + 1;
-    } else {
+    if (GetLabel() >= match_label_) {
       high = mid;
     }
+    size -= half;
   }
-
-  aiter_->Seek(low);
-  return low < narcs_ && GetLabel() == match_label_;
+  aiter_->Seek(high);
+  const auto label = GetLabel();
+  if (label == match_label_) {
+    return true;
+  }
+  if (label < match_label_) {
+    aiter_->Next();
+  }
+  return false;
 }
 
 // Returns true iff match to match_label_, positioning arc iterator at lower
@@ -378,9 +401,9 @@ inline bool SortedMatcher<FST>::LinearSearch() {
 // bound.
 template <class FST>
 inline bool SortedMatcher<FST>::Search() {
-  aiter_->SetFlags(match_type_ == MATCH_INPUT ?
-                   kArcILabelValue : kArcOLabelValue,
-                   kArcValueFlags);
+  aiter_->SetFlags(
+      match_type_ == MATCH_INPUT ? kArcILabelValue : kArcOLabelValue,
+      kArcValueFlags);
   if (match_label_ >= binary_label_) {
     return BinarySearch();
   } else {
@@ -433,18 +456,18 @@ class HashMatcher : public MatcherBase<typename F::Arc> {
   }
 
   // This makes a copy of the FST.
-  HashMatcher(const HashMatcher<FST> &matcher, bool safe = false)
+  HashMatcher(const HashMatcher &matcher, bool safe = false)
       : owned_fst_(matcher.fst_.Copy(safe)),
         fst_(*owned_fst_),
         state_(kNoStateId),
         match_type_(matcher.match_type_),
         loop_(matcher.loop_),
         error_(matcher.error_),
-        state_table_(
-            safe ? std::make_shared<StateTable>() : matcher.state_table_) {}
+        state_table_(safe ? std::make_shared<StateTable>()
+                          : matcher.state_table_) {}
 
-  HashMatcher<FST> *Copy(bool safe = false) const override {
-    return new HashMatcher<FST>(*this, safe);
+  HashMatcher *Copy(bool safe = false) const override {
+    return new HashMatcher(*this, safe);
   }
 
   // The argument is ignored as there are no relevant properties to test.
@@ -499,12 +522,12 @@ class HashMatcher : public MatcherBase<typename F::Arc> {
   using StateTable = std::unordered_map<StateId, std::unique_ptr<LabelTable>>;
 
   std::unique_ptr<const FST> owned_fst_;  // ptr to FST if owned.
-  const FST &fst_;     // FST for matching.
-  StateId state_;      // Matcher state.
+  const FST &fst_;                        // FST for matching.
+  StateId state_;                         // Matcher state.
   MatchType match_type_;
-  Arc loop_;            // The implicit loop itself.
-  bool current_loop_;   // Is the current arc the implicit loop?
-  bool error_;          // Error encountered?
+  Arc loop_;           // The implicit loop itself.
+  bool current_loop_;  // Is the current arc the implicit loop?
+  bool error_;         // Error encountered?
   std::unique_ptr<ArcIterator<FST>> aiter_;
   std::shared_ptr<StateTable> state_table_;  // Table from state to label table.
   LabelTable *label_table_;  // Pointer to current state's label table.
@@ -518,14 +541,14 @@ void HashMatcher<FST>::SetState(typename FST::Arc::StateId s) {
   // Resets everything for the state.
   state_ = s;
   loop_.nextstate = state_;
-  aiter_.reset(new ArcIterator<FST>(fst_, state_));
+  aiter_ = std::make_unique<ArcIterator<FST>>(fst_, state_);
   if (match_type_ == MATCH_NONE) {
     FSTERROR() << "HashMatcher: Bad match type";
     error_ = true;
   }
   // Attempts to insert a new label table.
   auto it_and_success = state_table_->emplace(
-    state_, std::unique_ptr<LabelTable>(new LabelTable()));
+      state_, std::make_unique<LabelTable>());
   // Sets instance's pointer to the label table for this state.
   label_table_ = it_and_success.first->second.get();
   // If it already exists, no additional work is done and we simply return.
@@ -563,14 +586,14 @@ enum MatcherRewriteMode {
 // For any requested label that doesn't match at a state, this matcher
 // considers the *unique* transition that matches the label 'phi_label'
 // (phi = 'fail'), and recursively looks for a match at its
-// destination.  When 'phi_loop' is true, if no match is found but a
+// destination. When 'phi_loop' is true, if no match is found but a
 // phi self-loop is found, then the phi transition found is returned
 // with the phi_label rewritten as the requested label (both sides if
 // an acceptor, or if 'rewrite_both' is true and both input and output
-// labels of the found transition are 'phi_label').  If 'phi_label' is
-// kNoLabel, this special matching is not done.  PhiMatcher is
+// labels of the found transition are 'phi_label'). If 'phi_label' is
+// kNoLabel, this special matching is not done. PhiMatcher is
 // templated itself on a matcher, which is used to perform the
-// underlying matching.  By default, the underlying matcher is
+// underlying matching. By default, the underlying matcher is
 // constructed by PhiMatcher. The user can instead pass in this
 // object; in that case, PhiMatcher takes its ownership.
 // Phi non-determinism not supported. No non-consuming symbols other
@@ -615,11 +638,10 @@ class PhiMatcher : public MatcherBase<typename M::Arc> {
              MatcherRewriteMode rewrite_mode = MATCHER_REWRITE_AUTO,
              M *matcher = nullptr)
       : PhiMatcher(*fst, match_type, phi_label, phi_loop, rewrite_mode,
-                   matcher ? matcher : new M(fst, match_type)) { }
-
+                   matcher ? matcher : new M(fst, match_type)) {}
 
   // This makes a copy of the FST.
-  PhiMatcher(const PhiMatcher<M> &matcher, bool safe = false)
+  PhiMatcher(const PhiMatcher &matcher, bool safe = false)
       : matcher_(new M(*matcher.matcher_, safe)),
         match_type_(matcher.match_type_),
         phi_label_(matcher.phi_label_),
@@ -628,8 +650,8 @@ class PhiMatcher : public MatcherBase<typename M::Arc> {
         phi_loop_(matcher.phi_loop_),
         error_(matcher.error_) {}
 
-  PhiMatcher<M> *Copy(bool safe = false) const override {
-    return new PhiMatcher<M>(*this, safe);
+  PhiMatcher *Copy(bool safe = false) const override {
+    return new PhiMatcher(*this, safe);
   }
 
   MatchType Type(bool test) const override { return matcher_->Type(test); }
@@ -789,7 +811,7 @@ inline uint64 PhiMatcher<M>::Properties(uint64 inprops) const {
     return outprops;
   } else if (match_type_ == MATCH_INPUT) {
     if (phi_label_ == 0) {
-      outprops &= ~kEpsilons | ~kIEpsilons | ~kOEpsilons;
+      outprops &= ~(kEpsilons | kIEpsilons | kOEpsilons);
       outprops |= kNoEpsilons | kNoIEpsilons;
     }
     if (rewrite_both_) {
@@ -803,7 +825,7 @@ inline uint64 PhiMatcher<M>::Properties(uint64 inprops) const {
     }
   } else if (match_type_ == MATCH_OUTPUT) {
     if (phi_label_ == 0) {
-      outprops &= ~kEpsilons | ~kIEpsilons | ~kOEpsilons;
+      outprops &= ~(kEpsilons | kIEpsilons | kOEpsilons);
       outprops |= kNoEpsilons | kNoOEpsilons;
     }
     if (rewrite_both_) {
@@ -824,14 +846,14 @@ inline uint64 PhiMatcher<M>::Properties(uint64 inprops) const {
 
 // For any requested label that doesn't match at a state, this matcher
 // considers all transitions that match the label 'rho_label' (rho =
-// 'rest').  Each such rho transition found is returned with the
+// 'rest'). Each such rho transition found is returned with the
 // rho_label rewritten as the requested label (both sides if an
 // acceptor, or if 'rewrite_both' is true and both input and output
-// labels of the found transition are 'rho_label').  If 'rho_label' is
-// kNoLabel, this special matching is not done.  RhoMatcher is
+// labels of the found transition are 'rho_label'). If 'rho_label' is
+// kNoLabel, this special matching is not done. RhoMatcher is
 // templated itself on a matcher, which is used to perform the
-// underlying matching.  By default, the underlying matcher is
-// constructed by RhoMatcher.  The user can instead pass in this
+// underlying matching. By default, the underlying matcher is
+// constructed by RhoMatcher. The user can instead pass in this
 // object; in that case, RhoMatcher takes its ownership.
 // No non-consuming symbols other than epsilon supported with
 // the underlying template argument matcher.
@@ -878,10 +900,10 @@ class RhoMatcher : public MatcherBase<typename M::Arc> {
              MatcherRewriteMode rewrite_mode = MATCHER_REWRITE_AUTO,
              M *matcher = nullptr)
       : RhoMatcher(*fst, match_type, rho_label, rewrite_mode,
-                   matcher ? matcher : new M(fst, match_type)) { }
+                   matcher ? matcher : new M(fst, match_type)) {}
 
   // This makes a copy of the FST.
-  RhoMatcher(const RhoMatcher<M> &matcher, bool safe = false)
+  RhoMatcher(const RhoMatcher &matcher, bool safe = false)
       : matcher_(new M(*matcher.matcher_, safe)),
         match_type_(matcher.match_type_),
         rho_label_(matcher.rho_label_),
@@ -890,8 +912,8 @@ class RhoMatcher : public MatcherBase<typename M::Arc> {
         state_(kNoStateId),
         has_rho_(false) {}
 
-  RhoMatcher<M> *Copy(bool safe = false) const override {
-    return new RhoMatcher<M>(*this, safe);
+  RhoMatcher *Copy(bool safe = false) const override {
+    return new RhoMatcher(*this, safe);
   }
 
   MatchType Type(bool test) const override { return matcher_->Type(test); }
@@ -992,9 +1014,8 @@ inline uint64 RhoMatcher<M>::Properties(uint64 inprops) const {
              ~(kODeterministic | kNonODeterministic | kString | kILabelSorted |
                kNotILabelSorted | kOLabelSorted | kNotOLabelSorted);
     } else {
-      return outprops &
-             ~(kODeterministic | kAcceptor | kString | kILabelSorted |
-               kNotILabelSorted);
+      return outprops & ~(kODeterministic | kAcceptor | kString |
+                          kILabelSorted | kNotILabelSorted);
     }
   } else if (match_type_ == MATCH_OUTPUT) {
     if (rewrite_both_) {
@@ -1002,9 +1023,8 @@ inline uint64 RhoMatcher<M>::Properties(uint64 inprops) const {
              ~(kIDeterministic | kNonIDeterministic | kString | kILabelSorted |
                kNotILabelSorted | kOLabelSorted | kNotOLabelSorted);
     } else {
-      return outprops &
-             ~(kIDeterministic | kAcceptor | kString | kOLabelSorted |
-               kNotOLabelSorted);
+      return outprops & ~(kIDeterministic | kAcceptor | kString |
+                          kOLabelSorted | kNotOLabelSorted);
     }
   } else {
     // Shouldn't ever get here.
@@ -1015,16 +1035,16 @@ inline uint64 RhoMatcher<M>::Properties(uint64 inprops) const {
 
 // For any requested label, this matcher considers all transitions
 // that match the label 'sigma_label' (sigma = "any"), and this in
-// additions to transitions with the requested label.  Each such sigma
+// additions to transitions with the requested label. Each such sigma
 // transition found is returned with the sigma_label rewritten as the
 // requested label (both sides if an acceptor, or if 'rewrite_both' is
 // true and both input and output labels of the found transition are
-// 'sigma_label').  If 'sigma_label' is kNoLabel, this special
-// matching is not done.  SigmaMatcher is templated itself on a
-// matcher, which is used to perform the underlying matching.  By
+// 'sigma_label'). If 'sigma_label' is kNoLabel, this special
+// matching is not done. SigmaMatcher is templated itself on a
+// matcher, which is used to perform the underlying matching. By
 // default, the underlying matcher is constructed by SigmaMatcher.
 // The user can instead pass in this object; in that case,
-// SigmaMatcher takes its ownership.  No non-consuming symbols other
+// SigmaMatcher takes its ownership. No non-consuming symbols other
 // than epsilon supported with the underlying template argument matcher.
 template <class M>
 class SigmaMatcher : public MatcherBase<typename M::Arc> {
@@ -1068,12 +1088,12 @@ class SigmaMatcher : public MatcherBase<typename M::Arc> {
   SigmaMatcher(const FST *fst, MatchType match_type,
                Label sigma_label = kNoLabel,
                MatcherRewriteMode rewrite_mode = MATCHER_REWRITE_AUTO,
-             M *matcher = nullptr)
+               M *matcher = nullptr)
       : SigmaMatcher(*fst, match_type, sigma_label, rewrite_mode,
-                     matcher ? matcher : new M(fst, match_type)) { }
+                     matcher ? matcher : new M(fst, match_type)) {}
 
   // This makes a copy of the FST.
-  SigmaMatcher(const SigmaMatcher<M> &matcher, bool safe = false)
+  SigmaMatcher(const SigmaMatcher &matcher, bool safe = false)
       : matcher_(new M(*matcher.matcher_, safe)),
         match_type_(matcher.match_type_),
         sigma_label_(matcher.sigma_label_),
@@ -1081,8 +1101,8 @@ class SigmaMatcher : public MatcherBase<typename M::Arc> {
         error_(matcher.error_),
         state_(kNoStateId) {}
 
-  SigmaMatcher<M> *Copy(bool safe = false) const override {
-    return new SigmaMatcher<M>(*this, safe);
+  SigmaMatcher *Copy(bool safe = false) const override {
+    return new SigmaMatcher(*this, safe);
   }
 
   MatchType Type(bool test) const override { return matcher_->Type(test); }
@@ -1186,20 +1206,17 @@ inline uint64 SigmaMatcher<M>::Properties(uint64 inprops) const {
   if (match_type_ == MATCH_NONE) {
     return outprops;
   } else if (rewrite_both_) {
-    return outprops &
-           ~(kIDeterministic | kNonIDeterministic | kODeterministic |
-             kNonODeterministic | kILabelSorted | kNotILabelSorted |
-             kOLabelSorted | kNotOLabelSorted | kString);
+    return outprops & ~(kIDeterministic | kNonIDeterministic | kODeterministic |
+                        kNonODeterministic | kILabelSorted | kNotILabelSorted |
+                        kOLabelSorted | kNotOLabelSorted | kString);
   } else if (match_type_ == MATCH_INPUT) {
-    return outprops &
-           ~(kIDeterministic | kNonIDeterministic | kODeterministic |
-             kNonODeterministic | kILabelSorted | kNotILabelSorted | kString |
-             kAcceptor);
+    return outprops & ~(kIDeterministic | kNonIDeterministic | kODeterministic |
+                        kNonODeterministic | kILabelSorted | kNotILabelSorted |
+                        kString | kAcceptor);
   } else if (match_type_ == MATCH_OUTPUT) {
-    return outprops &
-           ~(kIDeterministic | kNonIDeterministic | kODeterministic |
-             kNonODeterministic | kOLabelSorted | kNotOLabelSorted | kString |
-             kAcceptor);
+    return outprops & ~(kIDeterministic | kNonIDeterministic | kODeterministic |
+                        kNonODeterministic | kOLabelSorted | kNotOLabelSorted |
+                        kString | kAcceptor);
   } else {
     // Shouldn't ever get here.
     FSTERROR() << "SigmaMatcher: Bad match type: " << match_type_;
@@ -1252,7 +1269,7 @@ class MultiEpsMatcher {
   }
 
   // This makes a copy of the FST.
-  MultiEpsMatcher(const MultiEpsMatcher<M> &matcher, bool safe = false)
+  MultiEpsMatcher(const MultiEpsMatcher &matcher, bool safe = false)
       : matcher_(new M(*matcher.matcher_, safe)),
         flags_(matcher.flags_),
         own_matcher_(true),
@@ -1265,8 +1282,8 @@ class MultiEpsMatcher {
     if (own_matcher_) delete matcher_;
   }
 
-  MultiEpsMatcher<M> *Copy(bool safe = false) const {
-    return new MultiEpsMatcher<M>(*this, safe);
+  MultiEpsMatcher *Copy(bool safe = false) const {
+    return new MultiEpsMatcher(*this, safe);
   }
 
   MatchType Type(bool test) const { return matcher_->Type(test); }
@@ -1426,13 +1443,13 @@ class ExplicitMatcher : public MatcherBase<typename M::Arc> {
         error_(false) {}
 
   // This makes a copy of the FST.
-  ExplicitMatcher(const ExplicitMatcher<M> &matcher, bool safe = false)
+  ExplicitMatcher(const ExplicitMatcher &matcher, bool safe = false)
       : matcher_(new M(*matcher.matcher_, safe)),
         match_type_(matcher.match_type_),
         error_(matcher.error_) {}
 
-  ExplicitMatcher<M> *Copy(bool safe = false) const override {
-    return new ExplicitMatcher<M>(*this, safe);
+  ExplicitMatcher *Copy(bool safe = false) const override {
+    return new ExplicitMatcher(*this, safe);
   }
 
   MatchType Type(bool test) const override { return matcher_->Type(test); }
@@ -1506,29 +1523,26 @@ class Matcher {
 
   // This makes a copy of the FST.
   Matcher(const FST &fst, MatchType match_type)
-      : owned_fst_(fst.Copy()),
-        base_(owned_fst_->InitMatcher(match_type)) {
-    if (!base_) base_.reset(new SortedMatcher<FST>(owned_fst_.get(),
-                                                   match_type));
+      : owned_fst_(fst.Copy()), base_(owned_fst_->InitMatcher(match_type)) {
+    if (!base_)
+      base_ =
+          std::make_unique<SortedMatcher<FST>>(owned_fst_.get(), match_type);
   }
 
   // This doesn't copy the FST.
   Matcher(const FST *fst, MatchType match_type)
       : base_(fst->InitMatcher(match_type)) {
-    if (!base_) base_.reset(new SortedMatcher<FST>(fst, match_type));
+    if (!base_) base_ = std::make_unique<SortedMatcher<FST>>(fst, match_type);
   }
 
   // This makes a copy of the FST.
-  Matcher(const Matcher<FST> &matcher, bool safe = false)
-      : base_(matcher.base_->Copy(safe)) { }
+  Matcher(const Matcher &matcher, bool safe = false)
+      : base_(matcher.base_->Copy(safe)) {}
 
   // Takes ownership of the provided matcher.
-  explicit Matcher(MatcherBase<Arc> *base_matcher)
-      : base_(base_matcher) { }
+  explicit Matcher(MatcherBase<Arc> *base_matcher) : base_(base_matcher) {}
 
-  Matcher<FST> *Copy(bool safe = false) const {
-    return new Matcher<FST>(*this, safe);
-  }
+  Matcher *Copy(bool safe = false) const { return new Matcher(*this, safe); }
 
   MatchType Type(bool test) const { return base_->Type(test); }
 
@@ -1542,9 +1556,7 @@ class Matcher {
 
   void Next() { base_->Next(); }
 
-  const FST &GetFst() const {
-    return static_cast<const FST &>(base_->GetFst());
-  }
+  const FST &GetFst() const { return fst::down_cast<const FST &>(base_->GetFst()); }
 
   uint64 Properties(uint64 props) const { return base_->Properties(props); }
 
