@@ -1,3 +1,17 @@
+// Copyright 2005-2020 Google LLC
+//
+// Licensed under the Apache License, Version 2.0 (the 'License');
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+//
+//     http://www.apache.org/licenses/LICENSE-2.0
+//
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an 'AS IS' BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
+//
 // See www.openfst.org for extensive documentation on this weighted
 // finite-state transducer library.
 //
@@ -6,15 +20,18 @@
 #ifndef FST_RANDGEN_H_
 #define FST_RANDGEN_H_
 
-#include <math.h>
-#include <stddef.h>
+#include <algorithm>
+#include <cmath>
+#include <cstddef>
 #include <limits>
 #include <map>
 #include <memory>
+#include <numeric>
 #include <random>
 #include <utility>
 #include <vector>
 
+#include <fst/types.h>
 #include <fst/log.h>
 
 #include <fst/accumulator.h>
@@ -28,6 +45,8 @@
 #include <fst/util.h>
 #include <fst/weight.h>
 
+#include <vector>
+
 namespace fst {
 
 // The RandGenFst class is roughly similar to ArcMapFst in that it takes two
@@ -50,10 +69,8 @@ class UniformArcSelector {
   using StateId = typename Arc::StateId;
   using Weight = typename Arc::Weight;
 
-  // Constructs a selector with a non-deterministic seed.
-  UniformArcSelector() : rand_(std::random_device()()) {}
-  // Constructs a selector with a given seed.
-  explicit UniformArcSelector(uint64 seed) : rand_(seed) {}
+  explicit UniformArcSelector(uint64 seed = std::random_device()())
+      : rand_(seed) {}
 
   size_t operator()(const Fst<Arc> &fst, StateId s) const {
     const auto n = fst.NumArcs(s) + (fst.Final(s) != Weight::Zero());
@@ -77,9 +94,10 @@ class LogProbArcSelector {
   using Weight = typename Arc::Weight;
 
   // Constructs a selector with a non-deterministic seed.
-  LogProbArcSelector() : rand_(std::random_device()()) {}
+  LogProbArcSelector() : seed_(std::random_device()()), rand_(seed_) {}
+
   // Constructs a selector with a given seed.
-  explicit LogProbArcSelector(uint64 seed) : rand_(seed) {}
+  explicit LogProbArcSelector(uint64 seed) : seed_(seed), rand_(seed) {}
 
   size_t operator()(const Fst<Arc> &fst, StateId s) const {
     // Finds total weight leaving state.
@@ -101,9 +119,19 @@ class LogProbArcSelector {
     return n;
   }
 
+  uint64 Seed() const { return seed_; }
+
+ protected:
+  Log64Weight ToLogWeight(const Weight &weight) const {
+    return to_log_weight_(weight);
+  }
+
+  std::mt19937_64 &MutableRand() const { return rand_; }
+
  private:
+  const uint64 seed_;
   mutable std::mt19937_64 rand_;
-  WeightConvert<Weight, Log64Weight> to_log_weight_;
+  const WeightConvert<Weight, Log64Weight> to_log_weight_{};
 };
 
 // Useful alias when using StdArc.
@@ -117,34 +145,33 @@ class FastLogProbArcSelector : public LogProbArcSelector<Arc> {
   using StateId = typename Arc::StateId;
   using Weight = typename Arc::Weight;
 
+  using LogProbArcSelector<Arc>::MutableRand;
+  using LogProbArcSelector<Arc>::ToLogWeight;
   using LogProbArcSelector<Arc>::operator();
 
   // Constructs a selector with a non-deterministic seed.
-  FastLogProbArcSelector() : seed_(std::random_device()()), rand_(seed_) {}
+  FastLogProbArcSelector() : LogProbArcSelector<Arc>() {}
   // Constructs a selector with a given seed.
-  explicit FastLogProbArcSelector(uint64 seed) : seed_(seed), rand_(seed_) {}
+  explicit FastLogProbArcSelector(uint64 seed)
+      : LogProbArcSelector<Arc>(seed) {}
 
   size_t operator()(const Fst<Arc> &fst, StateId s,
                     CacheLogAccumulator<Arc> *accumulator) const {
     accumulator->SetState(s);
     ArcIterator<Fst<Arc>> aiter(fst, s);
     // Finds total weight leaving state.
-    const double sum = to_log_weight_(accumulator->Sum(fst.Final(s), &aiter, 0,
-                                                       fst.NumArcs(s)))
-                           .Value();
-    const double r = -log(std::uniform_real_distribution<>(0, 1)(rand_));
+    const double sum =
+        ToLogWeight(accumulator->Sum(fst.Final(s), &aiter, 0, fst.NumArcs(s)))
+            .Value();
+    const double r =
+        -log(std::uniform_real_distribution<>(0, 1)(MutableRand()));
     Weight w = from_log_weight_(r + sum);
     aiter.Reset();
     return accumulator->LowerBound(w, &aiter);
   }
 
-  uint64 Seed() const { return seed_; }
-
  private:
-  const uint64 seed_;
-  mutable std::mt19937_64 rand_;
-  WeightConvert<Weight, Log64Weight> to_log_weight_;
-  WeightConvert<Log64Weight, Weight> from_log_weight_;
+  const WeightConvert<Log64Weight, Weight> from_log_weight_{};
 };
 
 // Random path state info maintained by RandGenFst and passed to samplers.
@@ -247,19 +274,21 @@ class ArcSampler {
 template <class Result, class RNG>
 void OneMultinomialSample(const std::vector<double> &probs,
                           size_t num_to_sample, Result *result, RNG *rng) {
-  // Left-over probability mass.
-  double norm = 0;
-  for (double p : probs) norm += p;
+  using distribution = std::binomial_distribution<size_t>;
+  // Left-over probability mass. Keep an array of the partial sums because
+  // keeping a scalar and modifying norm -= probs[i] in the loop will result
+  // in round-off error and can have probs[i] > norm.
+  std::vector<double> norm(probs.size());
+  std::partial_sum(probs.rbegin(), probs.rend(), norm.rbegin());
   // Left-over number of samples needed.
   for (size_t i = 0; i < probs.size(); ++i) {
-    size_t num_sampled = 0;
+    distribution::result_type num_sampled = 0;
     if (probs[i] > 0) {
-      std::binomial_distribution<> d(num_to_sample, probs[i] / norm);
+      distribution d(num_to_sample, probs[i] / norm[i]);
       num_sampled = d(*rng);
     }
     if (num_sampled != 0) (*result)[i] = num_sampled;
-    norm -= probs[i];
-    num_to_sample -= num_sampled;
+    num_to_sample -= std::min(num_sampled, num_to_sample);
   }
 }
 
@@ -289,10 +318,10 @@ class ArcSampler<Arc, FastLogProbArcSelector<Arc>> {
         selector_(sampler.selector_),
         max_length_(sampler.max_length_) {
     if (fst) {
-      accumulator_.reset(new Accumulator());
+      accumulator_ = std::make_unique<Accumulator>();
       accumulator_->Init(*fst);
     } else {  // Shallow copy.
-      accumulator_.reset(new Accumulator(*sampler.accumulator_));
+      accumulator_ = std::make_unique<Accumulator>(*sampler.accumulator_);
     }
   }
 
@@ -360,7 +389,7 @@ class ArcSampler<Arc, FastLogProbArcSelector<Arc>> {
   std::unique_ptr<Accumulator> accumulator_;
   RNG rng_;                // Random number generator.
   std::vector<double> p_;  // Multinomial parameters.
-  WeightConvert<Weight, Log64Weight> to_log_weight_;
+  const WeightConvert<Weight, Log64Weight> to_log_weight_{};
 };
 
 // Options for random path generation with RandGenFst. The template argument is
@@ -394,7 +423,7 @@ class RandGenFstImpl : public CacheImpl<ToArc> {
   using FstImpl<ToArc>::SetInputSymbols;
   using FstImpl<ToArc>::SetOutputSymbols;
 
-  using CacheBaseImpl<CacheState<ToArc>>::PushArc;
+  using CacheBaseImpl<CacheState<ToArc>>::EmplaceArc;
   using CacheBaseImpl<CacheState<ToArc>>::HasArcs;
   using CacheBaseImpl<CacheState<ToArc>>::HasFinal;
   using CacheBaseImpl<CacheState<ToArc>>::HasStart;
@@ -489,7 +518,7 @@ class RandGenFstImpl : public CacheImpl<ToArc> {
   // states as needed.
   void Expand(StateId s) {
     if (s == superfinal_) {
-      SetFinal(s, ToWeight::One());
+      SetFinal(s);
       SetArcs(s);
       return;
     }
@@ -506,10 +535,10 @@ class RandGenFstImpl : public CacheImpl<ToArc> {
       if (pos < narcs) {  // Regular transition.
         aiter.Seek(sample_pair.first);
         const auto &aarc = aiter.Value();
-        const auto weight =
+        auto weight =
             weighted_ ? to_weight_(Log64Weight(-log(prob))) : ToWeight::One();
-        const ToArc barc(aarc.ilabel, aarc.olabel, weight, state_table_.size());
-        PushArc(s, barc);
+        EmplaceArc(s, aarc.ilabel, aarc.olabel, std::move(weight),
+                   state_table_.size());
         auto *nrstate = new RandState<FromArc>(aarc.nextstate, count,
                                                rstate.length + 1, pos, &rstate);
         state_table_.emplace_back(nrstate);
@@ -526,10 +555,7 @@ class RandGenFstImpl : public CacheImpl<ToArc> {
             state_table_.emplace_back(
                 new RandState<FromArc>(kNoStateId, 0, 0, 0, nullptr));
           }
-          for (size_t n = 0; n < count; ++n) {
-            const ToArc barc(0, 0, ToWeight::One(), superfinal_);
-            PushArc(s, barc);
-          }
+          for (size_t n = 0; n < count; ++n) EmplaceArc(s, 0, 0, superfinal_);
         }
       }
     }
@@ -544,7 +570,7 @@ class RandGenFstImpl : public CacheImpl<ToArc> {
   const bool weighted_;
   bool remove_total_weight_;
   StateId superfinal_;
-  WeightConvert<Log64Weight, ToWeight> to_weight_;
+  const WeightConvert<Log64Weight, ToWeight> to_weight_{};
 };
 
 }  // namespace internal
@@ -571,12 +597,12 @@ class RandGenFst
       : ImplToFst<Impl>(std::make_shared<Impl>(fst, opts)) {}
 
   // See Fst<>::Copy() for doc.
-  RandGenFst(const RandGenFst<FromArc, ToArc, Sampler> &fst, bool safe = false)
+  RandGenFst(const RandGenFst &fst, bool safe = false)
       : ImplToFst<Impl>(fst, safe) {}
 
   // Get a copy of this RandGenFst. See Fst<>::Copy() for further doc.
-  RandGenFst<FromArc, ToArc, Sampler> *Copy(bool safe = false) const override {
-    return new RandGenFst<FromArc, ToArc, Sampler>(*this, safe);
+  RandGenFst *Copy(bool safe = false) const override {
+    return new RandGenFst(*this, safe);
   }
 
   inline void InitStateIterator(StateIteratorData<ToArc> *data) const override;
@@ -619,7 +645,9 @@ class ArcIterator<RandGenFst<FromArc, ToArc, Sampler>>
 template <class FromArc, class ToArc, class Sampler>
 inline void RandGenFst<FromArc, ToArc, Sampler>::InitStateIterator(
     StateIteratorData<ToArc> *data) const {
-  data->base = new StateIterator<RandGenFst<FromArc, ToArc, Sampler>>(*this);
+  data->base =
+      std::make_unique<StateIterator<RandGenFst<FromArc, ToArc, Sampler>>>(
+          *this);
 }
 
 // Options for random path generation.
@@ -703,7 +731,7 @@ class RandGenVisitor {
       ofst_->AddArc(src, arc);
       src = dest;
     }
-    ofst_->SetFinal(src, Weight::One());
+    ofst_->SetFinal(src);
   }
 
   const Fst<FromArc> *ifst_;
@@ -721,8 +749,6 @@ class RandGenVisitor {
 template <class FromArc, class ToArc, class Selector>
 void RandGen(const Fst<FromArc> &ifst, MutableFst<ToArc> *ofst,
              const RandGenOptions<Selector> &opts) {
-  using State = typename ToArc::StateId;
-  using Weight = typename ToArc::Weight;
   using Sampler = ArcSampler<FromArc, Selector>;
   auto *sampler = new Sampler(ifst, opts.selector, opts.max_length);
   RandGenFstOptions<Sampler> fopts(CacheOptions(true, 0), sampler, opts.npath,
@@ -739,8 +765,9 @@ void RandGen(const Fst<FromArc> &ifst, MutableFst<ToArc> *ofst,
 // Randomly generate a path through an FST with the uniform distribution
 // over the transitions.
 template <class FromArc, class ToArc>
-void RandGen(const Fst<FromArc> &ifst, MutableFst<ToArc> *ofst) {
-  const UniformArcSelector<FromArc> uniform_selector;
+void RandGen(const Fst<FromArc> &ifst, MutableFst<ToArc> *ofst,
+             uint64 seed = std::random_device()()) {
+  const UniformArcSelector<FromArc> uniform_selector(seed);
   RandGenOptions<UniformArcSelector<ToArc>> opts(uniform_selector);
   RandGen(ifst, ofst, opts);
 }
