@@ -1,3 +1,17 @@
+// Copyright 2005-2020 Google LLC
+//
+// Licensed under the Apache License, Version 2.0 (the 'License');
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+//
+//     http://www.apache.org/licenses/LICENSE-2.0
+//
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an 'AS IS' BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
+//
 // See www.openfst.org for extensive documentation on this weighted
 // finite-state transducer library.
 //
@@ -11,11 +25,12 @@
 #include <string>
 #include <vector>
 
+#include <fst/types.h>
+
 #include <fst/mutable-fst.h>
 #include <fst/replace.h>
 #include <fst/test-properties.h>
 
-
 namespace fst {
 
 using RationalFstOptions = CacheOptions;
@@ -61,7 +76,7 @@ class RationalFstImpl : public FstImpl<A> {
   explicit RationalFstImpl(const RationalFstOptions &opts)
       : nonterminals_(0), replace_options_(opts, 0) {
     SetType("rational");
-    fst_tuples_.push_back(std::make_pair(0, nullptr));
+    fst_tuples_.emplace_back(0, nullptr);
   }
 
   RationalFstImpl(const RationalFstImpl<Arc> &impl)
@@ -113,14 +128,14 @@ class RationalFstImpl : public FstImpl<A> {
     rfst_.AddState();
     rfst_.AddState();
     rfst_.SetStart(0);
-    rfst_.SetFinal(1, Weight::One());
+    rfst_.SetFinal(1);
     rfst_.SetInputSymbols(fst1.InputSymbols());
     rfst_.SetOutputSymbols(fst1.OutputSymbols());
     nonterminals_ = 2;
-    rfst_.AddArc(0, Arc(0, -1, Weight::One(), 1));
-    rfst_.AddArc(0, Arc(0, -2, Weight::One(), 1));
-    fst_tuples_.push_back(std::make_pair(-1, fst1.Copy()));
-    fst_tuples_.push_back(std::make_pair(-2, fst2.Copy()));
+    rfst_.EmplaceArc(0, 0, -1, Weight::One(), 1);
+    rfst_.EmplaceArc(0, 0, -2, Weight::One(), 1);
+    fst_tuples_.emplace_back(-1, fst1.Copy());
+    fst_tuples_.emplace_back(-2, fst2.Copy());
     SetProperties(UnionProperties(props1, props2, true), kCopyProperties);
   }
 
@@ -135,14 +150,14 @@ class RationalFstImpl : public FstImpl<A> {
     rfst_.AddState();
     rfst_.AddState();
     rfst_.SetStart(0);
-    rfst_.SetFinal(2, Weight::One());
+    rfst_.SetFinal(2);
     rfst_.SetInputSymbols(fst1.InputSymbols());
     rfst_.SetOutputSymbols(fst1.OutputSymbols());
     nonterminals_ = 2;
-    rfst_.AddArc(0, Arc(0, -1, Weight::One(), 1));
-    rfst_.AddArc(1, Arc(0, -2, Weight::One(), 2));
-    fst_tuples_.push_back(std::make_pair(-1, fst1.Copy()));
-    fst_tuples_.push_back(std::make_pair(-2, fst2.Copy()));
+    rfst_.EmplaceArc(0, 0, -1, Weight::One(), 1);
+    rfst_.EmplaceArc(1, 0, -2, Weight::One(), 2);
+    fst_tuples_.emplace_back(-1, fst1.Copy());
+    fst_tuples_.emplace_back(-2, fst2.Copy());
     SetProperties(ConcatProperties(props1, props2, true), kCopyProperties);
   }
 
@@ -155,19 +170,19 @@ class RationalFstImpl : public FstImpl<A> {
     if (closure_type == CLOSURE_STAR) {
       rfst_.AddState();
       rfst_.SetStart(0);
-      rfst_.SetFinal(0, Weight::One());
-      rfst_.AddArc(0, Arc(0, -1, Weight::One(), 0));
+      rfst_.SetFinal(0);
+      rfst_.EmplaceArc(0, 0, -1, Weight::One(), 0);
     } else {
       rfst_.AddState();
       rfst_.AddState();
       rfst_.SetStart(0);
-      rfst_.SetFinal(1, Weight::One());
-      rfst_.AddArc(0, Arc(0, -1, Weight::One(), 1));
-      rfst_.AddArc(1, Arc(0, 0, Weight::One(), 0));
+      rfst_.SetFinal(1);
+      rfst_.EmplaceArc(0, 0, -1, Weight::One(), 1);
+      rfst_.EmplaceArc(1, 0, 0, Weight::One(), 0);
     }
     rfst_.SetInputSymbols(fst.InputSymbols());
     rfst_.SetOutputSymbols(fst.OutputSymbols());
-    fst_tuples_.push_back(std::make_pair(-1, fst.Copy()));
+    fst_tuples_.emplace_back(-1, fst.Copy());
     nonterminals_ = 1;
     SetProperties(ClosureProperties(props, closure_type == CLOSURE_STAR, true),
                   kCopyProperties);
@@ -182,11 +197,11 @@ class RationalFstImpl : public FstImpl<A> {
     afst.AddState();
     afst.AddState();
     afst.SetStart(0);
-    afst.SetFinal(1, Weight::One());
+    afst.SetFinal(1);
     ++nonterminals_;
-    afst.AddArc(0, Arc(0, -nonterminals_, Weight::One(), 1));
+    afst.EmplaceArc(0, 0, -nonterminals_, Weight::One(), 1);
     Union(&rfst_, afst);
-    fst_tuples_.push_back(std::make_pair(-nonterminals_, fst.Copy()));
+    fst_tuples_.emplace_back(-nonterminals_, fst.Copy());
     SetProperties(UnionProperties(props1, props2, true), kCopyProperties);
   }
 
@@ -199,15 +214,15 @@ class RationalFstImpl : public FstImpl<A> {
     afst.AddState();
     afst.AddState();
     afst.SetStart(0);
-    afst.SetFinal(1, Weight::One());
+    afst.SetFinal(1);
     ++nonterminals_;
-    afst.AddArc(0, Arc(0, -nonterminals_, Weight::One(), 1));
+    afst.EmplaceArc(0, 0, -nonterminals_, Weight::One(), 1);
     if (append) {
       Concat(&rfst_, afst);
     } else {
       Concat(afst, &rfst_);
     }
-    fst_tuples_.push_back(std::make_pair(-nonterminals_, fst.Copy()));
+    fst_tuples_.emplace_back(-nonterminals_, fst.Copy());
     SetProperties(ConcatProperties(props1, props2, true), kCopyProperties);
   }
 
@@ -225,7 +240,8 @@ class RationalFstImpl : public FstImpl<A> {
   ReplaceFst<Arc> *Replace() const {
     if (!replace_) {
       fst_tuples_[0].second = rfst_.Copy();
-      replace_.reset(new ReplaceFst<Arc>(fst_tuples_, replace_options_));
+      replace_ =
+          std::make_unique<ReplaceFst<Arc>>(fst_tuples_, replace_options_);
     }
     return replace_.get();
   }
@@ -277,7 +293,7 @@ class RationalFst : public ImplToFst<internal::RationalFstImpl<A>> {
       : ImplToFst<Impl>(std::make_shared<Impl>(opts)) {}
 
   // See Fst<>::Copy() for doc.
-  RationalFst(const RationalFst<Arc> &fst, bool safe = false)
+  RationalFst(const RationalFst &fst, bool safe = false)
       : ImplToFst<Impl>(fst, safe) {}
 
  private:
