@@ -1,36 +1,37 @@
-
-// Licensed under the Apache License, Version 2.0 (the "License");
+// Copyright 2005-2020 Google LLC
+//
+// Licensed under the Apache License, Version 2.0 (the 'License');
 // you may not use this file except in compliance with the License.
 // You may obtain a copy of the License at
 //
 //     http://www.apache.org/licenses/LICENSE-2.0
 //
 // Unless required by applicable law or agreed to in writing, software
-// distributed under the License is distributed on an "AS IS" BASIS,
+// distributed under the License is distributed on an 'AS IS' BASIS,
 // WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 // See the License for the specific language governing permissions and
 // limitations under the License.
 //
-// Copyright 2005-2010 Google, Inc.
-// Author: jpr@google.com (Jake Ratkiewicz)
+// See www.openfst.org for extensive documentation on this weighted
+// finite-state transducer library.
 
 #ifndef FST_SCRIPT_CLOSURE_H_
 #define FST_SCRIPT_CLOSURE_H_
 
-#include <fst/script/arg-packs.h>
-#include <fst/script/fst-class.h>
+#include <utility>
+
 #include <fst/closure.h>
+#include <fst/script/fst-class.h>
 
 namespace fst {
 namespace script {
 
-typedef args::Package<MutableFstClass*, const ClosureType> ClosureArgs;
+using ClosureArgs = std::pair<MutableFstClass *, const ClosureType>;
 
-template<class Arc>
+template <class Arc>
 void Closure(ClosureArgs *args) {
-  MutableFst<Arc> *fst = args->arg1->GetMutableFst<Arc>();
-
-  Closure(fst, args->arg2);
+  MutableFst<Arc> *fst = std::get<0>(*args)->GetMutableFst<Arc>();
+  Closure(fst, std::get<1>(*args));
 }
 
 void Closure(MutableFstClass *ofst, ClosureType closure_type);
