@@ -1,3 +1,17 @@
+// Copyright 2005-2020 Google LLC
+//
+// Licensed under the Apache License, Version 2.0 (the 'License');
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+//
+//     http://www.apache.org/licenses/LICENSE-2.0
+//
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an 'AS IS' BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
+//
 // See www.openfst.org for extensive documentation on this weighted
 // finite-state transducer library.
 //
@@ -16,6 +30,7 @@
 #include <fst/util.h>
 #include <fst/vector-fst.h>
 #include <unordered_map>
+#include <string_view>
 
 DECLARE_string(fst_field_separator);
 
@@ -36,18 +51,18 @@ class FstCompiler {
   // symbol tables. This is only useful if you set the (i/o)keep flag to attach
   // the final symbol table, or use the accessors. (The input symbol tables are
   // const and therefore not changed.)
-  FstCompiler(std::istream &istrm, const string &source,  // NOLINT
+  FstCompiler(std::istream &istrm, const std::string &source,
               const SymbolTable *isyms, const SymbolTable *osyms,
-              const SymbolTable *ssyms, bool accep, bool ikeep,
-              bool okeep, bool nkeep, bool allow_negative_labels = false) {
+              const SymbolTable *ssyms, bool accep, bool ikeep, bool okeep,
+              bool nkeep, bool allow_negative_labels = false) {
     std::unique_ptr<SymbolTable> misyms(isyms ? isyms->Copy() : nullptr);
     std::unique_ptr<SymbolTable> mosyms(osyms ? osyms->Copy() : nullptr);
     std::unique_ptr<SymbolTable> mssyms(ssyms ? ssyms->Copy() : nullptr);
-    Init(istrm, source, misyms.get(), mosyms.get(), mssyms.get(), accep,
-         ikeep, okeep, nkeep, allow_negative_labels, false);
+    Init(istrm, source, misyms.get(), mosyms.get(), mssyms.get(), accep, ikeep,
+         okeep, nkeep, allow_negative_labels, false);
   }
 
-  FstCompiler(std::istream &istrm, const string &source,  // NOLINT
+  FstCompiler(std::istream &istrm, const std::string &source,
               SymbolTable *isyms, SymbolTable *osyms, SymbolTable *ssyms,
               bool accep, bool ikeep, bool okeep, bool nkeep,
               bool allow_negative_labels, bool add_symbols) {
@@ -55,10 +70,10 @@ class FstCompiler {
          allow_negative_labels, add_symbols);
   }
 
-  void Init(std::istream &istrm, const string &source,  // NOLINT
-            SymbolTable *isyms, SymbolTable *osyms, SymbolTable *ssyms,
-            bool accep, bool ikeep, bool okeep, bool nkeep,
-            bool allow_negative_labels, bool add_symbols) {
+  void Init(std::istream &istrm, const std::string &source, SymbolTable *isyms,
+            SymbolTable *osyms, SymbolTable *ssyms, bool accep, bool ikeep,
+            bool okeep, bool nkeep, bool allow_negative_labels,
+            bool add_symbols) {
     nline_ = 0;
     source_ = source;
     isyms_ = isyms;
@@ -70,13 +85,12 @@ class FstCompiler {
     add_symbols_ = add_symbols;
     bool start_state_populated = false;
     char line[kLineLen];
-    const string separator = FLAGS_fst_field_separator + "\n";
+    const std::string separator =
+        FST_FLAGS_fst_field_separator + "\n";
     while (istrm.getline(line, kLineLen)) {
       ++nline_;
-      std::vector<char *> col;
-      SplitString(line, separator.c_str(), &col, true);
-      if (col.empty() || col[0][0] == '\0')
-        continue;
+      std::vector<std::string_view> col = SplitString(line, separator, true);
+      if (col.empty() || col[0].empty()) continue;
       if (col.size() > 5 || (col.size() > 4 && accep) ||
           (col.size() == 3 && !accep)) {
         FSTERROR() << "FstCompiler: Bad number of columns, source = " << source_
@@ -138,7 +152,7 @@ class FstCompiler {
   // Maximum line length in text file.
   static constexpr int kLineLen = 8096;
 
-  StateId StrToId(const char *s, SymbolTable *syms, const char *name,
+  StateId StrToId(std::string_view s, SymbolTable *syms, const char *name,
                   bool allow_negative = false) const {
     StateId n = 0;
     if (syms) {
@@ -151,18 +165,18 @@ class FstCompiler {
         fst_.SetProperties(kError, kError);
       }
     } else {
-      char *p;
-      n = strtoll(s, &p, 10);
-      if (p < s + strlen(s) || (!allow_negative && n < 0)) {
+      auto maybe_n = ParseInt64(s);
+      if (!maybe_n.has_value() || (!allow_negative && *maybe_n < 0)) {
         FSTERROR() << "FstCompiler: Bad " << name << " integer = \"" << s
                    << "\", source = " << source_ << ", line = " << nline_;
         fst_.SetProperties(kError, kError);
       }
+      n = *maybe_n;
     }
     return n;
   }
 
-  StateId StrToStateId(const char *s) {
+  StateId StrToStateId(std::string_view s) {
     StateId n = StrToId(s, ssyms_, "state ID");
     if (keep_state_numbering_) return n;
     // Remaps state IDs to make dense set.
@@ -175,17 +189,17 @@ class FstCompiler {
     }
   }
 
-  StateId StrToILabel(const char *s) const {
+  StateId StrToILabel(std::string_view s) const {
     return StrToId(s, isyms_, "arc ilabel", allow_negative_labels_);
   }
 
-  StateId StrToOLabel(const char *s) const {
+  StateId StrToOLabel(std::string_view s) const {
     return StrToId(s, osyms_, "arc olabel", allow_negative_labels_);
   }
 
-  Weight StrToWeight(const char *s, bool allow_zero) const {
+  Weight StrToWeight(std::string_view s, bool allow_zero) const {
     Weight w;
-    std::istringstream strm(s);
+    std::istringstream strm(std::string{s});
     strm >> w;
     if (!strm || (!allow_zero && w == Weight::Zero())) {
       FSTERROR() << "FstCompiler: Bad weight = \"" << s
@@ -198,12 +212,12 @@ class FstCompiler {
 
   mutable VectorFst<Arc> fst_;
   size_t nline_;
-  string source_;       // Text FST source name.
+  std::string source_;  // Text FST source name.
   SymbolTable *isyms_;  // ilabel symbol table (not owned).
   SymbolTable *osyms_;  // olabel symbol table (not owned).
   SymbolTable *ssyms_;  // slabel symbol table (not owned).
   std::unordered_map<StateId, StateId> states_;  // State ID map.
-  StateId nstates_;                              // Number of seen states.
+  StateId nstates_;                               // Number of seen states.
   bool keep_state_numbering_;
   bool allow_negative_labels_;  // Not recommended; may cause conflicts.
   bool add_symbols_;            // Add to symbol tables on-the fly.
