@@ -1,3 +1,17 @@
+// Copyright 2005-2020 Google LLC
+//
+// Licensed under the Apache License, Version 2.0 (the 'License');
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+//
+//     http://www.apache.org/licenses/LICENSE-2.0
+//
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an 'AS IS' BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
+//
 // See www.openfst.org for extensive documentation on this weighted
 // finite-state transducer library.
 
@@ -34,18 +48,19 @@ struct DeterminizeOptions {
         increment_subsequential_label(increment_subsequential_label) {}
 };
 
-using DeterminizeArgs = std::tuple<const FstClass &, MutableFstClass *,
-                                   const DeterminizeOptions &>;
+using DeterminizeArgs =
+    std::tuple<const FstClass &, MutableFstClass *, const DeterminizeOptions &>;
 
 template <class Arc>
 void Determinize(DeterminizeArgs *args) {
   using Weight = typename Arc::Weight;
-  const Fst<Arc> &ifst = *(std::get<0>(*args).GetFst<Arc>());
+  const Fst<Arc> &ifst = *std::get<0>(*args).GetFst<Arc>();
   MutableFst<Arc> *ofst = std::get<1>(*args)->GetMutableFst<Arc>();
   const auto &opts = std::get<2>(*args);
-  const auto weight_threshold = *(opts.weight_threshold.GetWeight<Weight>());
-  const fst::DeterminizeOptions<Arc> detargs(opts.delta, weight_threshold,
-      opts.state_threshold, opts.subsequential_label, opts.det_type,
+  const auto weight_threshold = *opts.weight_threshold.GetWeight<Weight>();
+  const fst::DeterminizeOptions<Arc> detargs(
+      opts.delta, weight_threshold, opts.state_threshold,
+      opts.subsequential_label, opts.det_type,
       opts.increment_subsequential_label);
   Determinize(ifst, ofst, detargs);
 }
