@@ -1,3 +1,17 @@
+// Copyright 2005-2020 Google LLC
+//
+// Licensed under the Apache License, Version 2.0 (the 'License');
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+//
+//     http://www.apache.org/licenses/LICENSE-2.0
+//
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an 'AS IS' BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
+//
 // See www.openfst.org for extensive documentation on this weighted
 // finite-state transducer library.
 
@@ -5,12 +19,15 @@
 #define FST_SCRIPT_FST_CLASS_H_
 
 #include <algorithm>
+#include <istream>
 #include <limits>
+#include <memory>
 #include <string>
 #include <type_traits>
 
 #include <fst/expanded-fst.h>
 #include <fst/fst.h>
+#include <fst/generic-register.h>
 #include <fst/mutable-fst.h>
 #include <fst/vector-fst.h>
 #include <fst/script/arc-class.h>
@@ -33,9 +50,9 @@ namespace script {
 
 class FstClassBase {
  public:
-  virtual const string &ArcType() const = 0;
+  virtual const std::string &ArcType() const = 0;
   virtual WeightClass Final(int64) const = 0;
-  virtual const string &FstType() const = 0;
+  virtual const std::string &FstType() const = 0;
   virtual const SymbolTable *InputSymbols() const = 0;
   virtual size_t NumArcs(int64) const = 0;
   virtual size_t NumInputEpsilons(int64) const = 0;
@@ -43,10 +60,10 @@ class FstClassBase {
   virtual const SymbolTable *OutputSymbols() const = 0;
   virtual uint64 Properties(uint64, bool) const = 0;
   virtual int64 Start() const = 0;
-  virtual const string &WeightType() const = 0;
+  virtual const std::string &WeightType() const = 0;
   virtual bool ValidStateId(int64) const = 0;
-  virtual bool Write(const string &) const = 0;
-  virtual bool Write(std::ostream &, const string &) const = 0;
+  virtual bool Write(const std::string &) const = 0;
+  virtual bool Write(std::ostream &, const std::string &) const = 0;
   virtual ~FstClassBase() {}
 };
 
@@ -55,6 +72,7 @@ class FstClassImplBase : public FstClassBase {
  public:
   virtual bool AddArc(int64, const ArcClass &) = 0;
   virtual int64 AddState() = 0;
+  virtual void AddStates(size_t) = 0;
   virtual FstClassImplBase *Copy() = 0;
   virtual bool DeleteArcs(int64, size_t) = 0;
   virtual bool DeleteArcs(int64) = 0;
@@ -65,9 +83,9 @@ class FstClassImplBase : public FstClassBase {
   virtual int64 NumStates() const = 0;
   virtual bool ReserveArcs(int64, size_t) = 0;
   virtual void ReserveStates(int64) = 0;
-  virtual void SetInputSymbols(SymbolTable *) = 0;
+  virtual void SetInputSymbols(const SymbolTable *) = 0;
   virtual bool SetFinal(int64, const WeightClass &) = 0;
-  virtual void SetOutputSymbols(SymbolTable *) = 0;
+  virtual void SetOutputSymbols(const SymbolTable *) = 0;
   virtual void SetProperties(uint64, uint64) = 0;
   virtual bool SetStart(int64) = 0;
   ~FstClassImplBase() override {}
@@ -81,8 +99,8 @@ class FstClassImplBase : public FstClassBase {
 template <class Arc>
 class FstClassImpl : public FstClassImplBase {
  public:
-  explicit FstClassImpl(Fst<Arc> *impl, bool should_own = false)
-      : impl_(should_own ? impl : impl->Copy()) {}
+  explicit FstClassImpl(std::unique_ptr<Fst<Arc>> impl)
+      : impl_(std::move(impl)) {}
 
   explicit FstClassImpl(const Fst<Arc> &impl) : impl_(impl.Copy()) {}
 
@@ -94,30 +112,35 @@ class FstClassImpl : public FstClassImplBase {
     // used to determine whether any arc has a nonexisting destination.
     Arc arc(ac.ilabel, ac.olabel, *ac.weight.GetWeight<typename Arc::Weight>(),
             ac.nextstate);
-    static_cast<MutableFst<Arc> *>(impl_.get())->AddArc(s, arc);
+    fst::down_cast<MutableFst<Arc> *>(impl_.get())->AddArc(s, arc);
     return true;
   }
 
   // Warning: calling this method casts the FST to a mutable FST.
   int64 AddState() final {
-    return static_cast<MutableFst<Arc> *>(impl_.get())->AddState();
+    return fst::down_cast<MutableFst<Arc> *>(impl_.get())->AddState();
   }
 
-  const string &ArcType() const final { return Arc::Type(); }
+  // Warning: calling this method casts the FST to a mutable FST.
+  void AddStates(size_t n) final {
+    return fst::down_cast<MutableFst<Arc> *>(impl_.get())->AddStates(n);
+  }
 
-  FstClassImpl *Copy() final { return new FstClassImpl<Arc>(impl_.get()); }
+  const std::string &ArcType() const final { return Arc::Type(); }
+
+  FstClassImpl *Copy() final { return new FstClassImpl<Arc>(*impl_); }
 
   // Warning: calling this method casts the FST to a mutable FST.
   bool DeleteArcs(int64 s, size_t n) final {
     if (!ValidStateId(s)) return false;
-    static_cast<MutableFst<Arc> *>(impl_.get())->DeleteArcs(s, n);
+    fst::down_cast<MutableFst<Arc> *>(impl_.get())->DeleteArcs(s, n);
     return true;
   }
 
   // Warning: calling this method casts the FST to a mutable FST.
   bool DeleteArcs(int64 s) final {
     if (!ValidStateId(s)) return false;
-    static_cast<MutableFst<Arc> *>(impl_.get())->DeleteArcs(s);
+    fst::down_cast<MutableFst<Arc> *>(impl_.get())->DeleteArcs(s);
     return true;
   }
 
@@ -129,13 +152,13 @@ class FstClassImpl : public FstClassImplBase {
     // the underlying FST will result in truncation.
     std::vector<typename Arc::StateId> typed_dstates(dstates.size());
     std::copy(dstates.begin(), dstates.end(), typed_dstates.begin());
-    static_cast<MutableFst<Arc> *>(impl_.get())->DeleteStates(typed_dstates);
+    fst::down_cast<MutableFst<Arc> *>(impl_.get())->DeleteStates(typed_dstates);
     return true;
   }
 
   // Warning: calling this method casts the FST to a mutable FST.
   void DeleteStates() final {
-    static_cast<MutableFst<Arc> *>(impl_.get())->DeleteStates();
+    fst::down_cast<MutableFst<Arc> *>(impl_.get())->DeleteStates();
   }
 
   WeightClass Final(int64 s) const final {
@@ -144,7 +167,7 @@ class FstClassImpl : public FstClassImplBase {
     return w;
   }
 
-  const string &FstType() const final { return impl_->Type(); }
+  const std::string &FstType() const final { return impl_->Type(); }
 
   const SymbolTable *InputSymbols() const final {
     return impl_->InputSymbols();
@@ -152,12 +175,12 @@ class FstClassImpl : public FstClassImplBase {
 
   // Warning: calling this method casts the FST to a mutable FST.
   SymbolTable *MutableInputSymbols() final {
-    return static_cast<MutableFst<Arc> *>(impl_.get())->MutableInputSymbols();
+    return fst::down_cast<MutableFst<Arc> *>(impl_.get())->MutableInputSymbols();
   }
 
   // Warning: calling this method casts the FST to a mutable FST.
   SymbolTable *MutableOutputSymbols() final {
-    return static_cast<MutableFst<Arc> *>(impl_.get())->MutableOutputSymbols();
+    return fst::down_cast<MutableFst<Arc> *>(impl_.get())->MutableOutputSymbols();
   }
 
   // Signals failure by returning size_t max.
@@ -180,7 +203,7 @@ class FstClassImpl : public FstClassImplBase {
 
   // Warning: calling this method casts the FST to a mutable FST.
   int64 NumStates() const final {
-    return static_cast<MutableFst<Arc> *>(impl_.get())->NumStates();
+    return fst::down_cast<MutableFst<Arc> *>(impl_.get())->NumStates();
   }
 
   uint64 Properties(uint64 mask, bool test) const final {
@@ -190,13 +213,13 @@ class FstClassImpl : public FstClassImplBase {
   // Warning: calling this method casts the FST to a mutable FST.
   bool ReserveArcs(int64 s, size_t n) final {
     if (!ValidStateId(s)) return false;
-    static_cast<MutableFst<Arc> *>(impl_.get())->ReserveArcs(s, n);
+    fst::down_cast<MutableFst<Arc> *>(impl_.get())->ReserveArcs(s, n);
     return true;
   }
 
   // Warning: calling this method casts the FST to a mutable FST.
-  void ReserveStates(int64 s) final {
-    static_cast<MutableFst<Arc> *>(impl_.get())->ReserveStates(s);
+  void ReserveStates(int64 n) final {
+    fst::down_cast<MutableFst<Arc> *>(impl_.get())->ReserveStates(n);
   }
 
   const SymbolTable *OutputSymbols() const final {
@@ -204,32 +227,32 @@ class FstClassImpl : public FstClassImplBase {
   }
 
   // Warning: calling this method casts the FST to a mutable FST.
-  void SetInputSymbols(SymbolTable *isyms) final {
-    static_cast<MutableFst<Arc> *>(impl_.get())->SetInputSymbols(isyms);
+  void SetInputSymbols(const SymbolTable *isyms) final {
+    fst::down_cast<MutableFst<Arc> *>(impl_.get())->SetInputSymbols(isyms);
   }
 
   // Warning: calling this method casts the FST to a mutable FST.
   bool SetFinal(int64 s, const WeightClass &weight) final {
     if (!ValidStateId(s)) return false;
-    static_cast<MutableFst<Arc> *>(impl_.get())
+    fst::down_cast<MutableFst<Arc> *>(impl_.get())
         ->SetFinal(s, *weight.GetWeight<typename Arc::Weight>());
     return true;
   }
 
   // Warning: calling this method casts the FST to a mutable FST.
-  void SetOutputSymbols(SymbolTable *osyms) final {
-    static_cast<MutableFst<Arc> *>(impl_.get())->SetOutputSymbols(osyms);
+  void SetOutputSymbols(const SymbolTable *osyms) final {
+    fst::down_cast<MutableFst<Arc> *>(impl_.get())->SetOutputSymbols(osyms);
   }
 
   // Warning: calling this method casts the FST to a mutable FST.
   void SetProperties(uint64 props, uint64 mask) final {
-    static_cast<MutableFst<Arc> *>(impl_.get())->SetProperties(props, mask);
+    fst::down_cast<MutableFst<Arc> *>(impl_.get())->SetProperties(props, mask);
   }
 
   // Warning: calling this method casts the FST to a mutable FST.
   bool SetStart(int64 s) final {
     if (!ValidStateId(s)) return false;
-    static_cast<MutableFst<Arc> *>(impl_.get())->SetStart(s);
+    fst::down_cast<MutableFst<Arc> *>(impl_.get())->SetStart(s);
     return true;
   }
 
@@ -249,12 +272,14 @@ class FstClassImpl : public FstClassImplBase {
     return true;
   }
 
-  const string &WeightType() const final { return Arc::Weight::Type(); }
+  const std::string &WeightType() const final { return Arc::Weight::Type(); }
 
-  bool Write(const string &fname) const final { return impl_->Write(fname); }
+  bool Write(const std::string &source) const final {
+    return impl_->Write(source);
+  }
 
-  bool Write(std::ostream &ostr, const string &fname) const final {
-    const FstWriteOptions opts(fname);
+  bool Write(std::ostream &ostr, const std::string &source) const final {
+    const FstWriteOptions opts(source);
     return impl_->Write(ostr, opts);
   }
 
@@ -275,7 +300,12 @@ class FstClass : public FstClassBase {
   FstClass() : impl_(nullptr) {}
 
   template <class Arc>
-  explicit FstClass(const Fst<Arc> &fst) : impl_(new FstClassImpl<Arc>(fst)) {}
+  explicit FstClass(std::unique_ptr<Fst<Arc>> fst)
+      : impl_(std::make_unique<FstClassImpl<Arc>>(std::move(fst))) {}
+
+  template <class Arc>
+  explicit FstClass(const Fst<Arc> &fst)
+      : impl_(std::make_unique<FstClassImpl<Arc>>(fst)) {}
 
   FstClass(const FstClass &other)
       : impl_(other.impl_ == nullptr ? nullptr : other.impl_->Copy()) {}
@@ -287,9 +317,9 @@ class FstClass : public FstClassBase {
 
   WeightClass Final(int64 s) const final { return impl_->Final(s); }
 
-  const string &ArcType() const final { return impl_->ArcType(); }
+  const std::string &ArcType() const final { return impl_->ArcType(); }
 
-  const string &FstType() const final { return impl_->FstType(); }
+  const std::string &FstType() const final { return impl_->FstType(); }
 
   const SymbolTable *InputSymbols() const final {
     return impl_->InputSymbols();
@@ -315,25 +345,28 @@ class FstClass : public FstClassBase {
     return impl_->Properties(mask, test);
   }
 
-  static FstClass *Read(const string &fname);
+  static FstClass *Read(const std::string &source);
 
-  static FstClass *Read(std::istream &istrm, const string &source);
+  static FstClass *Read(std::istream &istrm, const std::string &source);
 
   int64 Start() const final { return impl_->Start(); }
 
   bool ValidStateId(int64 s) const final { return impl_->ValidStateId(s); }
 
-  const string &WeightType() const final { return impl_->WeightType(); }
+  const std::string &WeightType() const final { return impl_->WeightType(); }
 
   // Helper that logs an ERROR if the weight type of an FST and a WeightClass
   // don't match.
 
-  bool WeightTypesMatch(const WeightClass &weight, const string &op_name) const;
+  bool WeightTypesMatch(const WeightClass &weight,
+                        const std::string &op_name) const;
 
-  bool Write(const string &fname) const final { return impl_->Write(fname); }
+  bool Write(const std::string &source) const final {
+    return impl_->Write(source);
+  }
 
-  bool Write(std::ostream &ostr, const string &fname) const final {
-    return impl_->Write(ostr, fname);
+  bool Write(std::ostream &ostr, const std::string &source) const final {
+    return impl_->Write(ostr, source);
   }
 
   ~FstClass() override {}
@@ -359,7 +392,7 @@ class FstClass : public FstClassBase {
       return nullptr;
     } else {
       FstClassImpl<Arc> *typed_impl =
-          static_cast<FstClassImpl<Arc> *>(impl_.get());
+          fst::down_cast<FstClassImpl<Arc> *>(impl_.get());
       return typed_impl->GetImpl();
     }
   }
@@ -379,20 +412,21 @@ class FstClass : public FstClassBase {
   }
 
  protected:
-  explicit FstClass(FstClassImplBase *impl) : impl_(impl) {}
+  explicit FstClass(std::unique_ptr<FstClassImplBase> impl)
+      : impl_(std::move(impl)) {}
 
   const FstClassImplBase *GetImpl() const { return impl_.get(); }
 
   FstClassImplBase *GetImpl() { return impl_.get(); }
 
   // Generic template method for reading an arc-templated FST of type
-  // UnderlyingT, and returning it wrapped as FstClassT, with appropriat
+  // UnderlyingT, and returning it wrapped as FstClassT, with appropriate
   // error checking. Called from arc-templated Read() static methods.
   template <class FstClassT, class UnderlyingT>
   static FstClassT *ReadTypedFst(std::istream &stream,
                                  const FstReadOptions &opts) {
     std::unique_ptr<UnderlyingT> u(UnderlyingT::Read(stream, opts));
-    return u ? new FstClassT(*u) : nullptr;
+    return u ? new FstClassT(std::move(u)) : nullptr;
   }
 
  private:
@@ -410,6 +444,8 @@ class MutableFstClass : public FstClass {
 
   int64 AddState() { return GetImpl()->AddState(); }
 
+  void AddStates(size_t n) { return GetImpl()->AddStates(n); }
+
   bool DeleteArcs(int64 s, size_t n) { return GetImpl()->DeleteArcs(s, n); }
 
   bool DeleteArcs(int64 s) { return GetImpl()->DeleteArcs(s); }
@@ -432,11 +468,11 @@ class MutableFstClass : public FstClass {
 
   bool ReserveArcs(int64 s, size_t n) { return GetImpl()->ReserveArcs(s, n); }
 
-  void ReserveStates(int64 s) { GetImpl()->ReserveStates(s); }
+  void ReserveStates(int64 n) { GetImpl()->ReserveStates(n); }
 
-  static MutableFstClass *Read(const string &fname, bool convert = false);
+  static MutableFstClass *Read(const std::string &source, bool convert = false);
 
-  void SetInputSymbols(SymbolTable *isyms) {
+  void SetInputSymbols(const SymbolTable *isyms) {
     GetImpl()->SetInputSymbols(isyms);
   }
 
@@ -445,7 +481,7 @@ class MutableFstClass : public FstClass {
     return GetImpl()->SetFinal(s, weight);
   }
 
-  void SetOutputSymbols(SymbolTable *osyms) {
+  void SetOutputSymbols(const SymbolTable *osyms) {
     GetImpl()->SetOutputSymbols(osyms);
   }
 
@@ -455,6 +491,13 @@ class MutableFstClass : public FstClass {
 
   bool SetStart(int64 s) { return GetImpl()->SetStart(s); }
 
+  template <class Arc>
+  explicit MutableFstClass(std::unique_ptr<MutableFst<Arc>> fst)
+      // NB: The natural cast-less way to do this doesn't compile for some
+      // arcane reason.
+      : FstClass(
+            fst::implicit_cast<std::unique_ptr<Fst<Arc>>>(std::move(fst))) {}
+
   template <class Arc>
   explicit MutableFstClass(const MutableFst<Arc> &fst) : FstClass(fst) {}
 
@@ -477,7 +520,7 @@ class MutableFstClass : public FstClass {
   template <class Arc>
   MutableFst<Arc> *GetMutableFst() {
     Fst<Arc> *fst = const_cast<Fst<Arc> *>(this->GetFst<Arc>());
-    MutableFst<Arc> *mfst = static_cast<MutableFst<Arc> *>(fst);
+    MutableFst<Arc> *mfst = fst::down_cast<MutableFst<Arc> *>(fst);
     return mfst;
   }
 
@@ -489,18 +532,20 @@ class MutableFstClass : public FstClass {
   }
 
  protected:
-  explicit MutableFstClass(FstClassImplBase *impl) : FstClass(impl) {}
+  explicit MutableFstClass(std::unique_ptr<FstClassImplBase> impl)
+      : FstClass(std::move(impl)) {}
 };
 
 class VectorFstClass : public MutableFstClass {
  public:
-  explicit VectorFstClass(FstClassImplBase *impl) : MutableFstClass(impl) {}
+  explicit VectorFstClass(std::unique_ptr<FstClassImplBase> impl)
+      : MutableFstClass(std::move(impl)) {}
 
   explicit VectorFstClass(const FstClass &other);
 
-  explicit VectorFstClass(const string &arc_type);
+  explicit VectorFstClass(const std::string &arc_type);
 
-  static VectorFstClass *Read(const string &fname);
+  static VectorFstClass *Read(const std::string &source);
 
   template <class Arc>
   static VectorFstClass *Read(std::istream &stream,
@@ -509,21 +554,105 @@ class VectorFstClass : public MutableFstClass {
     return mfst ? new VectorFstClass(*mfst) : nullptr;
   }
 
+  template <class Arc>
+  explicit VectorFstClass(std::unique_ptr<VectorFst<Arc>> fst)
+      // NB: The natural cast-less way to do this doesn't compile for some
+      // arcane reason.
+      : MutableFstClass(fst::implicit_cast<std::unique_ptr<MutableFst<Arc>>>(
+            std::move(fst))) {}
+
   template <class Arc>
   explicit VectorFstClass(const VectorFst<Arc> &fst) : MutableFstClass(fst) {}
 
   template <class Arc>
   static FstClassImplBase *Convert(const FstClass &other) {
-    return new FstClassImpl<Arc>(new VectorFst<Arc>(*other.GetFst<Arc>()),
-                                 true);
+    return new FstClassImpl<Arc>(
+        std::make_unique<VectorFst<Arc>>(*other.GetFst<Arc>()));
   }
 
   template <class Arc>
   static FstClassImplBase *Create() {
-    return new FstClassImpl<Arc>(new VectorFst<Arc>(), true);
+    return new FstClassImpl<Arc>(std::make_unique<VectorFst<Arc>>());
   }
 };
 
+// Registration stuff.
+
+// This class definition is to avoid a nested class definition inside the
+// FstClassIORegistration struct.
+template <class Reader, class Creator, class Converter>
+struct FstClassRegEntry {
+  Reader reader;
+  Creator creator;
+  Converter converter;
+
+  FstClassRegEntry(Reader r, Creator cr, Converter co)
+      : reader(r), creator(cr), converter(co) {}
+
+  FstClassRegEntry() : reader(nullptr), creator(nullptr), converter(nullptr) {}
+};
+
+// Actual FST IO method register.
+template <class Reader, class Creator, class Converter>
+class FstClassIORegister
+    : public GenericRegister<std::string,
+                             FstClassRegEntry<Reader, Creator, Converter>,
+                             FstClassIORegister<Reader, Creator, Converter>> {
+ public:
+  Reader GetReader(const std::string &arc_type) const {
+    return this->GetEntry(arc_type).reader;
+  }
+
+  Creator GetCreator(const std::string &arc_type) const {
+    return this->GetEntry(arc_type).creator;
+  }
+
+  Converter GetConverter(const std::string &arc_type) const {
+    return this->GetEntry(arc_type).converter;
+  }
+
+ protected:
+  std::string ConvertKeyToSoFilename(const std::string &key) const final {
+    std::string legal_type(key);
+    ConvertToLegalCSymbol(&legal_type);
+    return legal_type + "-arc.so";
+  }
+};
+
+// Struct containing everything needed to register a particular type
+// of FST class (e.g., a plain FstClass, or a MutableFstClass, etc.).
+template <class FstClassType>
+struct FstClassIORegistration {
+  using Reader = FstClassType *(*)(std::istream &stream,
+                                   const FstReadOptions &opts);
+
+  using Creator = FstClassImplBase *(*)();
+
+  using Converter = FstClassImplBase *(*)(const FstClass &other);
+
+  using Entry = FstClassRegEntry<Reader, Creator, Converter>;
+
+  // FST class Register.
+  using Register = FstClassIORegister<Reader, Creator, Converter>;
+
+  // FST class Register-er.
+  using Registerer =
+      GenericRegisterer<FstClassIORegister<Reader, Creator, Converter>>;
+};
+
+// Macros for registering other arc types.
+
+#define REGISTER_FST_CLASS(Class, Arc)                                         \
+  static FstClassIORegistration<Class>::Registerer Class##_##Arc##_registerer( \
+      Arc::Type(),                                                             \
+      FstClassIORegistration<Class>::Entry(                                    \
+          Class::Read<Arc>, Class::Create<Arc>, Class::Convert<Arc>))
+
+#define REGISTER_FST_CLASSES(Arc)           \
+  REGISTER_FST_CLASS(FstClass, Arc);        \
+  REGISTER_FST_CLASS(MutableFstClass, Arc); \
+  REGISTER_FST_CLASS(VectorFstClass, Arc);
+
 }  // namespace script
 }  // namespace fst
 
