@@ -1,3 +1,17 @@
+// Copyright 2005-2020 Google LLC
+//
+// Licensed under the Apache License, Version 2.0 (the 'License');
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+//
+//     http://www.apache.org/licenses/LICENSE-2.0
+//
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an 'AS IS' BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
+//
 // See www.openfst.org for extensive documentation on this weighted
 // finite-state transducer library.
 //
@@ -92,37 +106,33 @@ class AllFstOperationsRegisterer {
     REGISTER_FST_OPERATION(Compose, Arc, ComposeArgs);
     REGISTER_FST_OPERATION(Concat, Arc, ConcatArgs1);
     REGISTER_FST_OPERATION(Concat, Arc, ConcatArgs2);
+    REGISTER_FST_OPERATION(Concat, Arc, ConcatArgs3);
     REGISTER_FST_OPERATION(Connect, Arc, MutableFstClass);
     REGISTER_FST_OPERATION(Convert, Arc, ConvertArgs);
-    REGISTER_FST_OPERATION(Decode, Arc, DecodeArgs1);
-    REGISTER_FST_OPERATION(Decode, Arc, DecodeArgs2);
+    REGISTER_FST_OPERATION(Decode, Arc, DecodeArgs);
     REGISTER_FST_OPERATION(Determinize, Arc, DeterminizeArgs);
     REGISTER_FST_OPERATION(Difference, Arc, DifferenceArgs);
     REGISTER_FST_OPERATION(Disambiguate, Arc, DisambiguateArgs);
-    REGISTER_FST_OPERATION(DrawFst, Arc, FstDrawerArgs);
-    REGISTER_FST_OPERATION(Encode, Arc, EncodeArgs1);
-    REGISTER_FST_OPERATION(Encode, Arc, EncodeArgs2);
+    REGISTER_FST_OPERATION(Draw, Arc, DrawArgs);
+    REGISTER_FST_OPERATION(Encode, Arc, EncodeArgs);
     REGISTER_FST_OPERATION(EpsNormalize, Arc, EpsNormalizeArgs);
     REGISTER_FST_OPERATION(Equal, Arc, EqualArgs);
     REGISTER_FST_OPERATION(Equivalent, Arc, EquivalentArgs);
-    REGISTER_FST_OPERATION(PrintFstInfo, Arc, InfoArgs);
-    REGISTER_FST_OPERATION(GetFstInfo, Arc, GetInfoArgs);
-    REGISTER_FST_OPERATION(InitArcIteratorClass, Arc,
-                           InitArcIteratorClassArgs);
-    REGISTER_FST_OPERATION(InitEncodeMapperClass, Arc,
-                           InitEncodeMapperClassArgs);
+    REGISTER_FST_OPERATION(InitArcIteratorClass, Arc, InitArcIteratorClassArgs);
     REGISTER_FST_OPERATION(InitMutableArcIteratorClass, Arc,
                            InitMutableArcIteratorClassArgs);
     REGISTER_FST_OPERATION(InitStateIteratorClass, Arc,
                            InitStateIteratorClassArgs);
+    REGISTER_FST_OPERATION(Info, Arc, InfoArgs);
+    REGISTER_FST_OPERATION(Intersect, Arc, IntersectArgs);
+    REGISTER_FST_OPERATION(Invert, Arc, MutableFstClass);
+    REGISTER_FST_OPERATION(Isomorphic, Arc, IsomorphicArgs);
   }
 
   void RegisterBatch2() {
-    REGISTER_FST_OPERATION(Intersect, Arc, IntersectArgs);
-    REGISTER_FST_OPERATION(Invert, Arc, MutableFstClass);
     REGISTER_FST_OPERATION(Map, Arc, MapArgs);
     REGISTER_FST_OPERATION(Minimize, Arc, MinimizeArgs);
-    REGISTER_FST_OPERATION(PrintFst, Arc, FstPrinterArgs);
+    REGISTER_FST_OPERATION(Print, Arc, PrintArgs);
     REGISTER_FST_OPERATION(Project, Arc, ProjectArgs);
     REGISTER_FST_OPERATION(Prune, Arc, PruneArgs1);
     REGISTER_FST_OPERATION(Prune, Arc, PruneArgs2);
@@ -138,10 +148,12 @@ class AllFstOperationsRegisterer {
     REGISTER_FST_OPERATION(RmEpsilon, Arc, RmEpsilonArgs);
     REGISTER_FST_OPERATION(ShortestDistance, Arc, ShortestDistanceArgs1);
     REGISTER_FST_OPERATION(ShortestDistance, Arc, ShortestDistanceArgs2);
+    REGISTER_FST_OPERATION(ShortestDistance, Arc, ShortestDistanceArgs3);
     REGISTER_FST_OPERATION(ShortestPath, Arc, ShortestPathArgs);
     REGISTER_FST_OPERATION(Synchronize, Arc, SynchronizeArgs);
     REGISTER_FST_OPERATION(TopSort, Arc, TopSortArgs);
-    REGISTER_FST_OPERATION(Union, Arc, UnionArgs);
+    REGISTER_FST_OPERATION(Union, Arc, UnionArgs1);
+    REGISTER_FST_OPERATION(Union, Arc, UnionArgs2);
     REGISTER_FST_OPERATION(Verify, Arc, VerifyArgs);
   }
 };
