@@ -1,3 +1,17 @@
+// Copyright 2005-2020 Google LLC
+//
+// Licensed under the Apache License, Version 2.0 (the 'License');
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+//
+//     http://www.apache.org/licenses/LICENSE-2.0
+//
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an 'AS IS' BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
+//
 // See www.openfst.org for extensive documentation on this weighted
 // finite-state transducer library.
 //
@@ -14,8 +28,6 @@
 #include <fst/fstlib.h>
 #include <fst/util.h>
 
-DECLARE_string(fst_field_separator);
-
 namespace fst {
 
 // Print a binary FST in textual format (helper class for fstprint.cc).
@@ -28,23 +40,24 @@ class FstPrinter {
   using Label = typename Arc::Label;
   using Weight = typename Arc::Weight;
 
-  FstPrinter(const Fst<Arc> &fst, const SymbolTable *isyms,
-             const SymbolTable *osyms, const SymbolTable *ssyms, bool accep,
-             bool show_weight_one, const string &field_separator,
-             const string &missing_symbol = "")
+  explicit FstPrinter(const Fst<Arc> &fst, const SymbolTable *isyms,
+                      const SymbolTable *osyms, const SymbolTable *ssyms,
+                      bool accept, bool show_weight_one,
+                      const std::string &field_separator,
+                      const std::string &missing_symbol = "")
       : fst_(fst),
         isyms_(isyms),
         osyms_(osyms),
         ssyms_(ssyms),
-        accep_(accep && fst.Properties(kAcceptor, true)),
+        accept_(accept && (fst.Properties(kAcceptor, true) == kAcceptor)),
         ostrm_(nullptr),
         show_weight_one_(show_weight_one),
         sep_(field_separator),
         missing_symbol_(missing_symbol) {}
 
   // Prints FST to an output stream.
-  void Print(std::ostream *ostrm, const string &dest) {
-    ostrm_ = ostrm;
+  void Print(std::ostream &ostrm, const std::string &dest) {
+    ostrm_ = &ostrm;
     dest_ = dest;
     const auto start = fst_.Start();
     if (start == kNoStateId) return;
@@ -59,7 +72,7 @@ class FstPrinter {
  private:
   void PrintId(StateId id, const SymbolTable *syms, const char *name) const {
     if (syms) {
-      string symbol = syms->Find(id);
+      std::string symbol = syms->Find(id);
       if (symbol.empty()) {
         if (missing_symbol_.empty()) {
           FSTERROR() << "FstPrinter: Integer " << id
@@ -92,7 +105,7 @@ class FstPrinter {
       PrintStateId(arc.nextstate);
       *ostrm_ << sep_;
       PrintILabel(arc.ilabel);
-      if (!accep_) {
+      if (!accept_) {
         *ostrm_ << sep_;
         PrintOLabel(arc.olabel);
       }
@@ -112,17 +125,17 @@ class FstPrinter {
   }
 
   const Fst<Arc> &fst_;
-  const SymbolTable *isyms_;  // ilabel symbol table.
-  const SymbolTable *osyms_;  // olabel symbol table.
-  const SymbolTable *ssyms_;  // slabel symbol table.
-  bool accep_;                // Print as acceptor when possible?
-  std::ostream *ostrm_;       // Text FST destination.
-  string dest_;               // Text FST destination name.
-  bool show_weight_one_;      // Print weights equal to Weight::One()?
-  string sep_;                // Separator character between fields.
-  string missing_symbol_;     // Symbol to print when lookup fails (default
-                              // "" means raise error).
-                              //
+  const SymbolTable *isyms_;    // ilabel symbol table.
+  const SymbolTable *osyms_;    // olabel symbol table.
+  const SymbolTable *ssyms_;    // slabel symbol table.
+  bool accept_;                 // Print as acceptor when possible?
+  std::ostream *ostrm_;         // Text FST destination.
+  std::string dest_;            // Text FST destination name.
+  bool show_weight_one_;        // Print weights equal to Weight::One()?
+  std::string sep_;             // Separator character between fields.
+  std::string missing_symbol_;  // Symbol to print when lookup fails (default
+                                // "" means raise error).
+
   FstPrinter(const FstPrinter &) = delete;
   FstPrinter &operator=(const FstPrinter &) = delete;
 };
