@@ -1,3 +1,17 @@
+// Copyright 2005-2020 Google LLC
+//
+// Licensed under the Apache License, Version 2.0 (the 'License');
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+//
+//     http://www.apache.org/licenses/LICENSE-2.0
+//
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an 'AS IS' BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
+//
 // See www.openfst.org for extensive documentation on this weighted
 // finite-state transducer library.
 
@@ -6,6 +20,7 @@
 
 #include <tuple>
 
+#include <fst/types.h>
 #include <fst/push.h>
 #include <fst/script/fst-class.h>
 
@@ -20,12 +35,12 @@ void Push(PushArgs1 *args) {
   Push(fst, std::get<1>(*args), std::get<2>(*args), std::get<3>(*args));
 }
 
-using PushArgs2 = std::tuple<const FstClass &, MutableFstClass *, uint32,
-                             ReweightType, float>;
+using PushArgs2 =
+    std::tuple<const FstClass &, MutableFstClass *, uint8, ReweightType, float>;
 
 template <class Arc>
 void Push(PushArgs2 *args) {
-  const Fst<Arc> &ifst = *(std::get<0>(*args).GetFst<Arc>());
+  const Fst<Arc> &ifst = *std::get<0>(*args).GetFst<Arc>();
   MutableFst<Arc> *ofst = std::get<1>(*args)->GetMutableFst<Arc>();
   switch (std::get<3>(*args)) {
     case REWEIGHT_TO_FINAL: {
@@ -41,11 +56,11 @@ void Push(PushArgs2 *args) {
   }
 }
 
-void Push(MutableFstClass *fst, ReweightType rew_type, float delta = kDelta,
-          bool remove_total_weight = false);
+void Push(MutableFstClass *fst, ReweightType type = REWEIGHT_TO_INITIAL,
+          float delta = kShortestDelta, bool remove_total_weight = false);
 
-void Push(const FstClass &ifst, MutableFstClass *ofst, uint32 flags,
-          ReweightType rew_type, float delta = kDelta);
+void Push(const FstClass &ifst, MutableFstClass *ofst, uint8 flags,
+          ReweightType rew_type, float delta = kShortestDelta);
 
 }  // namespace script
 }  // namespace fst
