@@ -1,13 +1,27 @@
+// Copyright 2005-2020 Google LLC
+//
+// Licensed under the Apache License, Version 2.0 (the 'License');
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+//
+//     http://www.apache.org/licenses/LICENSE-2.0
+//
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an 'AS IS' BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
+//
 // See www.openfst.org for extensive documentation on this weighted
 // finite-state transducer library.
 
 #ifndef FST_SCRIPT_RANDGEN_H_
 #define FST_SCRIPT_RANDGEN_H_
 
-#include <ctime>
-
+#include <random>
 #include <tuple>
 
+#include <fst/types.h>
 #include <fst/randgen.h>
 #include <fst/script/fst-class.h>
 #include <fst/script/script-impl.h>
@@ -15,17 +29,18 @@
 namespace fst {
 namespace script {
 
-using RandGenArgs = std::tuple<const FstClass &, MutableFstClass *, time_t,
-                               const RandGenOptions<RandArcSelection> &>;
+using RandGenArgs =
+    std::tuple<const FstClass &, MutableFstClass *,
+               const RandGenOptions<RandArcSelection> &, uint64>;
 
 template <class Arc>
 void RandGen(RandGenArgs *args) {
-  const Fst<Arc> &ifst = *(std::get<0>(*args).GetFst<Arc>());
+  const Fst<Arc> &ifst = *std::get<0>(*args).GetFst<Arc>();
   MutableFst<Arc> *ofst = std::get<1>(*args)->GetMutableFst<Arc>();
-  const time_t seed = std::get<2>(*args);
-  const auto &opts = std::get<3>(*args);
+  const auto &opts = std::get<2>(*args);
+  const uint64 seed = std::get<3>(*args);
   switch (opts.selector) {
-    case UNIFORM_ARC_SELECTOR: {
+    case RandArcSelection::UNIFORM: {
       const UniformArcSelector<Arc> selector(seed);
       const RandGenOptions<UniformArcSelector<Arc>> ropts(
           selector, opts.max_length, opts.npath, opts.weighted,
@@ -33,7 +48,7 @@ void RandGen(RandGenArgs *args) {
       RandGen(ifst, ofst, ropts);
       return;
     }
-    case FAST_LOG_PROB_ARC_SELECTOR: {
+    case RandArcSelection::FAST_LOG_PROB: {
       const FastLogProbArcSelector<Arc> selector(seed);
       const RandGenOptions<FastLogProbArcSelector<Arc>> ropts(
           selector, opts.max_length, opts.npath, opts.weighted,
@@ -41,7 +56,7 @@ void RandGen(RandGenArgs *args) {
       RandGen(ifst, ofst, ropts);
       return;
     }
-    case LOG_PROB_ARC_SELECTOR: {
+    case RandArcSelection::LOG_PROB: {
       const LogProbArcSelector<Arc> selector(seed);
       const RandGenOptions<LogProbArcSelector<Arc>> ropts(
           selector, opts.max_length, opts.npath, opts.weighted,
@@ -53,9 +68,9 @@ void RandGen(RandGenArgs *args) {
 }
 
 void RandGen(const FstClass &ifst, MutableFstClass *ofst,
-             time_t seed = time(nullptr),
              const RandGenOptions<RandArcSelection> &opts =
-                 RandGenOptions<RandArcSelection>(UNIFORM_ARC_SELECTOR));
+                 RandGenOptions<RandArcSelection>(RandArcSelection::UNIFORM),
+             uint64 seed = std::random_device()());
 
 }  // namespace script
 }  // namespace fst
