@@ -1,3 +1,17 @@
+// Copyright 2005-2020 Google LLC
+//
+// Licensed under the Apache License, Version 2.0 (the 'License');
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+//
+//     http://www.apache.org/licenses/LICENSE-2.0
+//
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an 'AS IS' BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
+//
 // See www.openfst.org for extensive documentation on this weighted
 // finite-state transducer library.
 
@@ -7,6 +21,7 @@
 #include <utility>
 #include <vector>
 
+#include <fst/types.h>
 #include <fst/queue.h>
 #include <fst/rmepsilon.h>
 #include <fst/script/fst-class.h>
@@ -24,7 +39,7 @@ struct RmEpsilonOptions : public ShortestDistanceOptions {
   RmEpsilonOptions(QueueType queue_type, bool connect,
                    const WeightClass &weight_threshold,
                    int64 state_threshold = kNoStateId, float delta = kDelta)
-      : ShortestDistanceOptions(queue_type, EPSILON_ARC_FILTER, kNoStateId,
+      : ShortestDistanceOptions(queue_type, ArcFilterType::EPSILON, kNoStateId,
                                 delta),
         connect(connect),
         weight_threshold(weight_threshold),
@@ -68,8 +83,14 @@ void RmEpsilon(MutableFst<Arc> *fst, const RmEpsilonOptions &opts) {
       return;
     }
     case SHORTEST_FIRST_QUEUE: {
-      NaturalShortestFirstQueue<StateId, Weight> queue(distance);
-      RmEpsilon(fst, &distance, opts, &queue);
+      if constexpr (IsIdempotent<Weight>::value) {
+        NaturalShortestFirstQueue<StateId, Weight> queue(distance);
+        RmEpsilon(fst, &distance, opts, &queue);
+      } else {
+        FSTERROR() << "RmEpsilon: Bad queue type SHORTEST_FIRST_QUEUE for"
+                   << " non-idempotent Weight " << Weight::Type();
+        fst->SetProperties(kError, kError);
+      }
       return;
     }
     case STATE_ORDER_QUEUE: {
