@@ -1,3 +1,17 @@
+// Copyright 2005-2020 Google LLC
+//
+// Licensed under the Apache License, Version 2.0 (the 'License');
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+//
+//     http://www.apache.org/licenses/LICENSE-2.0
+//
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an 'AS IS' BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
+//
 // See www.openfst.org for extensive documentation on this weighted
 // finite-state transducer library.
 //
@@ -18,7 +32,6 @@
 #include <list>
 #include <stack>
 #include <string>
-#include <unordered_map>
 #include <utility>
 
 
@@ -47,6 +60,8 @@ class SparseTupleWeight {
 
   SparseTupleWeight() { Init(); }
 
+  ~SparseTupleWeight() noexcept = default;
+
   template <class Iterator>
   SparseTupleWeight(Iterator begin, Iterator end) {
     Init();
@@ -70,10 +85,11 @@ class SparseTupleWeight {
     }
   }
 
-  SparseTupleWeight(SparseTupleWeight &&weight)
-    // Don't move the default, so weight.default_ is still valid.
-    : default_(weight.default_), first_(std::move(weight.first_)),
-      rest_(std::move(weight.rest_)) {
+  SparseTupleWeight(SparseTupleWeight &&weight) noexcept
+      // Don't move the default, so weight.default_ is still valid.
+      : default_(weight.default_),  // NOLINT
+        first_(std::move(weight.first_)),
+        rest_(std::move(weight.rest_)) {
     // move leaves the source in a valid but unspecified state.
     // Make sure the source weight is empty.
     weight.first_ = Pair(kNoKey, W::NoWeight());
@@ -116,11 +132,16 @@ class SparseTupleWeight {
     return *this;
   }
 
-  SparseTupleWeight &operator=(SparseTupleWeight &&weight) {
+  SparseTupleWeight &operator=(SparseTupleWeight &&weight) noexcept {
     if (this == &weight) return *this;  // Checks for identity.
+    // Don't move the default, so weight.default_ is still valid.
     default_ = weight.default_;
-    std::swap(first_, weight.first_);
-    std::swap(rest_, weight.rest_);
+    first_ = std::move(weight.first_);
+    rest_ = std::move(weight.rest_);
+    // move leaves the source in a valid but unspecified state.
+    // Make sure the source weight is empty.
+    weight.first_ = Pair(kNoKey, W::NoWeight());
+    weight.rest_.clear();
     return *this;
   }
 
@@ -152,11 +173,11 @@ class SparseTupleWeight {
   }
 
   ReverseWeight Reverse() const {
-    SparseTupleWeight weight;
+    ReverseWeight weight(DefaultValue().Reverse());
     for (Iterator it(*this); !it.Done(); it.Next()) {
       weight.PushBack(it.Value().first, it.Value().second.Reverse());
     }
-    return ReverseWeight(weight);
+    return weight;
   }
 
   void Init(const W &default_value = W::Zero()) {
