@@ -1,3 +1,17 @@
+// Copyright 2005-2020 Google LLC
+//
+// Licensed under the Apache License, Version 2.0 (the 'License');
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+//
+//     http://www.apache.org/licenses/LICENSE-2.0
+//
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an 'AS IS' BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
+//
 // See www.openfst.org for extensive documentation on this weighted
 // finite-state transducer library.
 //
@@ -10,6 +24,7 @@
 #include <utility>
 #include <vector>
 
+#include <fst/types.h>
 #include <fst/log.h>
 
 #include <fst/bi-table.h>
@@ -118,7 +133,7 @@ class VectorStateTable : public VectorBiTable<typename T::StateId, T, FP> {
   using VectorBiTable<StateId, StateTuple, FP>::Size;
   using VectorBiTable<StateId, StateTuple, FP>::Fingerprint;
 
-  explicit VectorStateTable(FP *fingerprint = nullptr, size_t table_size = 0)
+  explicit VectorStateTable(const FP &fingerprint = FP(), size_t table_size = 0)
       : VectorBiTable<StateId, StateTuple, FP>(fingerprint, table_size) {}
 
   StateId FindState(const StateTuple &tuple) { return FindId(tuple); }
@@ -145,8 +160,9 @@ class VectorHashStateTable
   using VectorHashBiTable<StateId, StateTuple, Select, FP, H>::Fingerprint;
   using VectorHashBiTable<StateId, StateTuple, Select, FP, H>::Hash;
 
-  VectorHashStateTable(Select *select, FP *fingerprint, H *hash,
-                       size_t vector_size = 0, size_t tuple_size = 0)
+  VectorHashStateTable(const Select &select, const FP &fingerprint,
+                       const H &hash, size_t vector_size = 0,
+                       size_t tuple_size = 0)
       : VectorHashBiTable<StateId, StateTuple, Select, FP, H>(
             select, fingerprint, hash, vector_size, tuple_size) {}
 
@@ -195,7 +211,7 @@ class ErasableStateTable : public ErasableBiTable<typename T::StateId, T, H> {
 //   // Looks up a tuple by state ID.
 //   const ComposeStateTuple<StateId> &Tuple(StateId s) const;
 //
-//   // The number of of stored tuples.
+//   // The number of stored tuples.
 //   StateId Size() const;
 //
 //   // Return true if error was encountered.
@@ -262,7 +278,7 @@ class DefaultComposeStateTuple {
   FilterState fs_;  // State of composition filter.
 };
 
-// Specialization for TrivialFilterState that does not explicitely store the
+// Specialization for TrivialFilterState that does not explicitly store the
 // filter state since it is always the unique non-blocking state.
 template <typename S>
 class DefaultComposeStateTuple<S, TrivialFilterState> {
@@ -270,8 +286,7 @@ class DefaultComposeStateTuple<S, TrivialFilterState> {
   using StateId = S;
   using FilterState = TrivialFilterState;
 
-  DefaultComposeStateTuple()
-      : state_pair_(kNoStateId, kNoStateId) {}
+  DefaultComposeStateTuple() : state_pair_(kNoStateId, kNoStateId) {}
 
   DefaultComposeStateTuple(StateId s1, StateId s2, const FilterState &)
       : state_pair_(s1, s2) {}
@@ -289,7 +304,7 @@ class DefaultComposeStateTuple<S, TrivialFilterState> {
     return (&x == &y) || (x.state_pair_ == y.state_pair_);
   }
 
-  size_t Hash() const { return StateId1() + StateId2() * 7853; }
+  size_t Hash() const { return StateId1() + StateId2() * size_t{7853}; }
 
  private:
   std::pair<StateId, StateId> state_pair_;
@@ -340,7 +355,7 @@ class ComposeFingerprint {
   ComposeFingerprint(StateId nstates1, StateId nstates2)
       : mult1_(nstates1), mult2_(nstates1 * nstates2) {}
 
-  size_t operator()(const StateTuple &tuple) {
+  size_t operator()(const StateTuple &tuple) const {
     return tuple.StateId1() + tuple.StateId2() * mult1_ +
            tuple.GetFilterState().Hash() * mult2_;
   }
@@ -378,13 +393,13 @@ class ProductComposeStateTable
 
   ProductComposeStateTable(const Fst<Arc> &fst1, const Fst<Arc> &fst2,
                            size_t table_size = 0)
-      : StateTable(new ComposeFingerprint<StateTuple>(CountStates(fst1),
-                                                      CountStates(fst2)),
+      : StateTable(ComposeFingerprint<StateTuple>(CountStates(fst1),
+                                                  CountStates(fst2)),
                    table_size) {}
 
   ProductComposeStateTable(
       const ProductComposeStateTable<Arc, StateTuple> &table)
-      : StateTable(new ComposeFingerprint<StateTuple>(table.Fingerprint())) {}
+      : StateTable(ComposeFingerprint<StateTuple>(table.Fingerprint())) {}
 
   constexpr bool Error() const { return false; }
 
