@@ -1,3 +1,17 @@
+// Copyright 2005-2020 Google LLC
+//
+// Licensed under the Apache License, Version 2.0 (the 'License');
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+//
+//     http://www.apache.org/licenses/LICENSE-2.0
+//
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an 'AS IS' BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
+//
 // See www.openfst.org for extensive documentation on this weighted
 // finite-state transducer library.
 //
@@ -6,24 +20,26 @@
 #ifndef FST_STRING_WEIGHT_H_
 #define FST_STRING_WEIGHT_H_
 
-#include <cstdlib>
-
 #include <list>
+#include <random>
 #include <string>
 #include <vector>
 
+#include <fst/types.h>
+
 #include <fst/product-weight.h>
 #include <fst/union-weight.h>
 #include <fst/weight.h>
+#include <string_view>
 
 
 namespace fst {
 
-constexpr int kStringInfinity = -1;     // Label for the infinite string.
-constexpr int kStringBad = -2;          // Label for a non-string.
-constexpr char kStringSeparator = '_';  // Label separator in strings.
+constexpr int kStringInfinity = -1;       // Label for the infinite string.
+constexpr int kStringBad = -2;            // Label for a non-string.
+constexpr char kStringSeparator[] = "_";  // Label separator in strings.
 
-// Determines whether to use left or right string semiring.  Includes a
+// Determines whether to use left or right string semiring. Includes a
 // 'restricted' version that signals an error if proper prefixes/suffixes
 // would otherwise be returned by Plus, useful with various
 // algorithms that require functional transducer input with the
@@ -41,10 +57,10 @@ template <class>
 class StringWeightReverseIterator;
 
 // String semiring: (longest_common_prefix/suffix, ., Infinity, Epsilon)
-template <typename Label_, StringType S = STRING_LEFT>
+template <typename L, StringType S = STRING_LEFT>
 class StringWeight {
  public:
-  using Label = Label_;
+  using Label = L;
   using ReverseWeight = StringWeight<Label, ReverseStringType(S)>;
   using Iterator = StringWeightIterator<StringWeight>;
   using ReverseIterator = StringWeightReverseIterator<StringWeight>;
@@ -55,7 +71,7 @@ class StringWeight {
   StringWeight() {}
 
   template <typename Iterator>
-  StringWeight(const Iterator &begin, const Iterator &end) {
+  StringWeight(const Iterator begin, const Iterator end) {
     for (auto iter = begin; iter != end; ++iter) PushBack(*iter);
   }
 
@@ -76,8 +92,8 @@ class StringWeight {
     return *no_weight;
   }
 
-  static const string &Type() {
-    static const string *const type = new string(
+  static const std::string &Type() {
+    static const std::string *const type = new std::string(
         S == STRING_LEFT
             ? "left_string"
             : (S == STRING_RIGHT ? "right_string" : "restricted_string"));
@@ -311,7 +327,7 @@ inline std::ostream &operator<<(std::ostream &strm,
 template <typename Label, StringType S>
 inline std::istream &operator>>(std::istream &strm,
                                 StringWeight<Label, S> &weight) {
-  string str;
+  std::string str;
   strm >> str;
   using Weight = StringWeight<Label, S>;
   if (str == "Infinity") {
@@ -320,14 +336,13 @@ inline std::istream &operator>>(std::istream &strm,
     weight = Weight::One();
   } else {
     weight.Clear();
-    char *p = nullptr;
-    for (const char *cs = str.c_str(); !p || *p != '\0'; cs = p + 1) {
-      const Label label = strtoll(cs, &p, 10);
-      if (p == cs || (*p != 0 && *p != kStringSeparator)) {
+    for (std::string_view sv : SplitString(str, kStringSeparator, false)) {
+      auto maybe_label = ParseInt64(sv);
+      if (!maybe_label.has_value()) {
         strm.clear(std::ios::badbit);
         break;
       }
-      weight.PushBack(label);
+      weight.PushBack(*maybe_label);
     }
   }
   return strm;
@@ -495,30 +510,32 @@ class WeightGenerate<StringWeight<Label, S>> {
  public:
   using Weight = StringWeight<Label, S>;
 
-  explicit WeightGenerate(bool allow_zero = true,
+  explicit WeightGenerate(uint64 seed = std::random_device()(),
+                          bool allow_zero = true,
                           size_t alphabet_size = kNumRandomWeights,
                           size_t max_string_length = kNumRandomWeights)
-      : allow_zero_(allow_zero),
+      : rand_(seed),
+        allow_zero_(allow_zero),
         alphabet_size_(alphabet_size),
         max_string_length_(max_string_length) {}
 
   Weight operator()() const {
-    size_t n = rand() % (max_string_length_ + allow_zero_);  // NOLINT
+    const int n = std::uniform_int_distribution<>(
+        0, max_string_length_ + allow_zero_)(rand_);
     if (allow_zero_ && n == max_string_length_) return Weight::Zero();
     std::vector<Label> labels;
     labels.reserve(n);
-    for (size_t i = 0; i < n; ++i) {
-      labels.push_back(rand() % alphabet_size_ + 1);  // NOLINT
+    for (int i = 0; i < n; ++i) {
+      labels.push_back(
+          std::uniform_int_distribution<>(1, alphabet_size_)(rand_));
     }
     return Weight(labels.begin(), labels.end());
   }
 
  private:
-  // Permits Zero() and zero divisors.
+  mutable std::mt19937_64 rand_;
   const bool allow_zero_;
-  // Alphabet size for random weights.
   const size_t alphabet_size_;
-  // Number of alternative random weights.
   const size_t max_string_length_;
 };
 
@@ -565,7 +582,7 @@ struct GallicWeight
 
   GallicWeight(SW w1, W w2) : ProductWeight<SW, W>(w1, w2) {}
 
-  explicit GallicWeight(const string &s, int *nread = nullptr)
+  explicit GallicWeight(const std::string &s, int *nread = nullptr)
       : ProductWeight<SW, W>(s, nread) {}
 
   explicit GallicWeight(const ProductWeight<SW, W> &w)
@@ -586,8 +603,8 @@ struct GallicWeight
     return no_weight;
   }
 
-  static const string &Type() {
-    static const string *const type = new string(
+  static const std::string &Type() {
+    static const std::string *const type = new std::string(
         G == GALLIC_LEFT
             ? "left_gallic"
             : (G == GALLIC_RIGHT
@@ -650,7 +667,9 @@ class WeightGenerate<GallicWeight<Label, W, G>>
   using Generate = WeightGenerate<
       ProductWeight<StringWeight<Label, GallicStringType(G)>, W>>;
 
-  explicit WeightGenerate(bool allow_zero = true) : generate_(allow_zero) {}
+  explicit WeightGenerate(uint64 seed = std::random_device()(),
+                          bool allow_zero = true)
+      : generate_(seed, allow_zero) {}
 
   Weight operator()() const { return Weight(generate_()); }
 
@@ -712,7 +731,8 @@ struct GallicWeight<Label, W, GALLIC>
   GallicWeight() {}
 
   // Copy constructor.
-  GallicWeight(const UW &weight) : UW(weight) {}  // NOLINT
+  // NOLINTNEXTLINE(google-explicit-constructor)
+  GallicWeight(const UW &weight) : UW(weight) {}
 
   // Singleton constructors: create a GALLIC weight containing a single
   // GALLIC_RESTRICT weight. Takes as argument (1) a GALLIC_RESTRICT weight or
@@ -721,7 +741,7 @@ struct GallicWeight<Label, W, GALLIC>
 
   GallicWeight(SW w1, W w2) : UW(GW(w1, w2)) {}
 
-  explicit GallicWeight(const string &str, int *nread = nullptr)
+  explicit GallicWeight(const std::string &str, int *nread = nullptr)
       : UW(str, nread) {}
 
   static const GallicWeight<Label, W, GALLIC> &Zero() {
@@ -739,8 +759,8 @@ struct GallicWeight<Label, W, GALLIC>
     return no_weight;
   }
 
-  static const string &Type() {
-    static const string *const type = new string("gallic");
+  static const std::string &Type() {
+    static const std::string *const type = new std::string("gallic");
     return *type;
   }
 
@@ -794,7 +814,9 @@ class WeightGenerate<GallicWeight<Label, W, GALLIC>>
       WeightGenerate<UnionWeight<GallicWeight<Label, W, GALLIC_RESTRICT>,
                                  GallicUnionWeightOptions<Label, W>>>;
 
-  explicit WeightGenerate(bool allow_zero = true) : generate_(allow_zero) {}
+  explicit WeightGenerate(uint64 seed = std::random_device()(),
+                          bool allow_zero = true)
+      : generate_(seed, allow_zero) {}
 
   Weight operator()() const { return Weight(generate_()); }
 
