@@ -1,3 +1,17 @@
+// Copyright 2005-2020 Google LLC
+//
+// Licensed under the Apache License, Version 2.0 (the 'License');
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+//
+//     http://www.apache.org/licenses/LICENSE-2.0
+//
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an 'AS IS' BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
+//
 // See www.openfst.org for extensive documentation on this weighted
 // finite-state transducer library.
 //
@@ -7,15 +21,19 @@
 #define FST_SYNCHRONIZE_H_
 
 #include <algorithm>
+#include <functional>
+#include <memory>
 #include <string>
-#include <unordered_map>
-#include <unordered_set>
-#include <utility>
+#include <string_view>
 #include <vector>
 
+#include <fst/types.h>
+
 #include <fst/cache.h>
 #include <fst/test-properties.h>
 
+#include <unordered_map>
+#include <unordered_set>
 
 namespace fst {
 
@@ -38,7 +56,7 @@ class SynchronizeFstImpl : public CacheImpl<Arc> {
   using FstImpl<Arc>::SetInputSymbols;
   using FstImpl<Arc>::SetOutputSymbols;
 
-  using CacheBaseImpl<CacheState<Arc>>::PushArc;
+  using CacheBaseImpl<CacheState<Arc>>::EmplaceArc;
   using CacheBaseImpl<CacheState<Arc>>::HasArcs;
   using CacheBaseImpl<CacheState<Arc>>::HasFinal;
   using CacheBaseImpl<CacheState<Arc>>::HasStart;
@@ -46,20 +64,20 @@ class SynchronizeFstImpl : public CacheImpl<Arc> {
   using CacheBaseImpl<CacheState<Arc>>::SetFinal;
   using CacheBaseImpl<CacheState<Arc>>::SetStart;
 
-  using String = basic_string<Label>;
+  using String = std::basic_string<Label>;
+  using StringView = std::basic_string_view<Label>;
 
   struct Element {
     Element() {}
 
-    Element(StateId state_, const String *i, const String *o)
+    Element(StateId state_, StringView i, StringView o)
         : state(state_), istring(i), ostring(o) {}
 
-    StateId state;          // Input state ID.
-    const String *istring;  // Residual input labels.
-    const String *ostring;  // Residual output labels.
-    // Residual strings are represented by const pointers to
-    // basic_string<Label> and are stored in a hash_set. The pointed
-    // memory is owned by the hash_set string_set_.
+    StateId state;       // Input state ID.
+    StringView istring;  // Residual input labels.
+    StringView ostring;  // Residual output labels.
+    // Residual strings are represented by std::basic_string_view<Label> whose
+    // values are owned by the hash set string_set_.
   };
 
   SynchronizeFstImpl(const Fst<Arc> &fst, const SynchronizeFstOptions &opts)
@@ -79,15 +97,11 @@ class SynchronizeFstImpl : public CacheImpl<Arc> {
     SetOutputSymbols(impl.OutputSymbols());
   }
 
-  ~SynchronizeFstImpl() override {
-    for (const auto *ptr : string_set_) delete ptr;
-  }
-
   StateId Start() {
     if (!HasStart()) {
       auto start = fst_->Start();
       if (start == kNoStateId) return kNoStateId;
-      const auto *empty = FindString(new String());
+      const StringView empty = FindString(String());
       start = FindState(Element(fst_->Start(), empty, empty));
       SetStart(start);
     }
@@ -100,8 +114,8 @@ class SynchronizeFstImpl : public CacheImpl<Arc> {
       const auto weight = element.state == kNoStateId
                               ? Weight::One()
                               : fst_->Final(element.state);
-      if ((weight != Weight::Zero()) && (element.istring)->empty() &&
-          (element.ostring)->empty()) {
+      if ((weight != Weight::Zero()) && element.istring.empty() &&
+          element.ostring.empty()) {
         SetFinal(s, weight);
       } else {
         SetFinal(s, Weight::Zero());
@@ -142,9 +156,9 @@ class SynchronizeFstImpl : public CacheImpl<Arc> {
 
   // Returns the first character of the string obtained by concatenating the
   // string and the label.
-  Label Car(const String *str, Label label = 0) const {
-    if (!str->empty()) {
-      return (*str)[0];
+  Label Car(StringView str, Label label = 0) const {
+    if (!str.empty()) {
+      return str[0];
     } else {
       return label;
     }
@@ -152,45 +166,36 @@ class SynchronizeFstImpl : public CacheImpl<Arc> {
 
   // Computes the residual string obtained by removing the first
   // character in the concatenation of the string and the label.
-  const String *Cdr(const String *str, Label label = 0) {
-    auto *r = new String();
-    for (size_t i = 1; i < str->size(); ++i) r->push_back((*str)[i]);
-    if (label && !(str->empty())) r->push_back(label);
-    return FindString(r);
+  StringView Cdr(StringView str, Label label = 0) {
+    if (str.empty()) return FindString(String());
+    return Concat(str.substr(1), label);
   }
 
   // Computes the concatenation of the string and the label.
-  const String *Concat(const String *str, Label label = 0) {
-    auto *r = new String();
-    for (size_t i = 0; i < str->size(); ++i) r->push_back((*str)[i]);
-    if (label) r->push_back(label);
-    return FindString(r);
+  StringView Concat(StringView str, Label label = 0) {
+    String r(str);
+    if (label) r.push_back(label);
+    return FindString(std::move(r));
   }
 
   // Tests if the concatenation of the string and label is empty.
-  bool Empty(const String *str, Label label = 0) const {
-    if (str->empty()) {
+  bool Empty(StringView str, Label label = 0) const {
+    if (str.empty()) {
       return label == 0;
     } else {
       return false;
     }
   }
 
-  // Finds the string pointed by s in the hash set. Transfers the pointer
-  // ownership to the hash set.
-  const String *FindString(const String *str) {
-    const auto insert_result = string_set_.insert(str);
-    if (!insert_result.second) {
-      delete str;
-    }
+  StringView FindString(String &&str) {
+    const auto insert_result = string_set_.insert(std::forward<String>(str));
     return *insert_result.first;
   }
 
   // Finds state corresponding to an element. Creates new state if element
   // is not found.
   StateId FindState(const Element &element) {
-    const auto insert_result =
-        element_map_.insert(std::make_pair(element, elements_.size()));
+    const auto insert_result = element_map_.emplace(element, elements_.size());
     if (insert_result.second) {
       elements_.push_back(element);
     }
@@ -207,16 +212,16 @@ class SynchronizeFstImpl : public CacheImpl<Arc> {
         const auto &arc = aiter.Value();
         if (!Empty(element.istring, arc.ilabel) &&
             !Empty(element.ostring, arc.olabel)) {
-          const auto *istring = Cdr(element.istring, arc.ilabel);
-          const auto *ostring = Cdr(element.ostring, arc.olabel);
-          PushArc(s, Arc(Car(element.istring, arc.ilabel),
-                         Car(element.ostring, arc.olabel), arc.weight,
-                         FindState(Element(arc.nextstate, istring, ostring))));
+          StringView istring = Cdr(element.istring, arc.ilabel);
+          StringView ostring = Cdr(element.ostring, arc.olabel);
+          EmplaceArc(s, Car(element.istring, arc.ilabel),
+                     Car(element.ostring, arc.olabel), arc.weight,
+                     FindState(Element(arc.nextstate, istring, ostring)));
         } else {
-          const auto *istring = Concat(element.istring, arc.ilabel);
-          const auto *ostring = Concat(element.ostring, arc.olabel);
-          PushArc(s, Arc(0, 0, arc.weight,
-                         FindState(Element(arc.nextstate, istring, ostring))));
+          StringView istring = Concat(element.istring, arc.ilabel);
+          StringView ostring = Concat(element.ostring, arc.olabel);
+          EmplaceArc(s, 0, 0, arc.weight,
+                     FindState(Element(arc.nextstate, istring, ostring)));
         }
       }
     }
@@ -224,11 +229,11 @@ class SynchronizeFstImpl : public CacheImpl<Arc> {
                             ? Weight::One()
                             : fst_->Final(element.state);
     if ((weight != Weight::Zero()) &&
-        ((element.istring)->size() + (element.ostring)->size() > 0)) {
-      const auto *istring = Cdr(element.istring);
-      const auto *ostring = Cdr(element.ostring);
-      PushArc(s, Arc(Car(element.istring), Car(element.ostring), weight,
-                     FindState(Element(kNoStateId, istring, ostring))));
+        (element.istring.size() + element.ostring.size() > 0)) {
+      StringView istring = Cdr(element.istring);
+      StringView ostring = Cdr(element.ostring);
+      EmplaceArc(s, Car(element.istring), Car(element.ostring), weight,
+                 FindState(Element(kNoStateId, istring, ostring)));
     }
     SetArcs(s);
   }
@@ -238,8 +243,8 @@ class SynchronizeFstImpl : public CacheImpl<Arc> {
   class ElementEqual {
    public:
     bool operator()(const Element &x, const Element &y) const {
-      return x.state == y.state && x.istring == y.istring &&
-             x.ostring == y.ostring;
+      return x.state == y.state && x.istring.data() == y.istring.data() &&
+             x.ostring.data() == y.ostring.data();
     }
   };
 
@@ -248,43 +253,34 @@ class SynchronizeFstImpl : public CacheImpl<Arc> {
    public:
     size_t operator()(const Element &x) const {
       size_t key = x.state;
-      key = (key << 1) ^ (x.istring)->size();
-      for (size_t i = 0; i < (x.istring)->size(); ++i) {
-        key = (key << 1) ^ (*x.istring)[i];
+      key = (key << 1) ^ x.istring.size();
+      for (Label label : x.istring) {
+        key = (key << 1) ^ label;
       }
-      key = (key << 1) ^ (x.ostring)->size();
-      for (size_t i = 0; i < (x.ostring)->size(); ++i) {
-        key = (key << 1) ^ (*x.ostring)[i];
+      key = (key << 1) ^ x.ostring.size();
+      for (Label label : x.ostring) {
+        key = (key << 1) ^ label;
       }
       return key;
     }
   };
 
-  // Equality function for strings.
-  class StringEqual {
-   public:
-    bool operator()(const String *const &x, const String *const &y) const {
-      if (x->size() != y->size()) return false;
-      for (size_t i = 0; i < x->size(); ++i) {
-        if ((*x)[i] != (*y)[i]) return false;
-      }
-      return true;
-    }
-  };
-
-  // Hash function for set of strings
+  // Hash function for set of strings. This only has to be specified since
+  // `std::hash<std::basic_string<T>>` is only guaranteed to be defined for
+  // certain values of `T`. Not defining this works fine on clang, but fails
+  // under GCC.
   class StringKey {
    public:
-    size_t operator()(const String *const &x) const {
-      size_t key = x->size();
-      for (size_t i = 0; i < x->size(); ++i) key = (key << 1) ^ (*x)[i];
+    size_t operator()(StringView x) const {
+      size_t key = x.size();
+      for (Label label : x) key = (key << 1) ^ label;
       return key;
     }
   };
 
   using ElementMap =
       std::unordered_map<Element, StateId, ElementKey, ElementEqual>;
-  using StringSet = std::unordered_set<const String *, StringKey, StringEqual>;
+  using StringSet = std::unordered_set<String, StringKey>;
 
   std::unique_ptr<const Fst<Arc>> fst_;
   std::vector<Element> elements_;  // Maps FST state to Elements.
@@ -329,18 +325,17 @@ class SynchronizeFst : public ImplToFst<internal::SynchronizeFstImpl<A>> {
   friend class ArcIterator<SynchronizeFst<A>>;
   friend class StateIterator<SynchronizeFst<A>>;
 
-  explicit SynchronizeFst(
-      const Fst<A> &fst,
-      const SynchronizeFstOptions &opts = SynchronizeFstOptions())
+  explicit SynchronizeFst(const Fst<A> &fst, const SynchronizeFstOptions &opts =
+                                                 SynchronizeFstOptions())
       : ImplToFst<Impl>(std::make_shared<Impl>(fst, opts)) {}
 
   // See Fst<>::Copy() for doc.
-  SynchronizeFst(const SynchronizeFst<Arc> &fst, bool safe = false)
+  SynchronizeFst(const SynchronizeFst &fst, bool safe = false)
       : ImplToFst<Impl>(fst, safe) {}
 
   // Gets a copy of this SynchronizeFst. See Fst<>::Copy() for further doc.
-  SynchronizeFst<Arc> *Copy(bool safe = false) const override {
-    return new SynchronizeFst<Arc>(*this, safe);
+  SynchronizeFst *Copy(bool safe = false) const override {
+    return new SynchronizeFst(*this, safe);
   }
 
   inline void InitStateIterator(StateIteratorData<Arc> *data) const override;
@@ -381,7 +376,7 @@ class ArcIterator<SynchronizeFst<Arc>>
 template <class Arc>
 inline void SynchronizeFst<Arc>::InitStateIterator(
     StateIteratorData<Arc> *data) const {
-  data->base = new StateIterator<SynchronizeFst<Arc>>(*this);
+  data->base = std::make_unique<StateIterator<SynchronizeFst<Arc>>>(*this);
 }
 
 // Synchronizes a transducer. This version writes the synchronized result to a
@@ -405,7 +400,8 @@ inline void SynchronizeFst<Arc>::InitStateIterator(
 template <class Arc>
 void Synchronize(const Fst<Arc> &ifst, MutableFst<Arc> *ofst) {
   // Caches only the last state for fastest copy.
-  const SynchronizeFstOptions opts(FLAGS_fst_default_cache_gc, 0);
+  const SynchronizeFstOptions opts(FST_FLAGS_fst_default_cache_gc,
+                                   0);
   *ofst = SynchronizeFst<Arc>(ifst, opts);
 }
 
