@@ -1,3 +1,17 @@
+// Copyright 2005-2020 Google LLC
+//
+// Licensed under the Apache License, Version 2.0 (the 'License');
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+//
+//     http://www.apache.org/licenses/LICENSE-2.0
+//
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an 'AS IS' BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
+//
 // See www.openfst.org for extensive documentation on this weighted
 // finite-state transducer library.
 //
@@ -13,6 +27,7 @@
 #include <vector>
 
 #include <fst/flags.h>
+#include <fst/types.h>
 #include <fst/log.h>
 
 #include <fst/weight.h>
@@ -29,13 +44,6 @@ class TupleWeight {
   using Weight = W;
   using Index = size_t;
 
-  TupleWeight(const TupleWeight &other) { values_ = other.values_; }
-
-  TupleWeight<W, n> &operator=(const TupleWeight<W, n> &other) {
-    values_ = other.values_;
-    return *this;
-  }
-
   template <class Iterator>
   TupleWeight(Iterator begin, Iterator end) {
     std::copy(begin, end, values_.begin());
@@ -78,8 +86,7 @@ class TupleWeight {
   }
 
   bool Member() const {
-    return std::all_of(values_.begin(), values_.end(),
-                       std::mem_fn(&W::Member));
+    return std::all_of(values_.begin(), values_.end(), std::mem_fn(&W::Member));
   }
 
   size_t Hash() const {
